# Makefile Include file for all data plane applications
#
# Copyright (C) 2008-2011 Freescale Semiconductor, Inc.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
# NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

CROSS_COMPILE ?= powerpc-linux-gnu-

TMP_DIR=/tmp/$(USER)
MOUNT_POINT=$(TMP_DIR)/rootfs
APP_ELF=$(APP).elf
APP_MAP=$(APP).map

PREFIX ?= /usr/local
SYSROOT ?= $(DESTDIR)$(PREFIX)

CC ?= $(CROSS_COMPILE)gcc
AS ?= $(CROSS_COMPILE)as

LIBC_INC=-I$(SYSROOT)/include/libc

LIBC_CC_OPTS=-D_GCCPORT_ \
  -D__POWERPC__ \
  -D__PPC_EABI__ \
  -D__MWERKS__ \
  -include msl_c_directives_to_gcc.h \
  -fno-builtin \
  -mbig-endian \
  -mcpu=e500mc

O ?= $(DESTDIR)
LIB_DIR = $(SYSROOT)/lib
LWE_INC =  $(SYSROOT)/include/lwe
LIBFDT_INC = $(SYSROOT)/include

APP_LIB_INC := $(APP_LIB:%=-I%)

CC_OPTS += -m32 -Wa,-me500mc -nostdinc -I. \
	$(APP_LIB_INC) -I$(LWE_INC) -I$(LIBFDT_INC) -std=gnu99 $(FLAG)

CC_OPTS_C= -Wall \
  -c \
  -Wstrict-prototypes \
  -fno-strict-aliasing \
  -fno-common \
  -O3 \
  -pipe \
  -ffixed-r2 \
  -mmultiple \
  -mno-altivec \
  -funit-at-a-time \
  -mno-string \
  -fomit-frame-pointer \
  -fno-stack-protector \
  -Wunused \
  -Wno-undef \
  -Wno-unknown-pragmas \
  -Werror

LD ?= $(CROSS_COMPILE)ld

LD_OPTS=-Wl,-EB -Wl,-Bstatic -nostdlib -Wl,-Map -Wl,$(APP_MAP)  -L$(LIB_DIR) -Xlinker -wrap -Xlinker memcpy
MKDIR=mkdir -p

APP_SRCS = $(wildcard *.c)

APP_OBJS = $(basename $(APP_SRCS))
APP_OBJS := $(APP_OBJS:%=%.o)
APP_DEPS := $(APP_SRCS:%.c=%.d)

LIB_OBJS = $(basename $(LIB_SRCS))
LIB_OBJS := $(LIB_OBJS:%=%_$(APP).o)
LIB_DEPS := $(LIB_SRCS:%.c=%.d)

CC_OPTS+=-I$(APP_DIR)/include

all:	$(APP_ELF)

# compile and gen dependency file
%.o: %.c
	$(CC) $(LIBC_INC) $(LIBC_CC_OPTS) $(CC_OPTS) $(CC_OPTS_C) -MD -c -o $@ $<
%_$(APP).o: %.c
	$(CC) $(LIBC_INC) $(LIBC_CC_OPTS) $(CC_OPTS) $(CC_OPTS_C) -MD -c -o $@ $<

$(APP_ELF):	$(APP_OBJS) $(LIB_OBJS)
		$(CC) $(LD_OPTS) -Wl,-T$(APP_LCF) -o $@ $(APP_OBJS) $(LIB_OBJS) -llwe -lfdt -lc-msl -lgcc
		rm -rf $(LIB_OBJS) $(APP_DEPS) $(LIB_DEPS) $(APP_OBJS)
clean:
	rm -rf $(APP_OBJS) $(APP_MAP) $(APP_ELF) $(LIB_OBJS) $(APP_DEPS) $(LIB_DEPS)

-include $(APP_OBJS:.o=.d)
-include $(LIB_OBJS:.o=.d)

.PHONY: clean
