/**
\file       br_stats.c 
\brief      Bridge statistics implementation
*/
/*
 * Copyright (C) 2008-2010 Freescale Semiconductor, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 * NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include <stdlib.h>
#include <init.h>
#include <lwe_malloc.h>
#include <timer_wheel.h>
#include <shell.h>
#include <br.h>

union br_stats_s *br_stats;

/* Stats display shell command callback.*/
int br_stats_cmd_cb(int argc __UNUSED,
		char *argv[] __UNUSED)
{
	printf("\nBridge statistics. System time : %u ms\n",
	       get_current_time());
	printf("========================================================\n");
	PRINT_STAT(BR_STAT_RX_TOTAL);
	PRINT_STAT(BR_STAT_RX_UNICAST);
	PRINT_STAT(BR_STAT_RX_MULTICAST);
	PRINT_STAT(BR_STAT_RX_BROADCAST);
	PRINT_STAT(BR_STAT_RX_ERROR);
	PRINT_STAT(BR_STAT_RX_UNSUPPORTED);
	PRINT_STAT(BR_STAT_RX_LKUPFAIL);
	PRINT_STAT(BR_STAT_RX_LOCAL);
	PRINT_STAT(BR_STAT_TX_UNICAST);
	PRINT_STAT(BR_STAT_TX_MULTICAST);
	PRINT_STAT(BR_STAT_TX_BROADCAST);
	PRINT_STAT(BR_STAT_TX_ERROR);
	printf("========================================================\r\n");

	return 0;
}


/**
\brief          Stats initialisation

\details        Allocates region of memory for storing statistics

\return         0 - On Success
		-1 - On Failure

\note		Called only once
*/
int br_stats_init(void)
{
	/* Clear all the statistics */
	br_stats = stats_memalign(16, sizeof(union br_stats_s));
	if (unlikely(NULL == br_stats)) {
		LWE_ERROR("stats_memalign failed for bridging stats");
		return -1;
	}

	memset(br_stats, 0, sizeof(union br_stats_s));

	/*Add shell command to display statistics*/
	if (0 != add_command("br_stats", br_stats_cmd_cb,
			"Print bridging statistics")) {
		APP_ERROR("add_command failed to add stats display command");
		return -1;
	}

	return 0;
}

/**
\brief          Stats increment

\details        Increments statistics counter

\return         -1 - failure
		0 - success

\note		Called only once
*/
void
br_stats_update(enum br_stats_type_e stats_type, register uint32_t frm_size)
{
	decorated_store_64_inc_acc_32(&(br_stats->stats_array[stats_type]),
				      frm_size);
}
