/**
 \file ether.h
 \brief Ethernet protocol defines and data-types
 */
/*
 * Copyright (C) 2008-2009 Freescale Semiconductor, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 * NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef _ETHER_H
#define _ETHER_H

/*
 *	IEEE 802.3 Ethernet magic constants.  The frame sizes omit the preamble
 *	and FCS/CRC (frame check sequence). 
 */

#define ETH_ALEN	6	/**< Octets in one ethernet addr   */
#define ETH_PIDLEN	2	/**< Octets in one ethernet pid    */
#define ETH_HLEN	14	/**< Total octets in header.       */
#define ETH_ZLEN	60	/**< Min. octets in frame sans FCS */
#define ETH_DATA_LEN	1500	/**< Max. octets in payload        */
#define ETH_FRAME_LEN	1514	/**< Max. octets in frame sans FCS */
#define ETH_FCS_LEN	4	/**< Octets in the FCS             */

/**
 \brief	These are the defined Ethernet Protocol ID's.
 */

#define ETH_P_LOOP	0x0060	/**< Ethernet Loopback packet     */
#define ETH_P_PUP	0x0200	/**< Xerox PUP packet             */
#define ETH_P_PUPAT	0x0201	/**< Xerox PUP Addr Trans packet  */
#define ETH_P_IP	0x0800	/**< Internet Protocol packet     */
#define ETH_P_X25	0x0805	/**< CCITT X.25                   */
#define ETH_P_ARP	0x0806	/**< Address Resolution packet    */
#define	ETH_P_BPQ	0x08FF	/**< G8BPQ AX.25 Ethernet Packet  [ NOT AN OFFICIALLY REGISTERED ID ] */
#define ETH_P_IEEEPUP	0x0a00	/**< Xerox IEEE802.3 PUP packet */
#define ETH_P_IEEEPUPAT	0x0a01	/**< Xerox IEEE802.3 PUP Addr Trans packet */
#define ETH_P_DEC       0x6000	/**< DEC Assigned proto           */
#define ETH_P_DNA_DL    0x6001	/**< DEC DNA Dump/Load            */
#define ETH_P_DNA_RC    0x6002	/**< DEC DNA Remote Console       */
#define ETH_P_DNA_RT    0x6003	/* DEC DNA Routing              */
#define ETH_P_LAT       0x6004	/**< DEC LAT                      */
#define ETH_P_DIAG      0x6005	/**< DEC Diagnostics              */
#define ETH_P_CUST      0x6006	/**< DEC Customer use             */
#define ETH_P_SCA       0x6007	/**< DEC Systems Comms Arch       */
#define ETH_P_RARP      0x8035	/**< Reverse Addr Res packet      */
#define ETH_P_ATALK	0x809B	/**< Appletalk DDP                */
#define ETH_P_AARP	0x80F3	/**< Appletalk AARP               */
#define ETH_P_8021Q	0x8100	/**< 802.1Q VLAN Extended Header  */
#define ETH_P_IPX	0x8137	/**< IPX over DIX                 */
#define ETH_P_IPV6	0x86DD	/**< IPv6 over bluebook           */
#define ETH_P_PAUSE	0x8808	/**< IEEE Pause frames. See 802.3 31B */
#define ETH_P_SLOW	0x8809	/**< Slow Protocol. See 802.3ad 43B */
#define ETH_P_WCCP	0x883E	/**< Web-cache coordination protocol
				 * defined in draft-wilson-wrec-wccp-v2-00.txt */
#define ETH_P_PPP_DISC	0x8863	/**< PPPoE discovery messages     */
#define ETH_P_PPP_SES	0x8864	/**< PPPoE session messages       */
#define ETH_P_MPLS_UC	0x8847	/**< MPLS Unicast traffic         */
#define ETH_P_MPLS_MC	0x8848	/**< MPLS Multicast traffic       */
#define ETH_P_ATMMPOA	0x884c	/**< MultiProtocol Over ATM       */
#define ETH_P_ATMFATE	0x8884	/**< Frame-based ATM Transport
				 * over Ethernet
				 */
#define ETH_P_AOE	0x88A2	/**< ATA over Ethernet            */
#define ETH_P_TIPC	0x88CA	/**< TIPC                         */

/**
 \brief	Non DIX types. Won't clash for 1500 types.
 */

#define ETH_P_802_3	0x0001	/**< Dummy type for 802.3 frames  */
#define ETH_P_AX25	0x0002	/**< Dummy protocol id for AX.25  */
#define ETH_P_ALL	0x0003	/**< Every packet (be careful!!!) */
#define ETH_P_802_2	0x0004	/**< 802.2 frames                 */
#define ETH_P_SNAP	0x0005	/**< Internal only                */
#define ETH_P_DDCMP     0x0006	/**< DEC DDCMP: Internal only     */
#define ETH_P_WAN_PPP   0x0007	/**< Dummy type for WAN PPP frames */
#define ETH_P_PPP_MP    0x0008	/**< Dummy type for PPP MP frames */
#define ETH_P_LOCALTALK 0x0009	/**< Localtalk pseudo type        */
#define ETH_P_PPPTALK	0x0010	/**< Dummy type for Atalk over PPP */
#define ETH_P_TR_802_2	0x0011	/**< 802.2 frames                 */
#define ETH_P_MOBITEX	0x0015	/**< Mobitex (kaz@cafe.net)       */
#define ETH_P_CONTROL	0x0016	/**< Card specific control frames */
#define ETH_P_IRDA	0x0017	/**< Linux-IrDA                   */
#define ETH_P_ECONET	0x0018	/**< Acorn Econet                 */
#define ETH_P_HDLC	0x0019	/**< HDLC frames                  */
#define ETH_P_ARCNET	0x001A	/**< 1A for ArcNet :-)            */

#define ETH_DST_MULTICAST 0x1
#define ETH_DST_BROADCAST 0xFF

/**
 \brief	This is an Ethernet frame header.
 */
struct ethhdr {
	uint8_t h_dest[ETH_ALEN];	/**< destination eth addr */
	uint8_t h_source[ETH_ALEN];	/**< source ether addr    */
	uint16_t h_proto;	/**< packet type ID field */
} __attribute__ ((packed));

/**
 \brief	Display a 6 byte device address (MAC) in a readable format.
 */
#define MAC_FMT "%02x:%02x:%02x:%02x:%02x:%02x"

/**
 \brief String format MAC Address for printing
  Assumes character byte array 
 */
#define NMAC_STR(buf) \
	buf[0], buf[1], buf[2], buf[3], buf[4], buf[5]

/**
 \brief Double word(64 bit) MAC format for printing
  Assumes Network byte order with MAC addr in lower 6 bytes
 */
#define NMAC_64(addr) \
	((uint8_t  *)&addr)[2], \
	((uint8_t  *)&addr)[3], \
	((uint8_t  *)&addr)[4], \
	((uint8_t  *)&addr)[5], \
	((uint8_t  *)&addr)[6], \
	((uint8_t  *)&addr)[7]

#endif /* ETHER_H */
