/** 
 \file ipsecfwd_cp.c
 \brief Basic LWE IPsec Config Tool
 */
/*
 * Copyright (C) 2008-2010 Freescale Semiconductor, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 * NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include <stdio.h>
#include <argp.h>
#include <stdlib.h>
#include <string.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/mman.h>
#include "ipsecfwd_cp.h"
#include "ipsec/ipsec_cpdp.h"
#include "app_cpdp.h"

#ifdef DEBUG
#define lwe_ipsec_log_print		printf
#else
#define lwe_ipsec_log_print(a, ...)
#endif

unsigned int g_mndtr_param;
error_t g_parse_error;
volatile struct lwe_ctrl_op_info *g_p_shm_sa_Info;

unsigned char def_aes_enc_key[] =
    { 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0a, 0x0b, 0x0c,
	0x0d, 0x0e, 0x0f, 0x10
};

unsigned char def_tdes_enc_key[] =
    { 0x62, 0x7f, 0x46, 0x0e, 0x08, 0x10, 0x4a, 0x10, 0x43, 0xcd, 0x26, 0x5d,
	0x58, 0x40, 0xea, 0xf1, 0x31, 0x3e, 0xdf, 0x97, 0xdf, 0x2a, 0x8a, 0x8c
};

unsigned char def_auth_key[] =
    { 0x21, 0x22, 0x23, 0x24, 0x25, 0x26, 0x27, 0x28, 0x29, 0x2a, 0x2b, 0x2c,
	0x2d, 0x2e, 0x2f, 0x30, 0x31, 0x32, 0x33, 0x34
};

static error_t parse_opt(int key, char *arg, struct argp_state *state);
static error_t parse_add_opt(int key, char *arg, struct argp_state *state);
static error_t parse_route_opt(int key, char *arg, struct argp_state *state);
static error_t parse_route_add_opt(int key, char *arg,
				   struct argp_state *state);

static error_t parse_intf_conf_opt(int key, char *arg,
				   struct argp_state *state);

static error_t parse_arp_add_opt(int key, char *arg, struct argp_state *state);
static error_t parse_pme_add_del_opt(int key, char *arg,
				     struct argp_state *state);

static error_t parse_framecnt_edit_opt(int key, char *arg,
					struct argp_state *state);
static struct argp sastats_argp = {
	sastats_options, parse_add_opt, NULL, NULL, NULL, NULL, NULL };
static struct argp route_add_argp = {
	route_add_options, parse_route_add_opt, NULL, NULL, NULL, NULL, NULL };

static struct argp route_del_argp = {
	route_del_options, parse_route_add_opt, NULL, NULL, NULL, NULL, NULL };

static struct argp route_stats_argp = {
	route_stats_options, parse_route_add_opt, NULL, NULL, NULL, NULL,
	NULL };

static struct argp intf_conf_argp = {
	intf_conf_options, parse_intf_conf_opt, NULL, NULL, NULL, NULL, NULL };

static struct argp arp_add_argp = {
	arp_add_options, parse_arp_add_opt, NULL, NULL, NULL, NULL, NULL };

static struct argp arp_del_argp = {
	arp_del_options, parse_arp_add_opt, NULL, NULL, NULL, NULL, NULL };

static struct argp ifstats_argp = {
	ifstats_options, parse_add_opt, NULL, NULL, NULL, NULL, NULL };

static struct argp add_argp = {
	add_options, parse_add_opt, NULL, NULL, NULL, NULL, NULL };

static struct argp del_argp = {
	del_options, parse_add_opt, NULL, NULL, NULL, NULL, NULL };

static struct argp pme_add_argp = {
	pme_add_options, parse_pme_add_del_opt, NULL, NULL, NULL, NULL, NULL };

static struct argp pme_del_argp = {
	pme_del_options, parse_pme_add_del_opt, NULL, NULL, NULL, NULL, NULL };

static struct argp framecnt_argp = {
	framecnt_edit_options, parse_framecnt_edit_opt, NULL, NULL, NULL, NULL,
	NULL };

static struct argp_option options[] = {
	{"del", 'D', "TYPE", 0, "delete a SA", 0},
	{"add", 'A', "TYPE", 0, "adding a SA", 0},
	{"sastats", 'S', "type", 0, "show stats per SA", 0},
	{"ifstats", 'I', "type", 0, "show stats per Interface", 0},
	{"salist", 'L', "type", 0, "show Stats List - sa or if", 0},
	{"routeadd", 'B', "TYPE", 0, "adding a route", 0},
	{"routedel", 'C', "TYPE", 0, "deleting a route", 0},
	{"routestats", 'R', "TYPE", 0, "route stats", 0},
	{"sysstats", 'E', "TYPE", 0, "system stats", 0},
	{"intfconf", 'F', "TYPE", 0, "change intf config", 0},
	{"arpadd", 'G', "TYPE", 0, "adding a arp entry", 0},
	{"arpdel", 'H', "TYPE", 0, "deleting a arp entry", 0},
	{"pmeadd", 'J', "TYPE", 0, "adding a pme context entry", 0},
	{"pmedel", 'K', "TYPE", 0, "deleting a pme context entry", 0},
	{"framecnt", 'N', "TYPE", 0, "edit number of frames", 0},
	{"Start/ Go", 'O', "TYPE", 0,
	 "Start the steady phase/ processing of packets", 0},
	{"Reset", 'T', "TYPE", 0, "reset the application"},
	{"Resetstats", 'P', "TYPE", 0, "reset the application stats"},
	{0, 0, 0, 0, 0, 0}
};

/*
   The ARGP structure itself.
*/

static struct argp argp = {
	options, parse_opt, NULL, NULL, NULL, NULL, NULL };

/**
 \brief Defines actions for parsing the add/ delete command options; 
	it is called for each option parsed
 \param[in] key For each option that is parsed, parser is called with a value of 
		key from that option's key field in the option vector
 \param[in] arg If key is an option, arg is its given value.
 \param[in] state state points to a struct argp_state, containing pointer to pme_info structure
 \return 0 for success, ARGP_ERR_UNKNOWN if the value of key is not handled by this parser function
 */
static error_t parse_pme_add_del_opt(int key, char *arg,
				     struct argp_state *state)
{
	struct lwe_ctrl_op_info *sa_info = state->input;
	struct lwe_ctrl_spa_cfg_info *pme_info = &(sa_info->app_info.pme_info);
	uint32_t tmp = 0;
	int32_t val;

	lwe_ipsec_log_print("\r\nKey = %c, arg = %s", key, arg);

	switch (key) {

		/* flow_id */
	case 'f':
		pme_info->flow_id = atoi(arg);
		if ((pme_info->flow_id < LWE_CTRL_PME_FLOWID_MIN) ||
		    (pme_info->flow_id > LWE_CTRL_PME_FLOWID_MAX)) {
			printf("Invalid Value \"%s\" for '%c'\n", arg, key);
			g_parse_error = ERANGE;
			return ERANGE;
		}

		g_mndtr_param |= LWE_CTRL_PARAM_BMASK_PME_FLOWID;
		break;

		/* set */
	case 's':
		val = atoi(arg);

		if ((val < LWE_CTRL_PME_SET_MIN) ||
		    (val > LWE_CTRL_PME_SET_MAX)) {
			printf("Invalid Value \"%s\" for '%c'\n", arg, key);
			g_parse_error = ERANGE;
			return ERANGE;
		}

		pme_info->set = (uint8_t)val;
		g_mndtr_param |= LWE_CTRL_PARAM_BMASK_PME_SET;
		break;

		/* subset */
	case 'u':
		val = atoi(arg);
		if ((val < LWE_CTRL_PME_SUBSET_MIN) ||
		    (val > LWE_CTRL_PME_SUBSET_MAX)) {
			printf("Invalid Value \"%s\" for '%c'\n", arg, key);
			g_parse_error = ERANGE;
			return ERANGE;
		}
		pme_info->subset = (uint16_t)val;

		g_mndtr_param |= LWE_CTRL_PARAM_BMASK_PME_SUBSET;
		break;
		/* scan mode type - direct/ flow */
	case 'm':
		tmp = atoi(arg);
		if (tmp == 1)
			pme_info->flag |= SET_SCAN_MODE_FLOW;
		else
			pme_info->flag &= SET_SCAN_MODE_DIRECT;
		g_mndtr_param |= LWE_CTRL_PARAM_BMASK_PME_FLAG;
		break;

		/* residue - enable/ disable */
	case 'r':
		tmp = atoi(arg);
		if (tmp == 1)
			pme_info->flag |= SET_RESIDUE_ENABLE;
		else
			pme_info->flag &= SET_RESIDUE_DISABLE;
		g_mndtr_param |= LWE_CTRL_PARAM_BMASK_PME_FLAG;
		break;

		/* report verbosity */
	case 'p':
		val = atoi(arg);
		if ((val < LWE_CTRL_PME_REPORT_VERBOSITY_MIN)
		    || (val > LWE_CTRL_PME_REPORT_VERBOSITY_MAX)) {
			printf("Invalid Value \"%s\" for '%c'\n", arg, key);
			g_parse_error = ERANGE;
			return ERANGE;
		}
		pme_info->report_verbosity = val;

		g_mndtr_param |= LWE_CTRL_PARAM_BMASK_PME_REPORT_VERBOSITY;
		break;

		/* session id */
	case 'i':
		val = atoi(arg);
		if ((val < LWE_CTRL_PME_SESSION_ID_MIN) ||
		    (val > LWE_CTRL_PME_SESSION_ID_MAX)) {
			printf("Invalid Value \"%s\" for '%c'\n", arg, key);
			g_parse_error = ERANGE;
			return ERANGE;
		}

		pme_info->session_id = val;
		g_mndtr_param |= LWE_CTRL_PARAM_BMASK_PME_SESSION_ID;
		break;

		/* output buffer size */
	case 'b':
		val = atoi(arg);
		if ((val < LWE_CTRL_PME_INPUT_BUFF_CHUNK_MIN)
		    || (val > LWE_CTRL_PME_INPUT_BUFF_CHUNK_MAX)) {
			printf("Invalid Value \"%s\" for '%c'\n", arg, key);
			g_parse_error = ERANGE;
			return ERANGE;
		}

		pme_info->input_buff_chunk = val;
		g_mndtr_param |= LWE_CTRL_PARAM_BMASK_PME_INPUT_BUFF_CHUNK;
		break;

		/* Qos In */
	case 'q':
		val = atoi(arg);
		if ((val < LWE_CTRL_PME_QOS_IN_MIN) ||
		    (val > LWE_CTRL_PME_QOS_IN_MAX)) {
			printf("Invalid Value \"%s\" for '%c'\n", arg, key);
			g_parse_error = ERANGE;
			return ERANGE;
		}

		pme_info->qos_in = val;
		g_mndtr_param |= LWE_CTRL_PARAM_BMASK_PME_QOS_IN;
		break;

		/* Qos out */
	case 't':
		val = atoi(arg);
		if ((val < LWE_CTRL_PME_QOS_OUT_MIN) ||
		    (val > LWE_CTRL_PME_QOS_OUT_MAX)) {
			printf("Invalid Value \"%s\" for '%c'\n", arg, key);
			g_parse_error = ERANGE;
			return ERANGE;
		}

		pme_info->qos_out = val;
		g_mndtr_param |= LWE_CTRL_PARAM_BMASK_PME_QOS_OUT;
		break;

		/* match limit */
	case 'l':
		val = atoi(arg);
		if ((val < LWE_CTRL_PME_MATCH_LIMIT_MIN) ||
		    (val > LWE_CTRL_PME_MATCH_LIMIT_MAX)) {
			printf("Invalid Value \"%s\" for '%c'\n", arg, key);
			g_parse_error = ERANGE;
			return ERANGE;
		}

		pme_info->match_limit = val;
		g_mndtr_param |= LWE_CTRL_PARAM_BMASK_PME_MATCH_LIMIT;
		break;

		/* compare limit */
	case 'o':
		val = atoi(arg);
		if ((val < LWE_CTRL_PME_COMPARE_LIMIT_MIN)
		    || (val > LWE_CTRL_PME_COMPARE_LIMIT_MAX)) {
			printf("Invalid Value \"%s\" for '%c'\n", arg, key);
			g_parse_error = ERANGE;
			return ERANGE;
		}

		pme_info->compare_limit = val;
		g_mndtr_param |= LWE_CTRL_PARAM_BMASK_PME_COMPARE_LIMIT;
		break;

		/* scan function type - block/ non-block */
	case 'n':
		tmp = atoi(arg);
		if (tmp == 1)
			pme_info->flag |= SET_SCAN_FN_TYPE_BLOCK;
		else
			pme_info->flag &= SET_SCAN_FN_TYPE_NON_BLOCK;
		g_mndtr_param |= LWE_CTRL_PARAM_BMASK_PME_FLAG;
		break;

		/* output buffer from App */
	case 'e':
		val = atoi(arg);
		if ((val < LWE_CTRL_PME_OUTPUT_BUFF_FRM_APP_MIN)
		    || (val > LWE_CTRL_PME_OUTPUT_BUFF_FRM_APP_MAX)) {
			printf("Invalid Value \"%s\" for '%c'\n", arg, key);
			g_parse_error = ERANGE;
			return ERANGE;
		}

		pme_info->output_buff_frm_app = val;
		g_mndtr_param |= LWE_CTRL_PARAM_BMASK_PME_OUTPUT_BUFF_FRM_APP;
		break;

		/* input buffer size */
	case 'z':
		val = atoi(arg);
		if ((val < LWE_CTRL_PME_INPUT_BUFF_SIZE_MIN)
		    || (val > LWE_CTRL_PME_INPUT_BUFF_SIZE_MAX)) {
			printf("Invalid Value \"%s\" for '%c'\n", arg, key);
			g_parse_error = ERANGE;
			return ERANGE;
		}

		pme_info->input_buff_size = val;
		g_mndtr_param |= LWE_CTRL_PARAM_BMASK_PME_INPUT_BUFF_SIZE;
		break;

		/* input file to be read */
	case 'd':
		{
			FILE *fd;
			fd = fopen(arg, "r");
			if (!fd) {
				printf("File %s does not exist\n", arg);
				return ENOENT;
			}
			pme_info->size_of_file =
			    fread(pme_info->file, 1, PME_MAX_FILE_SIZE, fd);
			fclose(fd);

			if (pme_info->size_of_file)
				g_mndtr_param |=
				    LWE_CTRL_PARAM_BMASK_PME_INPUT_FILE;
			else
				return ARGP_ERR_UNKNOWN;
			break;
		}

	default:
		return ARGP_ERR_UNKNOWN;
	}

	return 0;
}

/**
 \brief Defines actions for parsing the add/ delete command options; 
	it is called for each option parsed
 \param[in] key For each option that is parsed, parser is called with a value of 
		key from that option's key field in the option vector
 \param[in] arg If key is an option, arg is its given value.
 \param[in] state state points to a struct argp_state, containing pointer to route_info structure
 \return 0 for success, ARGP_ERR_UNKNOWN if the value of key is not handled by this parser function
		ARG_KEY_ERROR if the interface name is too long
 */
static error_t parse_intf_conf_opt(int key, char *arg, struct argp_state *state)
{
	struct lwe_ctrl_op_info *route_info = state->input;
	struct in_addr in_addr;
	switch (key) {

	case 'a':
		inet_aton(arg, &in_addr);
		route_info->app_info.ip_info.intf_conf.ip_addr = in_addr.s_addr;
		g_mndtr_param |= LWE_CTRL_PARAM_BMASK_IPADDR;
		break;

	case 'm':
		{
			char *pch;
			uint32_t l = 0;
			uint32_t i = 0;
			pch = strtok(arg, ":");
			while (pch != NULL) {
				sscanf(pch, "%x", &l);
				route_info->app_info.ip_info.
				    intf_conf.mac_addr[i] = (uint8_t)l;
				pch = strtok(NULL, ":");
				i++;
			}
			g_mndtr_param |= LWE_CTRL_PARAM_BMASK_MACADDR;
			break;
		}
	case 'i':
		if ((strlen(arg)) < sizeof(route_info->app_info.
			ip_info.intf_conf.ifname)) {
			strcpy(route_info->app_info.ip_info.
				intf_conf.ifname, arg);
		} else {
			return ARGP_KEY_ERROR;
		}
		g_mndtr_param |= LWE_CTRL_PARAM_BMASK_IFNAME;
		break;

	case 't':
		route_info->app_info.ip_info.intf_conf.mtu = atoi(arg);
		g_mndtr_param |= LWE_CTRL_PARAM_BMASK_MTU;
		break;

	case 'e':
		if (strcmp(arg, "true") == 0) {
			route_info->app_info.ip_info.intf_conf.enable = 1;
		} else {
			route_info->app_info.ip_info.intf_conf.enable = 0;
		}
		g_mndtr_param |= LWE_CTRL_PARAM_BMASK_ENABLE;
		break;

	default:
		return ARGP_ERR_UNKNOWN;
	}

	route_info->app_info.ip_info.intf_conf.bitmask = g_mndtr_param;

	return 0;
}

/**
 \brief Defines actions for parsing the add/ delete command options; 
	it is called for each option parsed
 \param[in] key For each option that is parsed, parser is called with a value of 
		key from that option's key field in the option vector
 \param[in] arg If key is an option, arg is its given value.
 \param[in] state state points to a struct argp_state, containing pointer to route_info structure
 \return 0 for success, ARGP_ERR_UNKNOWN if the value of key is not handled by this parser function
	ARG_KEY_ERROR if the interface name is too long
 */
static error_t parse_route_add_opt(int key, char *arg, struct argp_state *state)
{
	struct lwe_ctrl_op_info *route_info = state->input;
	struct in_addr in_addr;

	switch (key) {

	case 's':
		inet_aton(arg, &in_addr);
		route_info->app_info.ip_info.src_ipaddr = in_addr.s_addr;
		g_mndtr_param |= LWE_CTRL_PARAM_BMASK_SRCIP;
		break;

	case 'd':
		inet_aton(arg, &in_addr);
		route_info->app_info.ip_info.dst_ipaddr = in_addr.s_addr;
		g_mndtr_param |= LWE_CTRL_PARAM_BMASK_DESTIP;
		lwe_ipsec_log_print("\nkey = %c; value = %s", key, arg);
		break;

	case 'g':

		inet_aton(arg, &in_addr);
		route_info->app_info.ip_info.gw_ipaddr = in_addr.s_addr;
		g_mndtr_param |= LWE_CTRL_PARAM_BMASK_GWIP;
		lwe_ipsec_log_print("\nkey = %c; value = %s", key, arg);
		break;

	case 't':
		/* 
		 ** TBD: Not Taking TOS from user right now- 
		 ** setting it as 0 in the starting - 
		 ** route_info->tos = atoi(arg);
		 */
		if ((atoi(arg) < LWE_CTRL_ROUTE_TOS_MIN) ||
		    (atoi(arg) > LWE_CTRL_ROUTE_TOS_MAX)) {
			printf("Invalid Value \"%s\" for '%c'\n", arg, key);
			g_parse_error = ERANGE;
			return ERANGE;
		}

		g_mndtr_param |= LWE_CTRL_PARAM_BMASK_TOS;
		lwe_ipsec_log_print("\nkey = %c; value = %s", key, arg);
		break;

	case 'f':
		route_info->app_info.ip_info.flow_id = atoi(arg);
		if ((atoi(arg) < LWE_CTRL_ROUTE_FLOWID_MIN) ||
		    (atoi(arg) > LWE_CTRL_ROUTE_FLOWID_MAX)) {
			printf("Invalid Value \"%s\" for '%c'\n", arg, key);
			g_parse_error = ERANGE;
			return ERANGE;
		}

		g_mndtr_param |= LWE_CTRL_PARAM_BMASK_FLOWID;
		lwe_ipsec_log_print("\nkey = %c; value = %s", key, arg);
		break;

	default:
		return ARGP_ERR_UNKNOWN;
	}

	return 0;
}

/**
 \brief Defines actions for parsing the add/ delete command options; 
	it is called for each option parsed
 \param[in] key For each option that is parsed, parser is called with a value of 
		key from that option's key field in the option vector
 \param[in] arg If key is an option, arg is its given value.
 \param[in] state state points to a struct argp_state, containing pointer to route_info structure
 \return 0 for success, ARGP_ERR_UNKNOWN if the value of key is not handled by this parser function
 */
static error_t parse_arp_add_opt(int key, char *arg, struct argp_state *state)
{
	struct lwe_ctrl_op_info *route_info = state->input;
	struct in_addr in_addr;

	switch (key) {

	case 's':
		inet_aton(arg, &in_addr);
		route_info->app_info.ip_info.src_ipaddr = in_addr.s_addr;
		g_mndtr_param |= LWE_CTRL_PARAM_BMASK_ARP_IPADDR;
		break;

	case 'm':
		{
			char *pch;
			uint32_t l = 0;
			uint32_t i = 0;
			pch = strtok(arg, ":");
			while (pch != NULL) {
				sscanf(pch, "%x", &l);
				route_info->app_info.ip_info.mac_addr[i]
					= (uint8_t)l;
				pch = strtok(NULL, ":");
				i++;
			}
			g_mndtr_param |= LWE_CTRL_PARAM_BMASK_ARP_MACADDR;
			break;
		}

	case 'r':
		if (strcmp(arg, "true") == 0)
			route_info->app_info.ip_info.replace_entry = 1;
		else
			route_info->app_info.ip_info.replace_entry = 0;
		g_mndtr_param |= LWE_CTRL_PARAM_BMASK_ARP_REPLACE;
		break;

	default:
		return ARGP_ERR_UNKNOWN;
	}

	return 0;
}

/**
 \brief Defines actions for parsing the add/ delete/ sastats/ ifstats command options; 
	it is called for each option parsed
 \param[in] key For each option that is parsed, parser is called with a value of 
		key from that option's key field in the option vector
 \param[in] arg If key is an option, arg is its given value.
 \param[in] state state points to a struct argp_state, containing pointer to sa_info structure
 \return 0 for success, ARGP_ERR_UNKNOWN if the value of key is not handled by this parser function
 */
static error_t parse_add_opt(int key, char *arg, struct argp_state *state)
{
	struct lwe_ctrl_op_info *sa_info = state->input;
	struct in_addr in_addr;
	int64_t val;

	switch (key) {

	case 's':
		inet_aton(arg, &in_addr);
		sa_info->app_info.ipsec_info.sel.saddr = in_addr.s_addr;
		g_mndtr_param |= LWE_CTRL_PARAM_BIT_SRCIP;
		lwe_ipsec_log_print
		    ("\nkey = %c; value = %s; sa_info->app_info.ipsec_info.sel.saddr = 0x%x",
		     key, arg, sa_info->app_info.ipsec_info.sel.saddr);
		break;

	case 'd':
		inet_aton(arg, &in_addr);
		sa_info->app_info.ipsec_info.sel.daddr = in_addr.s_addr;
		g_mndtr_param |= LWE_CTRL_PARAM_BIT_DESTIP;
		lwe_ipsec_log_print("\nkey = %c; value = %s", key, arg);
		break;

	case 'g':
		inet_aton(arg, &in_addr);
		sa_info->app_info.ipsec_info.id.saddr = in_addr.s_addr;
		g_mndtr_param |= LWE_CTRL_PARAM_BIT_SRCGW;
		lwe_ipsec_log_print("\nkey = %c; value = %s", key, arg);
		break;

	case 'G':
		inet_aton(arg, &in_addr);
		sa_info->app_info.ipsec_info.id.daddr = in_addr.s_addr;
		g_mndtr_param |= LWE_CTRL_PARAM_BIT_DESTGW;
		lwe_ipsec_log_print("\nkey = %c; value = %s", key, arg);
		break;

	case 'p':
		if ((atoi(arg) < LWE_CTRL_SA_IPSEC_PROTO_MIN) ||
		    (atoi(arg) > LWE_CTRL_SA_IPSEC_PROTO_MAX)) {
			printf("Invalid Value \"%s\" for '%c'\n", arg, key);
			g_parse_error = ERANGE;
			return ERANGE;
		}

		g_mndtr_param |= LWE_CTRL_PARAM_BIT_PROTO;
		lwe_ipsec_log_print("\nkey = %c; value = %s", key, arg);
		break;

	case 'a':
		lwe_ipsec_log_print("\nkey = %c; value = %s", key, arg);
		sa_info->app_info.ipsec_info.aalg.alg_key_len = strlen(arg);
		memcpy(sa_info->app_info.ipsec_info.aalg.alg_key, arg,
		       sa_info->app_info.ipsec_info.aalg.alg_key_len);
		g_mndtr_param |= LWE_CTRL_PARAM_BIT_AKEY;
		break;

	case 'e':
		lwe_ipsec_log_print("\nkey = %c; value = %s", key, arg);
		sa_info->app_info.ipsec_info.ealg.alg_key_len = strlen(arg);
		memcpy(sa_info->app_info.ipsec_info.ealg.alg_key, arg,
		       sa_info->app_info.ipsec_info.ealg.alg_key_len);
		g_mndtr_param |= LWE_CTRL_PARAM_BIT_EKEY;
		break;

	case 'i':
		lwe_ipsec_log_print("\nkey = %c; value = %s", key, arg);
		sa_info->app_info.ipsec_info.id.spi = atoi(arg);
		g_mndtr_param |= LWE_CTRL_PARAM_BIT_SPI;
		break;

	case 't':
		lwe_ipsec_log_print("\nkey = %c; value = %s", key, arg);
		val = atoi(arg);
		if ((val < LWE_CTRL_SA_ENCRIPTN_TYPE_MIN)
		    || (val > LWE_CTRL_SA_ENCRIPTN_TYPE_MAX)) {
			printf("Invalid Value \"%s\" for '%c'\n", arg, key);
			g_parse_error = ERANGE;
			return ERANGE;
		}

		sa_info->app_info.ipsec_info.ealg.alg_type = (uint32_t)val;
		g_mndtr_param |= LWE_CTRL_PARAM_BIT_ETYPE;
		break;

	case 'y':
		lwe_ipsec_log_print("\nkey = %c; value = %s", key, arg);
		val = atoi(arg);
		if ((val < LWE_CTRL_SA_AUTH_TYPE_MIN)
		    || (val > LWE_CTRL_SA_AUTH_TYPE_MAX)) {
			printf("Invalid Value \"%s\" for '%c'\n", arg, key);
			g_parse_error = ERANGE;
			return ERANGE;
		}

		sa_info->app_info.ipsec_info.aalg.alg_type = (uint32_t)val;
		g_mndtr_param |= LWE_CTRL_PARAM_BIT_ATYPE;
		break;

	case 'r':
		{
			char *dir = "in";
			if (strcmp(arg, dir) == 0) {
				sa_info->app_info.ipsec_info.id.dir =
				    IPSEC_DIR_IN;
			} else {
				sa_info->app_info.ipsec_info.id.dir =
				    IPSEC_DIR_OUT;
			}
			g_mndtr_param |= LWE_CTRL_PARAM_BIT_DIR;
			break;
		}

	case 'f':
		sa_info->app_info.ipsec_info.flow_id = atoi(arg);
		if ((atoi(arg) < LWE_CTRL_ROUTE_FLOWID_MIN) ||
		    (atoi(arg) > LWE_CTRL_ROUTE_FLOWID_MAX)) {
			printf("Invalid Value \"%s\" for '%c'\n", arg, key);
			g_parse_error = ERANGE;
			return ERANGE;
		}

		g_mndtr_param |= LWE_CTRL_PARAM_BIT_FLOWID;
		lwe_ipsec_log_print("\nkey = %c; value = %s", key, arg);
		break;


	default:
		return ARGP_ERR_UNKNOWN;
	}

	return 0;
}

/**
 \brief Defines actions for parsing the frame count edit command options; 
	it is called for each option parsed
 \param[in] key For each option that is parsed, parser is called with a value of 
		key from that option's key field in the option vector
 \param[in] arg If key is an option, arg is its given value.
 \param[in] state state points to a struct argp_state, containing pointer to route_info structure
 \return 0 for success, ARGP_ERR_UNKNOWN if the value of key is not handled by this parser function
 */
static error_t parse_framecnt_edit_opt(int key, char *arg,
					struct argp_state *state)
{
	struct lwe_ctrl_op_info *route_info = state->input;
	
	switch (key) {

	case 'n':
		route_info->app_info.ip_info.frame_cnt = atoi(arg);
		break;
	default:
		return ARGP_ERR_UNKNOWN;
	}
	
	return 0;
}

/**
 \brief Copies data from shared memory area into saInfo structure
 \param[in] saInfo Pointer to the structure buffer for copying data
 \return none
 */
void lwe_ipsec_copy_from_shared_memory(struct lwe_ctrl_op_info *saInfo)
{
	unsigned int result = LWE_CTRL_RSLT_SUCCESSFULL;

	memcpy(saInfo, (void *)g_p_shm_sa_Info,
	       sizeof(struct lwe_ctrl_op_info));

	result = g_p_shm_sa_Info->result;

	switch (saInfo->msg_type) {
	case LWE_CTRL_CMD_TYPE_IPFWD_SYS_STATS:
		{
			if (result == LWE_CTRL_RSLT_SUCCESSFULL) {
				/**< print the entry stats */
				printf("Following is System Stats :\n");
				printf
				    ("Number of enteries in the Route Cache		: 	0x%x\n",
				     saInfo->app_info.ip_info.
				     stats.entry_count);
				printf
				    ("Number of stale enteries in the Route Cache	: 	0x%x\n",
				     saInfo->app_info.ip_info.stats.
				     stale_entries);
				printf
				    ("Number of enteries added in the Route Cache	: 	0x%x\n",
				     saInfo->app_info.ip_info.stats.
				     new_entries);
				printf
				    ("Number of enteries removed in the Route Cache	: 	0x%x\n",
				     saInfo->app_info.ip_info.stats.
				     removed_entries);
				printf
				    ("Number of times the LookUp succeeded		: 	0x%08x%08x\n",
				     saInfo->app_info.ip_info.
				     stats.hits.words.hi,
				     saInfo->app_info.ip_info.
				     stats.hits.words.lo);
				printf
				    ("Number of times the LookUp failed		: 	0x%08x%08x\n",
				     saInfo->app_info.ip_info.
				     stats.misses.words.hi,
				     saInfo->app_info.ip_info.
				     stats.misses.words.lo);

			} else {
				printf
				    ("Route Entry/ System Stats Get failed\n");
			}
		}
		break;

	case LWE_CTRL_CMD_TYPE_ROUTE_STATS:
		{
			if (result == LWE_CTRL_RSLT_SUCCESSFULL) {
				/**< print the entry stats */
				printf("Following is Route Stats :\n");
				printf
				    ("Number of times the entry was accessed	: 	0x%08x%08x\n",
				     saInfo->app_info.ip_info.stats.hits.
				     words.hi,
				     saInfo->app_info.ip_info.stats.hits.
				     words.lo);
				printf
				    ("Number of Octets				: 	0x%08x%08x\n",
				     saInfo->app_info.
				     ip_info.
				     stats.misses.
				     words.hi,
				     saInfo->app_info.
				     ip_info.stats.misses.words.lo);

			} else {
				printf
				    ("Route Entry/ System Stats Get failed\n");
			}
		}
		break;

	case LWE_CTRL_CMD_TYPE_SASTATS:
		{
			if (result == LWE_CTRL_RSLT_SUCCESSFULL) {
				/**< print the entry stats */
				printf("Following are the SA entry Stats:\n");
				printf("Encap Packets	: %d\n",
				       saInfo->app_info.ipsec_info.stats.
				       encap_pkts);
				printf("Encap Octets		: %d\n",
				       saInfo->app_info.ipsec_info.
				       stats.encap_octets);
				printf("Decap Packets	: %d\n",
				       saInfo->app_info.ipsec_info.stats.
				       decap_pkts);
				printf("Decap Octets		: %d\n",
				       saInfo->app_info.ipsec_info.
				       stats.decap_octets);
			} else {
				printf("SA Entry Stats Get failed\n");
			}
		}
		break;

	case LWE_CTRL_CMD_TYPE_IFSTATS:
		{
			if (result == LWE_CTRL_RSLT_SUCCESSFULL) {
				/**< print the entry stats */
			} else {
				printf("SA Entry Stats Get failed\n");
			}
		}
		break;

	case LWE_CTRL_CMD_TYPE_IFLIST:
		{
			if (result == LWE_CTRL_RSLT_SUCCESSFULL) {
				/**< print the if stats list */

			} else {
				printf("SA Entry Stats Get failed\n");
			}
		}
		break;

	default:
		{
			if (result == LWE_CTRL_RSLT_SUCCESSFULL) {
				/**< print the sa stats list*/
			} else {
				printf("SA Stats List Get failed\n");
			}
		}
	}

	return;
}

/**
 \brief Copies data from the saInfo structure to the shared memory area.
 \param[in] saInfo Pointer to the structure that needs to be copies onto shared memory area
 \return none
 */
void lwe_ipsec_copy_to_shared_memory(struct lwe_ctrl_op_info *saInfo)
{
	unsigned int result = LWE_CTRL_RSLT_SUCCESSFULL;

	/**< Copy the structure onto the white board shared memory */
	memcpy((void *)g_p_shm_sa_Info, saInfo,
	       sizeof(struct lwe_ctrl_op_info));

	/**< Set the CmdFlag to 1 */
	g_p_shm_sa_Info->state = LWE_CTRL_CMD_STATE_BUSY;

	/**< Poll on CmdFlag to become 0 */
	while (g_p_shm_sa_Info->state != LWE_CTRL_CMD_STATE_IDLE) ;

	result = g_p_shm_sa_Info->result;
	if (saInfo->msg_type == LWE_CTRL_CMD_TYPE_SAADD) {
		if (result == LWE_CTRL_RSLT_SUCCESSFULL) {
			printf("SA Entry Added successfully\n");
		} else {
			printf("SA Entry Addition failed\n");
		}
	} else if (saInfo->msg_type == LWE_CTRL_CMD_TYPE_SADEL) {
		if (result == LWE_CTRL_RSLT_SUCCESSFULL) {
			printf("SA Entry Deleted successfully\n");
		} else {
			printf("SA Entry Deletion failed\n");
		}
	} else if (saInfo->msg_type == LWE_CTRL_CMD_TYPE_ROUTE_ADD) {
		if (result == LWE_CTRL_RSLT_SUCCESSFULL) {
			printf("Route Entry Added successfully\n");

		} else {
			printf("Route Entry Addition failed\n");
		}
	} else if (saInfo->msg_type == LWE_CTRL_CMD_TYPE_ROUTE_DEL) {
		if (result == LWE_CTRL_RSLT_SUCCESSFULL) {
			printf("Route Entry Deleted successfully\n");
		} else {
			printf("Route Entry Deletion failed\n");
		}
	} else if (saInfo->msg_type == LWE_CTRL_CMD_TYPE_INTF_CONF_CHNG) {
		if (result == LWE_CTRL_RSLT_SUCCESSFULL) {
			printf("Intf Configuration Changed successfully\n");
		} else {
			printf("Intf Configuration Change failed\n");
		}
	} else if (saInfo->msg_type == LWE_CTRL_CMD_TYPE_ARP_ADD) {
		if (result == LWE_CTRL_RSLT_SUCCESSFULL) {
			printf("ARP Entry Added successfully\n");

		} else {
			printf("ARP Entry Addition failed\n");
		}
	} else if (saInfo->msg_type == LWE_CTRL_CMD_TYPE_ARP_DEL) {
		if (result == LWE_CTRL_RSLT_SUCCESSFULL) {
			printf("ARP Entry Deleted successfully\n");
		} else {
			printf("ARP Entry Deletion failed\n");
		}
	} else if (saInfo->msg_type == LWE_CTRL_CMD_TYPE_PME_ADD) {
		if (result == LWE_CTRL_RSLT_SUCCESSFULL) {
			printf("PME Entry Added successfully\n");

		} else {
			printf("PME Entry Addition failed\n");
		}
	} else if (saInfo->msg_type == LWE_CTRL_CMD_TYPE_PME_DEL) {
		if (result == LWE_CTRL_RSLT_SUCCESSFULL) {
			printf("PME Entry Deleted successfully\n");
		} else {
			printf("PME Entry Deletion failed\n");
		}
	} else if (saInfo->msg_type == LWE_CTRL_CMD_TYPE_FRAMECNT_EDIT) {
		if (result == LWE_CTRL_RSLT_SUCCESSFULL) {
			printf("Frame count edited successfully\n");
		} else {
			printf("Frame count edition failed\n");
		}
	} else if (saInfo->msg_type == LWE_CTRL_CMD_TYPE_GO) {
		if (result == LWE_CTRL_RSLT_SUCCESSFULL) {
			printf("Application Started successfully\n");
		} else {
			printf("Application failed\n");
		}
	} else if (saInfo->msg_type == LWE_CTRL_CMD_TYPE_RESET) {
		if (result == LWE_CTRL_RSLT_SUCCESSFULL) {
			printf("Application is getting reset\n");
		} else {
			printf("Application reset failed\n");
		}
	} else if (saInfo->msg_type == LWE_CTRL_CMD_TYPE_RESET_STATS) {
		if (result == LWE_CTRL_RSLT_SUCCESSFULL) {
			printf("Application stats getting reset\n");
		} else {
			printf("Application stats reset failed\n");
		}
	} else {
		lwe_ipsec_copy_from_shared_memory(saInfo);
	}

	return;
}

/**
 \brief Processes the System/ Route Stats Request
 \param[in] argc Number of arguments
 \param[in] argv Arguments
 \param[in] type Message Type for the CP request - System/ Route Stats
 \return none
 */
void lwe_ip_stats_command(int argc, char **argv, char *type)
{
	unsigned int i = 0;
	struct lwe_ctrl_op_info route_info;
	unsigned int mndtr_param_map = LWE_CTRL_ROUTE_STATS_MDTR_PARAM_MAP;
	struct argp *route_argp = &route_stats_argp;

	lwe_ipsec_log_print("\nlwe_ip_stats_command: Enter");

	/*
	 ** Initializing the route info structure
	 */
	memset(&route_info, 0, sizeof(route_info));
	route_info.state = LWE_CTRL_CMD_STATE_IDLE;
	route_info.msg_type = (unsigned int)type;
	route_info.result = LWE_CTRL_RSLT_FAILURE;

	g_mndtr_param = 0;

	/* If we need system stats, no parameter is needed */
	if ((route_info.msg_type == LWE_CTRL_CMD_TYPE_IPFWD_SYS_STATS) ||
	    (LWE_CTRL_CMD_TYPE_GO == route_info.msg_type) ||
	    (LWE_CTRL_CMD_TYPE_RESET == route_info.msg_type) ||
	    (LWE_CTRL_CMD_TYPE_RESET_STATS == route_info.msg_type))
		goto copy;

	/* Where the magic happens */
	argp_parse(route_argp, argc, argv, 0, 0, &route_info);

	/*
	 ** If all the mandatory parameters for the operation are present,
	 ** Copy the data onto the shared memory area
	 */
	if ((g_mndtr_param & mndtr_param_map) == mndtr_param_map)
		goto copy;

	/*
	 ** Check for the mandatory parameter which is misisng
	 */
	for (i = 0; i < LWE_CTRL_PARAM_MAX_IP_BIT_NO; i++) {
		if (((mndtr_param_map & (1 << i)) != 0)
		    && ((g_mndtr_param & (1 << i)) == 0)) {
			printf
			    ("Route Entry Operation failed as mandatory parameters missing; --%s or -%c\n",
			     route_add_options[i].name,
			     route_add_options[i].key);
			return;
		}
	}			/* end of for (i = 0; i < LWE_CTRL_PARAM_MAX_B .... */

copy:
	lwe_ipsec_copy_to_shared_memory(&route_info);

	return;
}

/**
 \brief Processes the IP Interface Config Request
 \param[in] argc Number of arguments
 \param[in] argv Arguments
 \param[in] type Message Type for the CP request - IP Intf Config
 \return none
 */
void lwe_ip_intf_chng_command(int argc, char **argv, char *type)
{
	unsigned int i = 0;
	struct lwe_ctrl_op_info route_info;
	unsigned int mndtr_param_map = LWE_CTRL_INTF_CONF_MDTR_PARAM_MAP;
	struct argp *route_argp = &intf_conf_argp;

	lwe_ipsec_log_print("\nlwe_ip_intf_chng_command: Enter");

	/*
	 ** Initializing the route info structure
	 */
	memset(&route_info, 0, sizeof(route_info));
	route_info.state = LWE_CTRL_CMD_STATE_IDLE;
	route_info.msg_type = (unsigned int)type;
	route_info.result = LWE_CTRL_RSLT_FAILURE;

	g_mndtr_param = 0;

	/* Where the magic happens */
	argp_parse(route_argp, argc, argv, 0, 0, &route_info);

	/*
	 ** If all the mandatory parameters for the operation are present,
	 ** Copy the data onto the shared memory area
	 */
	if ((g_mndtr_param & mndtr_param_map) == mndtr_param_map)
		goto copy;

	/*
	 ** Check for the mandatory parameter which is misisng
	 */
	for (i = 0; i < LWE_CTRL_PARAM_MAX_INTF_BIT_NO; i++) {
		if (((mndtr_param_map & (1 << i)) != 0)
		    && ((g_mndtr_param & (1 << i)) == 0)) {
			printf
			    ("Interface Config Operation failed as mandatory parameters missing; --%s or -%c\n",
			     route_add_options[i].name,
			     route_add_options[i].key);
			return;
		}
	}			/* end of for (i = 0; i < LWE_CTRL_PARAM_MAX_B .... */

copy:
	lwe_ipsec_copy_to_shared_memory(&route_info);

	return;
}

/**
 \brief Processes the Route Add/ Delete Request
 \param[in] argc Number of arguments
 \param[in] argv Arguments
 \param[in] type Message Type for the CP request - Route Add or delete
 \return none
 */
void lwe_ip_add_del_command(int argc, char **argv, char *type)
{
	unsigned int i = 0;
	struct lwe_ctrl_op_info route_info;
	unsigned int mndtr_param_map[] = { LWE_CTRL_ROUTE_ADD_MDTR_PARAM_MAP,
		LWE_CTRL_ROUTE_DEL_MDTR_PARAM_MAP
	};
	struct argp *route_argp[] = { &route_add_argp, &route_del_argp };

	/*
	 ** Initializing the route info structure
	 */
	memset(&route_info, 0, sizeof(route_info));
	route_info.state = LWE_CTRL_CMD_STATE_IDLE;
	route_info.msg_type = (unsigned int)type;
	route_info.result = LWE_CTRL_RSLT_FAILURE;

	g_mndtr_param = 0;
	g_parse_error = 0;

	/* Where the magic happens */
	argp_parse(route_argp
		   [route_info.msg_type - LWE_CTRL_CMD_TYPE_ROUTE_ADD], argc,
		   argv, 0, 0, &route_info);

	if (g_parse_error != 0)
		return;

	/*
	 ** If all the mandatory parameters for the operation are present,
	 ** Copy the data onto the shared memory area
	 */
	if ((g_mndtr_param &
	     mndtr_param_map[route_info.msg_type -
			     LWE_CTRL_CMD_TYPE_ROUTE_ADD]) ==
	    mndtr_param_map[route_info.msg_type - LWE_CTRL_CMD_TYPE_ROUTE_ADD])
		goto copy;

	/*
	 ** Check for the mandatory parameter which is misisng
	 */
	for (i = 0; i < LWE_CTRL_PARAM_MAX_IP_BIT_NO; i++) {
		if (((mndtr_param_map
		      [route_info.msg_type -
		       LWE_CTRL_CMD_TYPE_ROUTE_ADD] & (1 << i)) != 0)
		    && ((g_mndtr_param & (1 << i)) == 0)) {
			printf
			    ("Route Entry Operation failed as mandatory parameters missing; --%s or -%c\n",
			     route_add_options[i].name,
			     route_add_options[i].key);
			return;
		}
	}			/* end of for (i = 0; i < LWE_CTRL_PARAM_MAX_B .... */

copy:
	lwe_ipsec_copy_to_shared_memory(&route_info);

	return;
}

/**
 \brief Processes the PME Add/ Delete Request
 \param[in] argc Number of arguments
 \param[in] argv Arguments
 \param[in] type Message Type for the CP request - PME Add or delete
 \return none
 */
void lwe_pme_add_del_command(int argc, char **argv, char *type)
{
	unsigned int i = 0;
	struct lwe_ctrl_op_info pme_info;
	unsigned int mndtr_param_map[] = { LWE_CTRL_PME_ADD_MDTR_PARAM_MAP,
		LWE_CTRL_PME_DEL_MDTR_PARAM_MAP
	};
	struct argp *pme_argp[] = { &pme_add_argp, &pme_del_argp };

	lwe_ipsec_log_print("\r\nlwe_pme_add_del_command: Enter");

	/*
	 ** Initializing the route info structure
	 */
	memset(&pme_info, 0, sizeof(pme_info));
	pme_info.state = LWE_CTRL_CMD_STATE_IDLE;
	pme_info.msg_type = (unsigned int)type;
	pme_info.result = LWE_CTRL_RSLT_FAILURE;

	if (pme_info.msg_type == LWE_CTRL_CMD_TYPE_PME_ADD) {
		struct lwe_ctrl_spa_cfg_info *tmp_pme_info =
		    &pme_info.app_info.pme_info;

		/* 
		 ** Set the default values before 
		 ** extracting the values coming  
		 ** from the CLI command
		 */
		tmp_pme_info->set = LWE_CTRL_PME_DEF_SET;
		tmp_pme_info->subset = LWE_CTRL_PME_DEF_SUBSET;
		tmp_pme_info->flag = LWE_CTRL_PME_DEF_FLAG;
		tmp_pme_info->report_verbosity =
		    LWE_CTRL_PME_DEF_REPORT_VERBOSITY;
		tmp_pme_info->session_id = LWE_CTRL_PME_DEF_SESSION_ID;
		tmp_pme_info->input_buff_chunk =
		    LWE_CTRL_PME_DEF_INPUT_BUFF_CHUNK;
		tmp_pme_info->qos_in = LWE_CTRL_PME_DEF_QOS_IN;
		tmp_pme_info->qos_out = LWE_CTRL_PME_DEF_QOS_OUT;
		tmp_pme_info->match_limit = LWE_CTRL_PME_DEF_MATCH_LIMIT;
		tmp_pme_info->compare_limit = LWE_CTRL_PME_DEF_COMPARE_LIMIT;
		tmp_pme_info->output_buff_frm_app =
		    LWE_CTRL_PME_DEF_OUTPUT_BUFF_FRM_APP;
		tmp_pme_info->input_buff_size =
		    LWE_CTRL_PME_DEF_INPUT_BUFF_SIZE;
	}

	g_mndtr_param = 0;
	g_parse_error = 0;

	/* Where the magic happens */
	argp_parse(pme_argp[pme_info.msg_type - LWE_CTRL_CMD_TYPE_PME_ADD],
		   argc, argv, 0, 0, &pme_info);

	if (g_parse_error != 0)
		return;

	/*
	 ** If all the mandatory parameters for the operation are present,
	 ** Copy the data onto the shared memory area
	 */
	if ((g_mndtr_param &
	     mndtr_param_map[pme_info.msg_type -
			     LWE_CTRL_CMD_TYPE_ARP_ADD]) ==
	    mndtr_param_map[pme_info.msg_type - LWE_CTRL_CMD_TYPE_PME_ADD])
		goto copy;

	/*
	 ** Check for the mandatory parameter which is misisng
	 */
	for (i = 0; i < LWE_CTRL_PARAM_PME_MAX_BIT_NO; i++) {
		if (((mndtr_param_map
		      [pme_info.msg_type -
		       LWE_CTRL_CMD_TYPE_PME_ADD] & (1 << i)) != 0)
		    && ((g_mndtr_param & (1 << i)) == 0)) {
			printf
			    ("Route Entry Operation failed as mandatory parameters missing; --%s or -%c\n",
			     pme_add_options[i].name, pme_add_options[i].key);
			return;
		}
	}			/* end of for (i = 0; i < LWE_CTRL_PARAM_PME_MAX_B .... */

copy:

	lwe_ipsec_copy_to_shared_memory(&pme_info);

	lwe_ipsec_log_print("\r\nlwe_pme_add_del_command: Exit");
	return;
}

/**
 \brief Processes the ARP Add/ Delete Request
 \param[in] argc Number of arguments
 \param[in] argv Arguments
 \param[in] type Message Type for the CP request - ARP Add or delete
 \return none
 */
void lwe_arp_add_del_command(int argc, char **argv, char *type)
{
	unsigned int i = 0;
	struct lwe_ctrl_op_info route_info;
	unsigned int mndtr_param_map[] = { LWE_CTRL_ARP_ADD_MDTR_PARAM_MAP,
		LWE_CTRL_ARP_DEL_MDTR_PARAM_MAP
	};
	struct argp *route_argp[] = { &arp_add_argp, &arp_del_argp };

	lwe_ipsec_log_print("\r\nlwe_arp_add_del_command: Enter");
	/*
	 ** Initializing the route info structure
	 */
	memset(&route_info, 0, sizeof(route_info));
	route_info.state = LWE_CTRL_CMD_STATE_IDLE;
	route_info.msg_type = (unsigned int)type;
	route_info.result = LWE_CTRL_RSLT_FAILURE;

	g_mndtr_param = 0;

	if (route_info.msg_type == LWE_CTRL_CMD_TYPE_ARP_ADD) {
		route_info.app_info.ip_info.replace_entry = 0;
	}

	/* Where the magic happens */
	argp_parse(route_argp[route_info.msg_type - LWE_CTRL_CMD_TYPE_ARP_ADD],
		   argc, argv, 0, 0, &route_info);

	/*
	 ** If all the mandatory parameters for the operation are present,
	 ** Copy the data onto the shared memory area
	 */
	if ((g_mndtr_param &
	     mndtr_param_map[route_info.msg_type -
			     LWE_CTRL_CMD_TYPE_ARP_ADD]) ==
	    mndtr_param_map[route_info.msg_type - LWE_CTRL_CMD_TYPE_ARP_ADD])
		goto copy;

	/*
	 ** Check for the mandatory parameter which is misisng
	 */
	for (i = 0; i < LWE_CTRL_PARAM_ARP_MAX_BIT_NO; i++) {
		if (((mndtr_param_map
		      [route_info.msg_type -
		       LWE_CTRL_CMD_TYPE_ARP_ADD] & (1 << i)) != 0)
		    && ((g_mndtr_param & (1 << i)) == 0)) {
			printf
			    ("ARP Entry Operation failed as mandatory parameters missing; --%s or -%c\n",
			     arp_add_options[i].name, arp_add_options[i].key);
			return;
		}
	}			/* end of for (i = 0; i < LWE_CTRL_PARAM_MAX_B .... */

copy:
	lwe_ipsec_copy_to_shared_memory(&route_info);

	lwe_ipsec_log_print("\r\nlwe_arp_add_del_command: Exit");
	return;
}

/**
 \brief Processes the SA Add/ Delete Request
 \param[in] argc Number of arguments
 \param[in] argv Arguments
 \param[in] type Message Type for the CP request - SA Add or delete
 \return none
 */
void lwe_ipsec_add_del_command(int argc, char **argv, char *type)
{
	struct lwe_ctrl_op_info sa_info;

	sa_info.state = LWE_CTRL_CMD_STATE_IDLE;		/**< State of Command */
	sa_info.msg_type = (unsigned int)type;		/**<Type of Request>*/
	sa_info.result = LWE_CTRL_RSLT_FAILURE;		/**<Result - Successful, Failure>*/

	memset(&sa_info.app_info.ipsec_info.id, 0,
	       sizeof(struct lwe_ctrl_sa_id));
	memset(&sa_info.app_info.ipsec_info.sel, 0,
	       sizeof(struct lwe_ctrl_sa_selector));
	memset(&sa_info.app_info.ipsec_info.aalg, 0,
	       sizeof(struct lwe_ctrl_sa_algo));
	memset(&sa_info.app_info.ipsec_info.ealg, 0,
	       sizeof(struct lwe_ctrl_sa_algo));

	sa_info.app_info.ipsec_info.id.proto = IPSEC_PROTO_ESP;

	if (sa_info.msg_type == LWE_CTRL_CMD_TYPE_SAADD) {
		/* Where the magic happens */
		g_mndtr_param = 0;
		g_parse_error = 0;

		argp_parse(&add_argp, argc, argv, 0, 0, &sa_info);

		if (g_parse_error != 0)
			return;

		if ((g_mndtr_param & LWE_CTRL_ADD_MDTR_PARAM_MAP) !=
		    LWE_CTRL_ADD_MDTR_PARAM_MAP) {
			unsigned int i = 0;
			for (i = 0; i < LWE_CTRL_PARAM_MAX_IPSEC_BIT_NO; i++) {
				if (((LWE_CTRL_ADD_MDTR_PARAM_MAP & (1 << i)) !=
				     0)
				    && ((g_mndtr_param & (1 << i)) == 0)) {
					printf
					    ("SA Entry Addition failed as mandatory parameters missing; --%s or -%c\n",
					     add_options[i].name,
					     add_options[i].key);
					return;
				}
			}
		}

		/* Setting the default */
		if ((g_mndtr_param & LWE_CTRL_PARAM_BIT_EKEY) == 0) {
			if (sa_info.app_info.ipsec_info.ealg.alg_type ==
			    TRIP_DES_CBC) {
				memcpy(&sa_info.app_info.ipsec_info.
				       ealg.alg_key, def_tdes_enc_key, 24);
				sa_info.app_info.ipsec_info.ealg.alg_key_len =
				    24;

			} else {
				memcpy(&sa_info.app_info.ipsec_info.
				       ealg.alg_key, def_aes_enc_key, 16);
				sa_info.app_info.ipsec_info.ealg.alg_key_len =
				    16;
			}
		}

		/* Setting the default */
		if ((g_mndtr_param & LWE_CTRL_PARAM_BIT_AKEY) == 0) {
			memcpy(&sa_info.app_info.ipsec_info.aalg.alg_key,
			       def_auth_key, 20);
			sa_info.app_info.ipsec_info.aalg.alg_key_len = 20;
		}

	} else {
		g_mndtr_param = 0;
		argp_parse(&del_argp, argc, argv, 0, 0, &sa_info);
		if ((g_mndtr_param & LWE_CTRL_DEL_MDTR_PARAM_MAP) !=
		    LWE_CTRL_DEL_MDTR_PARAM_MAP) {
			unsigned int i = 0;
			for (i = 0; i < LWE_CTRL_PARAM_MAX_IPSEC_BIT_NO; i++) {
				if (((LWE_CTRL_DEL_MDTR_PARAM_MAP & (1 << i)) !=
				     0)
				    && ((g_mndtr_param & (1 << i)) == 0)) {
					printf
					    ("SA Entry Deletion failed as mandatory parameters missing; --%s or -%c\n",
					     add_options[i].name,
					     add_options[i].key);
					return;
				}
			}
		}
	}

	lwe_ipsec_copy_to_shared_memory(&sa_info);

	return;
}

/**
 \brief Processes the SA Stats Request
 \param[in] argc Number of arguments
 \param[in] argv Arguments
 \param[in] type Message Type for the CP request - SA Stats
 \return none
 */
void lwe_ipsec_stats_command(int argc, char **argv, char *type)
{
	struct lwe_ctrl_op_info sa_info;

	sa_info.state = LWE_CTRL_CMD_STATE_IDLE;		/**< State of Command */
	sa_info.msg_type = (unsigned int)type;		/**<Type of Request>*/
	sa_info.result = LWE_CTRL_RSLT_FAILURE;		/**<Result - Successful, Failure>*/

	memset(&sa_info.app_info.ipsec_info.id, 0,
	       sizeof(struct lwe_ctrl_sa_id));
	memset(&sa_info.app_info.ipsec_info.sel, 0,
	       sizeof(struct lwe_ctrl_sa_selector));
	memset(&sa_info.app_info.ipsec_info.aalg, 0,
	       sizeof(struct lwe_ctrl_sa_algo));
	memset(&sa_info.app_info.ipsec_info.ealg, 0,
	       sizeof(struct lwe_ctrl_sa_algo));

	sa_info.app_info.ipsec_info.id.proto = IPSEC_PROTO_ESP;

	if (sa_info.msg_type == LWE_CTRL_CMD_TYPE_SASTATS) {
		/* Where the magic happens */
		g_mndtr_param = 0;
		argp_parse(&sastats_argp, argc, argv, 0, 0, &sa_info);
		if ((g_mndtr_param & LWE_CTRL_SASTATS_MDTR_PARAM_MAP) !=
		    LWE_CTRL_SASTATS_MDTR_PARAM_MAP) {
			unsigned int i = 0;
			for (i = 0; i < LWE_CTRL_PARAM_MAX_IPSEC_BIT_NO; i++) {
				if (((LWE_CTRL_SASTATS_MDTR_PARAM_MAP &
				      (1 << i)) != 0)
				    && ((g_mndtr_param & (1 << i)) == 0)) {
					printf
					    ("SA Entry Get failed as mandatory parameters missing; --%s or -%c\n",
					     add_options[i].name,
					     add_options[i].key);
					return;
				}
			}
		}
	} else if (sa_info.msg_type == LWE_CTRL_CMD_TYPE_IFSTATS) {
		/* Where the magic happens */
		g_mndtr_param = 0;
		argp_parse(&ifstats_argp, argc, argv, 0, 0, &sa_info);
	}

	lwe_ipsec_copy_to_shared_memory(&sa_info);
	return;
}

/**
 \brief Processes the Edit Frame count Request
 \param[in] argc Number of arguments
 \param[in] argv Arguments
 \param[in] type Message Type for the CP request - Edit Frame Count 
 \return none
 */
void lwe_ip_edit_frame_cnt_command(int argc, char **argv, char *type)
{
	struct lwe_ctrl_op_info route_info;
	struct argp *route_argp[] = { &framecnt_argp };

	/*
	 ** Initializing the route info structure
	 */
	memset(&route_info, 0, sizeof(route_info));
	route_info.state = LWE_CTRL_CMD_STATE_IDLE;
	route_info.msg_type = (unsigned int)type;
	route_info.result = LWE_CTRL_RSLT_FAILURE;

	/* Where the magic happens */
	argp_parse(route_argp
		   [route_info.msg_type - LWE_CTRL_CMD_TYPE_FRAMECNT_EDIT], argc,
		   argv, 0, 0, &route_info);

	lwe_ipsec_copy_to_shared_memory(&route_info);

	return;
}

/**
 \brief Defines actions for parsing the lwe_ipsec command options - add/ delete/ sastats/ ifstats/ salist/ iflist; 
	it is called for each option parsed
 \param[in] key For each option that is parsed, parser is called with a value of 
		key from that option's key field in the option vector
 \param[in] arg If key is an option, arg is its given value.
 \param[in] state state points to a struct argp_state, containing pointer to sa_info structure
 \return 0 for success, ARGP_ERR_UNKNOWN if the value of key is not handled by this parser function
 */
static error_t parse_opt(int key, char *arg, struct argp_state *state)
{
	struct lwe_ctrl_op_info *sa_info = state->input;

	switch (key) {
	case 'A':
		sa_info->msg_type = LWE_CTRL_CMD_TYPE_SAADD;
		lwe_ipsec_log_print
		    ("\n::FILE: %s : LINE: %d :IN PARSE_OPT::ADD OPTION SELECTED",
		     __FILE__, __LINE__);
		break;

	case 'D':
		sa_info->msg_type = LWE_CTRL_CMD_TYPE_SADEL;
		lwe_ipsec_log_print
		    ("\nFILE: %s : LINE: %d :IN PARSE_OPT::DELETE OPTION SELECTED",
		     __FILE__, __LINE__);
		break;

	case 'S':
		sa_info->msg_type = LWE_CTRL_CMD_TYPE_SASTATS;
		lwe_ipsec_log_print
		    ("\nFILE: %s : LINE: %d :IN PARSE_OPT:: LIST OPTION SELECTED",
		     __FILE__, __LINE__);
		break;

	case 'I':
		sa_info->msg_type = LWE_CTRL_CMD_TYPE_IFSTATS;
		lwe_ipsec_log_print
		    ("\nFILE: %s : LINE: %d :IN PARSE_OPT:: LIST OPTION SELECTED",
		     __FILE__, __LINE__);
		break;

	case 'L':
		{
			char *list_type = "sa";
			if (strcmp(arg, list_type) == 0) {
				sa_info->msg_type = LWE_CTRL_CMD_TYPE_SALIST;
				lwe_ipsec_log_print
				    ("\nFILE: %s : LINE: %d :IN PARSE_OPT:: LIST SA SELECTED",
				     __FILE__, __LINE__);
			} else {
				sa_info->msg_type = LWE_CTRL_CMD_TYPE_IFLIST;
				lwe_ipsec_log_print
				    ("\nFILE: %s : LINE: %d :IN PARSE_OPT:: LIST IF SELECTED",
				     __FILE__, __LINE__);
			}
			break;
		}

		/* Request for Route Entry Addition */
	case 'B':
		sa_info->msg_type = LWE_CTRL_CMD_TYPE_ROUTE_ADD;
		lwe_ipsec_log_print
		    ("\n::FILE: %s : LINE: %d :IN PARSE_OPT::ADD OPTION SELECTED",
		     __FILE__, __LINE__);
		break;

		/* Request for route Entry Deletion */
	case 'C':
		sa_info->msg_type = LWE_CTRL_CMD_TYPE_ROUTE_DEL;
		lwe_ipsec_log_print
		    ("\nFILE: %s : LINE: %d :IN PARSE_OPT::DELETE OPTION SELECTED",
		     __FILE__, __LINE__);
		break;

		/* Request for route Entry stats */
	case 'R':
		sa_info->msg_type = LWE_CTRL_CMD_TYPE_ROUTE_STATS;
		lwe_ipsec_log_print
		    ("\nFILE: %s : LINE: %d :IN PARSE_OPT::ROUTE STATS OPTION SELECTED",
		     __FILE__, __LINE__);
		break;

	case 'F':
		sa_info->msg_type = LWE_CTRL_CMD_TYPE_INTF_CONF_CHNG;
		lwe_ipsec_log_print
		    ("\nFILE: %s : LINE: %d :IN PARSE_OPT::INTF CONF CHNG OPTION SELECTED",
		     __FILE__, __LINE__);
		break;

	case 'G':
		sa_info->msg_type = LWE_CTRL_CMD_TYPE_ARP_ADD;
		lwe_ipsec_log_print
		    ("\nFILE: %s : LINE: %d :IN PARSE_OPT::ARP ADD OPTION SELECTED",
		     __FILE__, __LINE__);
		break;

	case 'H':
		sa_info->msg_type = LWE_CTRL_CMD_TYPE_ARP_DEL;
		lwe_ipsec_log_print
		    ("\nFILE: %s : LINE: %d :IN PARSE_OPT::ARP DEL OPTION SELECTED",
		     __FILE__, __LINE__);
		break;
	case 'J':
		sa_info->msg_type = LWE_CTRL_CMD_TYPE_PME_ADD;
		lwe_ipsec_log_print
		    ("\nFILE: %s : LINE: %d :IN PARSE_OPT::PME ADD OPTION SELECTED",
		     __FILE__, __LINE__);
		break;

	case 'K':
		sa_info->msg_type = LWE_CTRL_CMD_TYPE_PME_DEL;
		lwe_ipsec_log_print
		    ("\nFILE: %s : LINE: %d :IN PARSE_OPT::PME DEL OPTION SELECTED",
		     __FILE__, __LINE__);
		break;

	case 'N':
		sa_info->msg_type = LWE_CTRL_CMD_TYPE_FRAMECNT_EDIT;
		lwe_ipsec_log_print
		    ("\nFILE: %s : LINE: %d :IN PARSE_OPT::FRAME COUNT EDIT OPTION SELECTED",
		     __FILE__, __LINE__);
		break;
	case 'P':
		sa_info->msg_type = LWE_CTRL_CMD_TYPE_RESET_STATS;
		lwe_ipsec_log_print
		    ("\nFILE: %s : LINE: %d :IN PARSE_OPT::RESETSTATS SELECTED",
		     __FILE__, __LINE__);
		break;
	default:
		return ARGP_ERR_UNKNOWN;
	}

	return ARGP_KEY_END;
}

/**
 * Parse a number from the string str.
 *
 * Parameters:
 * str - string containing the number
 * base - the base in which the number is represented
 * min - the minimum accepted value
 * max - the maximum accepted value
 * r - place to store the result
 *
 * Returns 0 on success, 1 on parse error.
 */
static int
parse_number(char *str, int base, unsigned long min,
	     unsigned long max, unsigned long *r)
{
	char *endptr;
	unsigned long n;

	errno = 0;
	n = strtoul(str, &endptr, base);
	if (n < min || n > max || errno == ERANGE || endptr == str
	    || *endptr != 0)
		return 1;

	*r = n;

	return 0;
}

/**
 \brief The main function. The only function call needed to process
   	all command-line options and arguments nicely is argp_parse.
  \param[in] argc
  \param[in] argv
*/
int main(int argc, char **argv)
{
	struct lwe_ctrl_op_info sa_info;
	void *where;
	int fd;
	char *tmp_argv = "-E";
	char *tmp2_argv = "-O";
	char *tmp3_argv = "-T";
	char *tmp4_argv = "-P";

	fd = open("/dev/mem", O_RDWR);
	if (fd < 0) {
		printf("Failed to open\n");
		return 0;
	}

	where = mmap(0,
		     LWE_CTRL_SHM_SIZE,
		     PROT_READ | PROT_WRITE, MAP_SHARED, fd, LWE_CTRL_SHM_ADDR);
	if (where == (void *)0xffffffff) {
		close(fd);
		printf("mmap failed\n");
		return -1;
	}

	g_p_shm_sa_Info = (struct lwe_ctrl_op_info *)where;

	memset(&sa_info, 0, sizeof(struct lwe_ctrl_op_info));

	sa_info.state = LWE_CTRL_CMD_STATE_IDLE;		/**< State of Command */
	sa_info.msg_type = LWE_CTRL_CMD_TYPE_IFSTATS;		/**<Type of Request>*/
	sa_info.result = LWE_CTRL_RSLT_FAILURE;		/**<Result - Successful, Failure>*/

	if (argc == 1) {
		printf("Mandatory Parameter missing\n");
		printf("Try `ipsecfwd_hybrid --help' for more information\n");
		goto _close;
	}

	if (strcmp(argv[1], tmp_argv) == 0) {
		sa_info.msg_type = LWE_CTRL_CMD_TYPE_IPFWD_SYS_STATS;
		lwe_ipsec_log_print
		    ("\nFILE: %s : LINE %d : IN MAIN : THE TYPE PROVIDED FOR SYS STATS OPTION IS : %d",
		     __FILE__, __LINE__, sa_info.msg_type);
		lwe_ip_stats_command(argc - 1, &argv[1],
				     (char *)sa_info.msg_type);
		goto _close;
	}

	if (strcmp(argv[1], tmp2_argv) == 0) {
		sa_info.msg_type = LWE_CTRL_CMD_TYPE_GO;
		lwe_ipsec_log_print
		    ("\nFILE:%s :LINE %d:IN MAIN:TYPE PROVIDED FOR GO OPT: %d",
		     __FILE__, __LINE__, sa_info.msg_type);
		lwe_ip_stats_command(argc - 1, &argv[1],
				     (char *)sa_info.msg_type);
		goto _close;
	}


	if (strcmp(argv[1], tmp3_argv) == 0) {
		sa_info.msg_type = LWE_CTRL_CMD_TYPE_RESET;
		lwe_ipsec_log_print
		    ("\nFILE:%s:LINE %d:IN MAIN:TYPE PROV FOR RESET OPT: %d",
		     __FILE__, __LINE__, sa_info.msg_type);
		lwe_ip_stats_command(argc - 1, &argv[1],
				     (char *)sa_info.msg_type);
		goto _close;
	}

	if (strcmp(argv[1], tmp4_argv) == 0) {
		sa_info.msg_type = LWE_CTRL_CMD_TYPE_RESET_STATS;
		lwe_ipsec_log_print
		    ("\nFILE:%s:LINE %d:IN MAIN:TYPE FOR RESETSTATS OPT IS:%d",
		     __FILE__, __LINE__, sa_info.msg_type);
		lwe_ip_stats_command(argc - 1, &argv[1],
				     (char *)sa_info.msg_type);
		goto _close;
	}

	/* Where the magic happens */
	argp_parse(&argp, argc, argv, 0, 0, &sa_info);

	switch (sa_info.msg_type) {
	case LWE_CTRL_CMD_TYPE_SAADD:
		{
			lwe_ipsec_log_print
			    ("\nFILE: %s : LINE %d : IN MAIN : ADD OPTION SELECTED",
			     __FILE__, __LINE__);
			lwe_ipsec_log_print
			    ("\nFILE: %s : LINE %d : IN MAIN : THE TYPE PROVIDED FOR ADD OPTION IS : %d",
			     __FILE__, __LINE__, sa_info.msg_type);
			lwe_ipsec_add_del_command(argc - 1, &argv[1],
						  (char *)sa_info.msg_type);
		}
		break;

	case LWE_CTRL_CMD_TYPE_SADEL:
		{
			lwe_ipsec_log_print
			    ("\nFILE: %s : LINE %d : IN MAIN : DELETE OPTION SELECTED",
			     __FILE__, __LINE__);
			lwe_ipsec_log_print
			    ("\nFILE: %s : LINE %d : IN MAIN : THE TYPE PROVIDED FOR ADD OPTION IS : %d",
			     __FILE__, __LINE__, sa_info.msg_type);
			lwe_ipsec_add_del_command(argc - 1, &argv[1],
						  (char *)sa_info.msg_type);
		}
		break;

	case LWE_CTRL_CMD_TYPE_SASTATS:
	case LWE_CTRL_CMD_TYPE_IFSTATS:
		{
			lwe_ipsec_log_print
			    ("\nFILE : %s :LINE %d : IN MAIN : LIST OPTION SELECTED::",
			     __FILE__, __LINE__);
			lwe_ipsec_stats_command(argc - 1, &argv[1],
						(char *)sa_info.msg_type);
		}
		break;

	case LWE_CTRL_CMD_TYPE_SALIST:
	case LWE_CTRL_CMD_TYPE_IFLIST:
		{
			lwe_ipsec_copy_to_shared_memory(&sa_info);
		}
		break;
	case LWE_CTRL_CMD_TYPE_ROUTE_ADD:
		{
			lwe_ipsec_log_print
			    ("\nFILE: %s : LINE %d : IN MAIN : THE TYPE PROVIDED FOR ADD OPTION IS : %d",
			     __FILE__, __LINE__, sa_info.msg_type);
			lwe_ip_add_del_command(argc - 1, &argv[1],
					       (char *)sa_info.msg_type);
		}
		break;

	case LWE_CTRL_CMD_TYPE_ROUTE_DEL:
		{
			lwe_ipsec_log_print
			    ("\nFILE: %s : LINE %d : IN MAIN : THE TYPE PROVIDED FOR DELETE OPTION IS : %d",
			     __FILE__, __LINE__, sa_info.msg_type);
			lwe_ip_add_del_command(argc - 1, &argv[1],
					       (char *)sa_info.msg_type);
		}
		break;

	case LWE_CTRL_CMD_TYPE_ROUTE_STATS:
		{
			lwe_ipsec_log_print
			    ("\nFILE: %s : LINE %d : IN MAIN : THE TYPE PROVIDED FOR STATS OPTION IS : %d",
			     __FILE__, __LINE__, sa_info.msg_type);
			lwe_ip_stats_command(argc - 1, &argv[1],
					     (char *)sa_info.msg_type);
		}
		break;

	case LWE_CTRL_CMD_TYPE_INTF_CONF_CHNG:
		{
			lwe_ipsec_log_print
			    ("\nFILE: %s : LINE %d : IN MAIN : THE TYPE PROVIDED FOR INTF CONF OPTION IS : %d",
			     __FILE__, __LINE__, sa_info.msg_type);
			lwe_ip_intf_chng_command(argc - 1, &argv[1],
						 (char *)sa_info.msg_type);
		}
		break;

	case LWE_CTRL_CMD_TYPE_ARP_ADD:
		{
			lwe_ipsec_log_print
			    ("\nFILE: %s : LINE %d : IN MAIN : THE TYPE PROVIDED FOR ADD OPTION IS : %d",
			     __FILE__, __LINE__, sa_info.msg_type);
			lwe_arp_add_del_command(argc - 1, &argv[1],
						(char *)sa_info.msg_type);
		}
		break;

	case LWE_CTRL_CMD_TYPE_ARP_DEL:
		{
			lwe_ipsec_log_print
			    ("\nFILE: %s : LINE %d : IN MAIN : THE TYPE PROVIDED FOR DELETE OPTION IS : %d",
			     __FILE__, __LINE__, sa_info.msg_type);
			lwe_arp_add_del_command(argc - 1, &argv[1],
						(char *)sa_info.msg_type);
		}
		break;

	case LWE_CTRL_CMD_TYPE_PME_ADD:
		{
			lwe_ipsec_log_print
			    ("\nFILE: %s : LINE %d : IN MAIN : THE TYPE PROVIDED FOR ADD OPTION IS : %d",
			     __FILE__, __LINE__, sa_info.msg_type);
			lwe_pme_add_del_command(argc - 1, &argv[1],
						(char *)sa_info.msg_type);
		}
		break;

	case LWE_CTRL_CMD_TYPE_PME_DEL:
		{
			lwe_ipsec_log_print
			    ("\nFILE: %s : LINE %d : IN MAIN : THE TYPE PROVIDED FOR DELETE OPTION IS : %d",
			     __FILE__, __LINE__, sa_info.msg_type);
			lwe_pme_add_del_command(argc - 1, &argv[1],
						(char *)sa_info.msg_type);
		}
		break;

	case LWE_CTRL_CMD_TYPE_FRAMECNT_EDIT:
		{
			lwe_ipsec_log_print
			    ("\nFILE: %s : LINE %d : IN MAIN : THE TYPE PROVIDED FOR EDIT OPTION IS : %d",
			     __FILE__, __LINE__, sa_info.msg_type);
			lwe_ip_edit_frame_cnt_command(argc - 1, &argv[1],
						(char *)sa_info.msg_type);
		}
		break;

	default:
		printf("Invalid Option\n");
	}

_close:
	munmap(where, LWE_CTRL_SHM_SIZE);
	close(fd);

	return 0;
}
