/** 
 \file lwe_ipsec.h
 \brief Basic LWE IPsec Config Tool defines and Data structures
 */
/*
 * Copyright (C) 2008-2010 Freescale Semiconductor, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 * NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

union stat64_t {
	volatile uint64_t stat;
	struct {
		volatile uint32_t hi;
		volatile uint32_t lo;
	} words;
};

const char *argp_program_version = "argex 1.0";

#define LWE_CTRL_PARAM_BIT_SRCIP	(1 << 0)	/**< Bit Mask for Src IP */
#define LWE_CTRL_PARAM_BIT_DESTIP	(1 << 1)	/**< Bit Mask for Dst IP */
#define LWE_CTRL_PARAM_BIT_SRCGW	(1 << 2)	/**< Bit Mask for Tunnel Src IP */
#define LWE_CTRL_PARAM_BIT_DESTGW	(1 << 3)	/**< Bit Mask for Tunnel Dst IP */
#define LWE_CTRL_PARAM_BIT_PROTO	(1 << 4)	/**< Bit Mask for Protocol */
#define LWE_CTRL_PARAM_BIT_EKEY	(1 << 5)	/**< Bit Mask for Encryption Key */
#define LWE_CTRL_PARAM_BIT_AKEY	(1 << 6)	/**< Bit Mask for Authentication Key */
#define LWE_CTRL_PARAM_BIT_SPI	(1 << 7)	/**< Bit Mask for Security Policy Index */
#define LWE_CTRL_PARAM_BIT_DIR	(1 << 8)	/**< Bit Mask for Direction */
#define LWE_CTRL_PARAM_BIT_ETYPE	(1 << 9)	/**< Bit Mask for Encryption type */
#define LWE_CTRL_PARAM_BIT_ATYPE	(1 << 10)	/**< Bit Mask for Authentication type */
#define LWE_CTRL_PARAM_BIT_FLOWID       (1 << 11)
#define LWE_CTRL_PARAM_MAX_IPSEC_BIT_NO	12

/**< Mandatory Parameters needed for creating SA */
#define LWE_CTRL_ADD_MDTR_PARAM_MAP (LWE_CTRL_PARAM_BIT_SRCIP | \
				LWE_CTRL_PARAM_BIT_DESTIP | \
				LWE_CTRL_PARAM_BIT_SRCGW | \
				LWE_CTRL_PARAM_BIT_DESTGW | \
				LWE_CTRL_PARAM_BIT_DIR | \
				LWE_CTRL_PARAM_BIT_SPI )

#define LWE_CTRL_SA_IPSEC_PROTO_MIN	0
#define LWE_CTRL_SA_IPSEC_PROTO_MAX	0
#define LWE_CTRL_SA_IPSEC_PROTO_DEF	0

#define LWE_CTRL_SA_ENCRIPTN_TYPE_MIN	0
#define LWE_CTRL_SA_ENCRIPTN_TYPE_MAX	1
#define LWE_CTRL_SA_ENCRIPTN_TYPE_DEF	0

#define LWE_CTRL_SA_AUTH_TYPE_MIN		0
#define LWE_CTRL_SA_AUTH_TYPE_MAX		0
#define LWE_CTRL_SA_AUTH_TYPE_DEF		0

static struct argp_option add_options[] = {
	{"s", 's', "SRCIP", 0, "Source IP", 0},
	{"d", 'd', "DESTIP", 0, "Destination IP", 0},
	{"ss", 'g', "SRCGW", 0, "Source gateway IP", 0},
	{"sd", 'G', "DESTGW", 0, "Destination gateway IP", 0},
	{"p", 'p', "PROTO", 0, "IPsec Proto type - ESP(0) {Default: 0}", 0},
	{"t", 't', "ETYPE", 0,
	 "Encryption Type - AES-CBC(0), 3DES-CBC(1) {Default: 0}", 0},
	{"y", 'y', "ATYPE", 0,
	 "Authentication Type - HMAC-SHA1(0) {Default: 0}", 0},
	{"e", 'e', "EKEY", 0, "Encryption Key", 0},
	{"a", 'a', "AKEY", 0, "Authentication Key", 0},
	{"spi", 'i', "SPI", 0, "SPI - 32 bit unsigned int", 0},
	{"dir", 'r', "DIR", 0, "DIR- in/ out", 0},
	{"f", 'f', "FLOWID", 0, "Flow ID - (0 - 64) {Default: 0}", 0},
	{0, 0, 0, 0, 0, 0}
};

/**< Mandatory Parameters needed for deleting SA */
#define LWE_CTRL_DEL_MDTR_PARAM_MAP (LWE_CTRL_PARAM_BIT_SRCIP | \
				LWE_CTRL_PARAM_BIT_DESTIP | \
				LWE_CTRL_PARAM_BIT_SRCGW | \
				LWE_CTRL_PARAM_BIT_DESTGW | \
				LWE_CTRL_PARAM_BIT_SPI )

static struct argp_option del_options[] = {
	{"s", 's', "SRCIP", 0, "Source IP", 0},
	{"d", 'd', "DESTIP", 0, "Destination IP", 0},
	{"ss", 'g', "SRCGW", 0, "Source gateway IP", 0},
	{"sd", 'G', "DESTGW", 0, "Destination gateway IP", 0},
	{"spi", 'i', "SPI", 0, "SPI", 0},
	{0, 0, 0, 0, 0, 0}
};

/**< Mandatory Parameters needed for getting SA Stats */
#define LWE_CTRL_SASTATS_MDTR_PARAM_MAP (LWE_CTRL_PARAM_BIT_SRCIP | \
				LWE_CTRL_PARAM_BIT_DESTIP | \
				LWE_CTRL_PARAM_BIT_SRCGW | \
				LWE_CTRL_PARAM_BIT_DESTGW | \
				LWE_CTRL_PARAM_BIT_SPI )

#define LWE_CTRL_IFSTATS_MDTR_PARAM_MAP 0

static struct argp_option sastats_options[] = {
	{"s", 's', "SRCIP", 0, "Source IP", 0},
	{"d", 'd', "DESTIP", 0, "Destination IP", 0},
	{"ss", 'g', "SRCGW", 0, "Source gateway IP", 0},
	{"sd", 'G', "DESTGW", 0, "Destination gateway IP", 0},
	{"spi", 'i', "SPI", 0, "SPI", 0},
	{"dir", 'r', "DIR", 0, "DIR- in/ out", 0},
	{0, 0, 0, 0, 0, 0}
};

static struct argp_option ifstats_options[] = {
	{0, 0, 0, 0, 0, 0}
};

#define LWE_CTRL_PARAM_BMASK_SRCIP		(1 << 0)	/**< Bit Mask for Src IP */
#define LWE_CTRL_PARAM_BMASK_DESTIP	(1 << 1)	/**< Bit Mask for Dst IP */
#define LWE_CTRL_PARAM_BMASK_GWIP		(1 << 2)	/**< Bit Mask for Gateway IP */
#define LWE_CTRL_PARAM_BMASK_TOS		(1 << 3)	/**< Bit Mask for TOS */
#define LWE_CTRL_PARAM_BMASK_FLOWID	(1 << 4)	/**< Bit Mask for Flow ID */
#define LWE_CTRL_PARAM_MAX_IP_BIT_NO		5

#define LWE_CTRL_ROUTE_TOS_MIN 					0
#define LWE_CTRL_ROUTE_TOS_MAX 					256
#define LWE_CTRL_ROUTE_TOS_DEF					0

#define LWE_CTRL_ROUTE_FLOWID_MIN 					0
#define LWE_CTRL_ROUTE_FLOWID_MAX					1024
#define LWE_CTRL_ROUTE_FLOWID_DEF					0

/**< Mandatory Parameters needed for creating Route */
#define LWE_CTRL_ROUTE_ADD_MDTR_PARAM_MAP (LWE_CTRL_PARAM_BMASK_SRCIP | \
				LWE_CTRL_PARAM_BMASK_DESTIP | \
				LWE_CTRL_PARAM_BMASK_GWIP )

static struct argp_option route_add_options[] = {
	{"s", 's', "SRCIP", 0, "Source IP", 0},
	{"d", 'd', "DESTIP", 0, "Destination IP", 0},
	{"g", 'g', "GWIP", 0, "Gateway IP", 0},
	{"t", 't', "TOS", 0, "Type of Service - (0 - 256) {Default: 0}", 0},
	{"f", 'f', "FLOWID", 0, "Flow ID - (0 - 64) {Default: 0}", 0},
	{0, 0, 0, 0, 0, 0}
};

/**< Mandatory Parameters needed for deleting Route */
#define LWE_CTRL_ROUTE_DEL_MDTR_PARAM_MAP (LWE_CTRL_PARAM_BMASK_SRCIP | \
				LWE_CTRL_PARAM_BMASK_DESTIP )

static struct argp_option route_del_options[] = {
	{"s", 's', "SRCIP", 0, "Source IP", 0},
	{"d", 'd', "DESTIP", 0, "Destination IP", 0},
	{"t", 't', "TOS", 0, "Type of Service - (0 - 256) {Default: 0}", 0},
	{0, 0, 0, 0, 0, 0}
};

/**< Mandatory Parameters needed for getting Route Stats */
#define LWE_CTRL_ROUTE_STATS_MDTR_PARAM_MAP (LWE_CTRL_PARAM_BMASK_SRCIP | \
				LWE_CTRL_PARAM_BMASK_DESTIP )

static struct argp_option route_stats_options[] = {
	{"s", 's', "SRCIP", 0, "Source IP", 0},
	{"d", 'd', "DESTIP", 0, "Destination IP", 0},
	{"t", 't', "TOS", 0, "Type of Service - (0 - 256) {Default: 0}", 0},
	{0, 0, 0, 0, 0, 0}
};

static struct argp_option intf_conf_options[] = {
	{"i", 'i', "IFNAME", 0, "If Name", 0},
	{"a", 'a', "IPADDR", 0, "IP Address", 0},
	{"m", 'm', "MACADDR", 0, "MAC Address", 0},
	{"t", 't', "MTU", 0, "MTU", 0},
	{"e", 'e', "ENABLE", 0, "Admin Status - true/ false", 0},
	{0, 0, 0, 0, 0, 0}
};

#define LWE_CTRL_PARAM_BMASK_ARP_IPADDR		(1 << 0)	/**< Bit Mask for ARP IP Address */
#define LWE_CTRL_PARAM_BMASK_ARP_MACADDR		(1 << 1)	/**< Bit Mask for MAC Address */
#define LWE_CTRL_PARAM_BMASK_ARP_REPLACE		(1 << 2)	/**< Bit Mask for Replace variable */
#define LWE_CTRL_PARAM_ARP_MAX_BIT_NO		3

/**< Mandatory Parameters needed for creating ARP */
#define LWE_CTRL_ARP_ADD_MDTR_PARAM_MAP (LWE_CTRL_PARAM_BMASK_ARP_IPADDR | \
				LWE_CTRL_PARAM_BMASK_ARP_MACADDR )

static struct argp_option arp_add_options[] = {
	{"s", 's', "IPADDR", 0, "IP Address", 0},
	{"m", 'm', "MACADDR", 0, "MAC Address", 0},
	{"r", 'r', "Replace", 0,
	 "Replace Exiting Entry - true/ false {Default: false}", 0},
	{0, 0, 0, 0, 0, 0}
};

/**< Mandatory Parameters needed for deleting ARP */
#define LWE_CTRL_ARP_DEL_MDTR_PARAM_MAP (LWE_CTRL_PARAM_BMASK_ARP_IPADDR )

static struct argp_option arp_del_options[] = {
	{"s", 's', "IPADDR", 0, "IP Address", 0},
	{0, 0, 0, 0, 0, 0}
};

#define LWE_CTRL_PARAM_BMASK_PME_FLOWID			(1 << 0)	/**< Bit Mask for Flow ID */
#define LWE_CTRL_PARAM_BMASK_PME_SET			(1 << 1)	/**< Bit Mask for Pattern Set */
#define LWE_CTRL_PARAM_BMASK_PME_SUBSET			(1 << 2)	/**< Bit Mask for Pattern Subset */
#define LWE_CTRL_PARAM_BMASK_PME_FLAG			(1 << 3)	/**< Bit Mask for Flag - residue */
#define LWE_CTRL_PARAM_BMASK_PME_REPORT_VERBOSITY	(1 << 6)	/**< Bit Mask for Report Verbosity */
#define LWE_CTRL_PARAM_BMASK_PME_SESSION_ID		(1 << 7)	/**< Bit Mask for session ID */
#define LWE_CTRL_PARAM_BMASK_PME_INPUT_BUFF_CHUNK	(1 << 8)	/**< Bit Mask for Input Chunk Size  */
#define LWE_CTRL_PARAM_BMASK_PME_QOS_IN			(1 << 9)	/**< Bit Mask for QOS In */
#define LWE_CTRL_PARAM_BMASK_PME_QOS_OUT			(1 << 10)	/**< Bit Mask for QOS Out */
#define LWE_CTRL_PARAM_BMASK_PME_MATCH_LIMIT		(1 << 11)	/**< Bit Mask for Match Limit */
#define LWE_CTRL_PARAM_BMASK_PME_COMPARE_LIMIT		(1 << 12)	/**< Bit Mask for Compare Limit */
#define LWE_CTRL_PARAM_BMASK_PME_OUTPUT_BUFF_FRM_APP	(1 << 13)	/**< Bit Mask for Output Buffer from App */
#define LWE_CTRL_PARAM_BMASK_PME_INPUT_BUFF_SIZE		(1 << 14)	/**< Bit Mask for Input Buffer Size */
#define LWE_CTRL_PARAM_BMASK_PME_INPUT_FILE		(1 << 15)	/**< Bit Mask for Input File */

#define LWE_CTRL_PARAM_PME_MAX_BIT_NO		16

#define LWE_CTRL_PME_FLOWID_MIN 					1
#define LWE_CTRL_PME_FLOWID_MAX						1024
#define LWE_CTRL_PME_SET_MIN 					0
#define LWE_CTRL_PME_SET_MAX 					0xff
#define LWE_CTRL_PME_DEF_SET					0

#define LWE_CTRL_PME_SUBSET_MIN 					0
#define LWE_CTRL_PME_SUBSET_MAX 					0xffff
#define LWE_CTRL_PME_DEF_SUBSET					0xffff

#define LWE_CTRL_PME_DEF_FLAG					0

#define LWE_CTRL_PME_REPORT_VERBOSITY_MIN				0
#define LWE_CTRL_PME_REPORT_VERBOSITY_MAX				4
#define LWE_CTRL_PME_DEF_REPORT_VERBOSITY				4

#define LWE_CTRL_PME_SESSION_ID_MIN				0
#define LWE_CTRL_PME_SESSION_ID_MAX				0x7ffffff
#define LWE_CTRL_PME_DEF_SESSION_ID				0

#define LWE_CTRL_PME_INPUT_BUFF_CHUNK_MIN				0
#define LWE_CTRL_PME_INPUT_BUFF_CHUNK_MAX				1600
#define LWE_CTRL_PME_DEF_INPUT_BUFF_CHUNK				0

#define LWE_CTRL_PME_QOS_IN_MIN					0
#define LWE_CTRL_PME_QOS_IN_MAX					7
#define LWE_CTRL_PME_DEF_QOS_IN					2

#define LWE_CTRL_PME_QOS_OUT_MIN					0
#define LWE_CTRL_PME_QOS_OUT_MAX					7
#define LWE_CTRL_PME_DEF_QOS_OUT					0

#define LWE_CTRL_PME_MATCH_LIMIT_MIN				0
#define LWE_CTRL_PME_MATCH_LIMIT_MAX				0xffff
#define LWE_CTRL_PME_DEF_MATCH_LIMIT				0xffff

#define LWE_CTRL_PME_COMPARE_LIMIT_MIN				0
#define LWE_CTRL_PME_COMPARE_LIMIT_MAX				0xffff
#define LWE_CTRL_PME_DEF_COMPARE_LIMIT				0xffff

#define LWE_CTRL_PME_OUTPUT_BUFF_FRM_APP_MIN			0
#define LWE_CTRL_PME_OUTPUT_BUFF_FRM_APP_MAX			1
#define LWE_CTRL_PME_DEF_OUTPUT_BUFF_FRM_APP			1

#define LWE_CTRL_PME_INPUT_BUFF_SIZE_MIN				0
#define LWE_CTRL_PME_INPUT_BUFF_SIZE_MAX				1600
#define LWE_CTRL_PME_DEF_INPUT_BUFF_SIZE				100

/**< Mandatory Parameters needed for creating PME Flow */
#define LWE_CTRL_PME_ADD_MDTR_PARAM_MAP (LWE_CTRL_PARAM_BMASK_PME_FLOWID)

static struct argp_option pme_add_options[] = {
	{"f", 'f', "FLOWID", 0, "Flow ID - (1 - 64)", 0},
	{"s", 's', "SET", 0, "Set - {Default: 0}", 0},
	{"u", 'u', "SUBSET", 0,
	 "Subset - 16 bit Bitmask representing 16 subsets - {Default: 65535}",
	 0},
	{"m", 'm', "SCANMODE", 0,
	 "Scan Mode - Direct (0) or Flow (1) - {Default: 0}", 0},
	{"r", 'r', "RESIDUEENBL", 0,
	 "Residue - Disable (0) or Enable (1) - {Default: 0}", 0},
	{"n", 'n', "SCANFNTYPE", 0,
	 "Scan Fn Type - Non-Block (0) or Block (1) - {Default: 0}",
	 0},
	{"p", 'p', "REPRTVRBSTY", 0, "Report Verbosity - {Default: 0}", 0},
	{"i", 'i', "SESSID", 0, "Session Id - {Default: 0}", 0},
	{"b", 'b', "INPUTBUFFCHNK", 0, "Input Buff Chunk - {Default: 0}", 0},
	{"q", 'q', "QOSIN", 0, "Qos In - {Default: 0}", 0},
	{"t", 't', "QOSOUT", 0, "Qos Out - {Default: 0}", 0},
	{"l", 'l', "MATCHLMT", 0, "Match Limit - {Default: 65535}", 0},
	{"o", 'o', "CMPRLMT", 0, "Compare Limit - {Default: 65535}", 0},
	{"e", 'e', "OUTPUTBUFFENBL - {Default: 1}", 0,
	 "Output Buff Allocation from PME (0) or App (1)", 0},
	{"z", 'z', "INPUTBUFFSZ", 0, "Input Buff Size", 0},
	{"d", 'd', "INPUTFILE", 0, "Input File to be scanned", 0},
	{0, 0, 0, 0, 0, 0}
};

/**< Mandatory Parameters needed for deleting PME Flow */
#define LWE_CTRL_PME_DEL_MDTR_PARAM_MAP (LWE_CTRL_PARAM_BMASK_PME_FLOWID )

static struct argp_option pme_del_options[] = {
	{"f", 'f', "FLOWID", 0, "Flow ID", 0},
	{0, 0, 0, 0, 0, 0}
};

static struct argp_option framecnt_edit_options[] = {
	{"n", 'n', "FRAMECNT", 0, "Frame Count", 0},
	{0, 0, 0, 0, 0, 0}
};

static struct argp_option reset_stats_options[] = {
	{"s", 's', "IPFWDSYSSTATS", 0, "Ipfwd system stats", 0},
	{"r", 'r', "ROUTESTATS", 0, "Ipfwd route stats", 0},
	{"e", 'e', "SASTATS", 0, "SA stats", 0},
	{"i", 'i', "IFSTATS", 0, "Interface stats", 0},
	{"p", 'p', "PMESTATS", 0, "PME stats", 0},
	{"a", 'a', "ALLSTATS", 0, "All stats", 0},
	{0, 0, 0, 0, 0, 0}
};
