#!/bin/bash
# Copyright (C) 2009 Freescale Semiconductor, Inc.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
# NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# TODO: 
# 1) Parse command line arguments using getopt
# 2) Use socat instead of xterm

print_usage_and_exit()
{
	echo "This is the script that automatically loads up the IPSecFwd_Hybrid LWE application"
	echo "on a given P4080 board. Currently, the script that automates traffic generation"
	echo "with the help of SmartBits is under debug and this functionality is commented out."
	echo -e "\nThis script accepts the following positional arguments:\n"
	echo "\$1 : The IP address of the SmartBits hub (use 10.232.134.209 for IDC hub)"
	echo "\$2 : The path to the 'expect' utility"
	echo "\$3 : The string of port numbers of the mux_server that opens byte channels to the"
	echo "     selected board. Note that the board must be ON and the HV> prompt must display"
	echo "     on the first byte channel for this script to work correctly. Also note that"
	echo "     this script is currently hardcoded to open 3 application consoles."
	echo "\$4 : The frame count with which the application will be initialized with."
	echo "\$5 : The name of the route setup script to run (ex. ipsecfwd_hybrid_fwd_5_script.sh)"
	echp "\$6 : The value of core_mask to use for this test"
	exit 1
}

if [ "$#" -ne "6" ]; then print_usage_and_exit; fi

host=$1
expect=$2
mux_port_string=$3
port=`echo $3 | tr -s " " | cut -d' ' -f1`
frame_count=$4
script=$5
coremask=$6

if [ ! -f hv_reset.exp ] ; then echo "Required file hv_reset.expect not found in the current path"; exit; fi
if [ ! -f ipsecfwd_hybrid.exp ] ; then echo "Required file ipsecfwd_hybrid.expect not found in the current path"; exit; fi
if [ ! -f exit.exp ] ; then echo "Required file exit.expect not found in the current path"; exit; fi
if [ ! -f "$script" ] ; then echo "Required script $script not found in current path"; exit; fi

# reset the board (assuming that mux_server is running)
$expect	hv_reset.exp "$host" "$port" &
spawned_procs=`echo "$!"`

sleep 10

# start Linux 
port=$((port+1))
$expect ipsecfwd_hybrid.exp "$host" "$port" "$frame_count" "$script" "$coremask" &
cp_initialization_pid="$!"

# start loading the application consoles
# TODO: Enable opening of variable number of app consoles
echo "starting the application consoles"
port=$((port+1))
xterm -exec "$expect" exit.exp "$host" "$port" "ipsecfwd_hybrid.$host.$port.$frame_count.$script.log" &
app_console_pids=`echo "$!"`

port=$((port+1))
xterm -exec "$expect" exit.exp "$host" "$port" "ipsecfwd_hybrid.$host.$port.$frame_count.$script.log" &
app_console_pids=`echo "$app_console_pids" "$!"`

port=$((port+1))
xterm -exec "$expect" exit.exp "$host" "$port" "ipsecfwd_hybrid.$host.$port.$frame_count.$script.log" &
app_console_pids=`echo "$app_console_pids" "$!"`
echo "done starting the application consoles"


# pump traffic as soon as the CP initialization is complete
while :
do
	kill -0 "$cp_initialization_pid" 1>/dev/null 2>/dev/null
	if [ "$?" -eq "1" ]
	then
		echo "start pumping traffic"
		tclsh pumpt.tcl "$script"
		break
	fi
done

# cleanup
for i in `echo $spawned_procs $app_console_pids`
do
	kill $i 1>/dev/null 2>/dev/null
done

# scan the log files for test passed and report accordingly
# TODO: This requires revisiting
#for i in `ls ipsecfwd_hybrid.$host.*.log`
#do
#	grep -i pass "$i" 1>/dev/null 2>/dev/null
#	if [ "$?" -eq "0" ]
#	then
#		echo "Test PASSED"
#		exit 0
#	fi
#done

#echo "Test FAILED"
#exit 3

