# Copyright (C) 2009 Freescale Semiconductor, Inc.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
# NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# Aruments: <route_setup_script>
# TODO:
# 1) Enable 4 flows (currently only 2 are enabled)
# 2) Pick up MAC-address to IP-address mapping from separate text file, rather
#    than hardcoding it in the script.
# 3) Make the slot(s), port(s) and IP address of the SMB hub configurable.
# 4) Allow specification of traffic rate (rather than determining the throughput).
# 5) Allow configurable location of the SmartLib TCL script.

#  source smartlib.tcl

# If smartlib.tcl is not loaded, attempt to locate it at the current directory.
# The actual location is different on different platforms.
if  {$tcl_platform(platform) == "windows"} {
   set libPath "c:/Program Files/SmartBits/SmartLib/tcl/tclfiles/smartlib.tcl"
} else {
   set libPath "./smartlib.tcl"
}
set ::LD_LIBRARY_PATH "c:/Program Files/SmartBits/SmartLib/tcl/tclfiles/"
# if "smartlib.tcl" is not loaded, try to source it from the default path
if { ! [info exists __SMARTLIB_TCL__] } {
   if {[file exists $libPath]} {
      source $libPath
   } else {   
      puts "Could not find the file $libPath."
      exit
   }
}

global hub
global slot1
global slot2
global port1
global port2
set hub 0
set slot1 0
set slot2 4
set port1 0
set port2 1
set speed $SPEED_1GHZ
set dataValues {64} 
set PREAMBLE_LENGTH 64
set CRC_LENGTH 32
set 1GIG 1000000000
set MIN_GAP 96

set stat [NSSocketLink 10.232.133.195 16385 $RESERVE_NONE]
if {$stat < 0} { 
   puts "Error in connecting to the SMB hub"
   exit
}
unset stat
set stat [LIBCMD HTSlotReserve $hub $slot1]
if {$stat < 0} { 
   puts "Error in reserving slot $slot1 in hub"
   ETUnLink
   exit
}
unset stat
set stat [LIBCMD HTSlotReserve $hub $slot2]
if {$stat < 0} { 
   puts "Error in reserving slot $slot2 in hub"
   ETUnLink
   exit
}



###############################################################################
# Uses NSCalculateGap to set TX as Packets Per Second
# Returns the GAP value needed to set the requested rate
###############################################################################
proc setFramesPerSecond {H S P speed dataLength framesPerSecond} {
    set gap ""
    NSCalculateGap $::PPS_TO_GAP_BITS $speed $dataLength $framesPerSecond gap $H $S $P
    return $gap
}
###############################################################################

  
###############################################################################
proc displayCount {H S P} {
  
    struct_new cs HTCountStructure
    after 100
    LIBCMD HTGetCounters cs $H $S $P
  
   puts "======================================================="
   puts "              Counter Data Card [expr $S +1]                |"
   puts "======================================================="
   puts "            Events (Total)    |     Rates (Per Second) |"
   puts "======================================================="
   puts "Tx Packets      [format "%12d" $cs(TmtPkt)] |            [format "%12d" $cs(TmtPktRate)] |"
   puts "Rx Packets      [format "%12d" $cs(RcvPkt)] |            [format "%12d" $cs(RcvPktRate)] |"
   puts "-------------------------------------------------------"

   set rates {}
   set rates $cs(TmtPktRate)
   lappend rates $cs(RcvPktRate)
   return $rates 
}
###############################################################################
  
  
proc sendData {framesPerSecond hub slot port speed dataLength} {
     
   # switch rate to pps - use function return as argument to HTGap command
   puts "Setting rate to $framesPerSecond"
   LIBCMD HTGap [setFramesPerSecond $hub $slot $port $speed $dataLength $framesPerSecond] $hub $slot $port
   puts "Transmitting at $framesPerSecond frames per second"
   LIBCMD HTRun $::HTRUN $hub $slot $port
}

proc stop {hub slot port} {
   LIBCMD HTRun $::HTSTOP $hub $slot $port
}

# map of IP address with P4080 MAC addresses
array set p4080_mac_addr {
   24 {0x00 0x04 0x9f 0x02 0x00 0xfd}
   29 {0x00 0x04 0x9f 0x02 0x01 0xfd}
   21 {0x00 0x04 0x9f 0x02 0x02 0xfd}
   22 {0x00 0x04 0x9f 0x02 0x03 0xfd}
}

# map of IP address with Slot and Port
array set ip_slot_port {
   24 {1A}
   29 {3A}
   21 {1B}
   22 {3B}
}

# map of IP address with slot MAC addresses
array set sb_mac_addr {
   24 {0x00 0x00 0x00 0x00 0x00 0x01}
   29 {0x00 0x00 0x00 0x00 0x00 0x02}
   21 {0x00 0x00 0x00 0x00 0x00 0x05}
   22 {0x00 0x00 0x00 0x00 0x00 0x06}
}

set data [exec cat [lindex $argv 0] | grep {\-s}]
set slot1_port1_traffic 0
set slot1_port2_traffic 0
set slot2_port1_traffic 0
set slot2_port2_traffic 0

set data [split $data "\n"]
set num_lines [llength $data]

proc createStream {dataLength} {
  
   global hub
   global slot1
   global slot2
   global port1
   global port2
   
   LIBCMD HTResetPort $::RESET_FULL $hub $slot1 $port1
#   LIBCMD HTResetPort $::RESET_FULL $hub $slot1 $port2
   LIBCMD HTResetPort $::RESET_FULL $hub $slot2 $port1
#   LIBCMD HTResetPort $::RESET_FULL $hub $slot2 $port2

   LIBCMD HTClearPort $hub $slot1 $port1
#   LIBCMD HTClearPort $hub $slot1 $port2
   LIBCMD HTClearPort $hub $slot2 $port1
#   LIBCMD HTClearPort $hub $slot2 $port2

   struct_new ip_stream_list1 StreamIP*1024
   struct_new ip_stream_list2 StreamIP*1024
   struct_new ip_stream_list3 StreamIP*1024
   struct_new ip_stream_list4 StreamIP*1024
   set i1 0
   set i2 0
   set i3 0
   set i4 0

   foreach line $::data {
      set temp $line
      regsub -all {\.} [lindex $temp 3] { } src_ip
      regsub -all {\.} [lindex $temp 5] { } dest_ip
      regsub -all {\.} [lindex $temp 7] { } gateway
      set dest_mac [lindex [array get ::p4080_mac_addr [lindex [split $src_ip " "] 2]] 1]
      set src_mac [lindex [array get ::sb_mac_addr [lindex [split $src_ip " "] 2]] 1]
      set port [lindex [array get ::ip_slot_port [lindex [split $src_ip " "] 2]] 1]
      puts "Adding stream for port $port"

      if {$port == "1A"} {
         set ::slot1_port1_traffic 1
         set ip_stream_list1($i1.ucActive)       1
         set ip_stream_list1($i1.ucProtocolType) $::L3_STREAM_IP
         set ip_stream_list1($i1.uiFrameLength)  $dataLength
         set ip_stream_list1($i1.ucTagField)     0
         set ip_stream_list1($i1.DestinationMAC.0) [lindex $dest_mac 0]
         set ip_stream_list1($i1.DestinationMAC.1) [lindex $dest_mac 1]
         set ip_stream_list1($i1.DestinationMAC.2) [lindex $dest_mac 2]
         set ip_stream_list1($i1.DestinationMAC.3) [lindex $dest_mac 3]
         set ip_stream_list1($i1.DestinationMAC.4) [lindex $dest_mac 4]
         set ip_stream_list1($i1.DestinationMAC.5) [lindex $dest_mac 5]
         set ip_stream_list1($i1.SourceMAC.0)      [lindex $src_mac 0]
         set ip_stream_list1($i1.SourceMAC.1)      [lindex $src_mac 1]
         set ip_stream_list1($i1.SourceMAC.2)      [lindex $src_mac 2]
         set ip_stream_list1($i1.SourceMAC.3)      [lindex $src_mac 3]
         set ip_stream_list1($i1.SourceMAC.4)      [lindex $src_mac 4]
         set ip_stream_list1($i1.SourceMAC.5)      [lindex $src_mac 5]
         set ip_stream_list1($i1.TimeToLive)     64
         set ip_stream_list1($i1.DestinationIP.0)  [lindex $dest_ip 0]
         set ip_stream_list1($i1.DestinationIP.1)  [lindex $dest_ip 1]
         set ip_stream_list1($i1.DestinationIP.2)  [lindex $dest_ip 2]
         set ip_stream_list1($i1.DestinationIP.3)  [lindex $dest_ip 3]
         set ip_stream_list1($i1.SourceIP.0)  [lindex $src_ip 0]
         set ip_stream_list1($i1.SourceIP.1)  [lindex $src_ip 1]
         set ip_stream_list1($i1.SourceIP.2)  [lindex $src_ip 2]
         set ip_stream_list1($i1.SourceIP.3)  [lindex $src_ip 3]
         set ip_stream_list1($i1.Netmask)                   {255 255 255 0}
         set ip_stream_list1($i1.Gateway.0)  [lindex $gateway 0]
         set ip_stream_list1($i1.Gateway.1)  [lindex $gateway 1]
         set ip_stream_list1($i1.Gateway.2)  [lindex $gateway 2]
         set ip_stream_list1($i1.Gateway.3)  [lindex $gateway 3]
         set i1 [expr 1 + $i1]
      }
      if {$port == "1B"} {
         set ::slot1_port2_traffic 1
         set ip_stream_list2($i2.ucActive)       1
         set ip_stream_list2($i2.ucProtocolType) $::L3_STREAM_IP
         set ip_stream_list2($i2.uiFrameLength)  $dataLength
         set ip_stream_list2($i2.ucTagField)     0
         set ip_stream_list2($i2.DestinationMAC.0) [lindex $dest_mac 0]
         set ip_stream_list2($i2.DestinationMAC.1) [lindex $dest_mac 1]
         set ip_stream_list2($i2.DestinationMAC.2) [lindex $dest_mac 2]
         set ip_stream_list2($i2.DestinationMAC.3) [lindex $dest_mac 3]
         set ip_stream_list2($i2.DestinationMAC.4) [lindex $dest_mac 4]
         set ip_stream_list2($i2.DestinationMAC.5) [lindex $dest_mac 5]
         set ip_stream_list2($i2.SourceMAC.0)      [lindex $src_mac 0]
         set ip_stream_list2($i2.SourceMAC.1)      [lindex $src_mac 1]
         set ip_stream_list2($i2.SourceMAC.2)      [lindex $src_mac 2]
         set ip_stream_list2($i2.SourceMAC.3)      [lindex $src_mac 3]
         set ip_stream_list2($i2.SourceMAC.4)      [lindex $src_mac 4]
         set ip_stream_list2($i2.SourceMAC.5)      [lindex $src_mac 5]
         set ip_stream_list2($i2.TimeToLive)     64
         set ip_stream_list2($i2.DestinationIP.0)  [lindex $dest_ip 0]
         set ip_stream_list2($i2.DestinationIP.1)  [lindex $dest_ip 1]
         set ip_stream_list2($i2.DestinationIP.2)  [lindex $dest_ip 2]
         set ip_stream_list2($i2.DestinationIP.3)  [lindex $dest_ip 3]
         set ip_stream_list2($i2.SourceIP.0)  [lindex $src_ip 0]
         set ip_stream_list2($i2.SourceIP.1)  [lindex $src_ip 1]
         set ip_stream_list2($i2.SourceIP.2)  [lindex $src_ip 2]
         set ip_stream_list2($i2.SourceIP.3)  [lindex $src_ip 3]
         set ip_stream_list2($i2.Netmask)                   {255 255 255 0}
         set ip_stream_list2($i2.Gateway.0)  [lindex $gateway 0]
         set ip_stream_list2($i2.Gateway.1)  [lindex $gateway 1]
         set ip_stream_list2($i2.Gateway.2)  [lindex $gateway 2]
         set ip_stream_list2($i2.Gateway.3)  [lindex $gateway 3]
         set i2 [expr 1 + $i2]
      }
      if {$port == "3A"} {
         set ::slot2_port1_traffic 1
         set ip_stream_list3($i3.ucActive)       1
         set ip_stream_list3($i3.ucProtocolType) $::L3_STREAM_IP
         set ip_stream_list3($i3.uiFrameLength)  $dataLength
         set ip_stream_list3($i3.ucTagField)     0
         set ip_stream_list3($i3.DestinationMAC.0) [lindex $dest_mac 0]
         set ip_stream_list3($i3.DestinationMAC.1) [lindex $dest_mac 1]
         set ip_stream_list3($i3.DestinationMAC.2) [lindex $dest_mac 2]
         set ip_stream_list3($i3.DestinationMAC.3) [lindex $dest_mac 3]
         set ip_stream_list3($i3.DestinationMAC.4) [lindex $dest_mac 4]
         set ip_stream_list3($i3.DestinationMAC.5) [lindex $dest_mac 5]
         set ip_stream_list3($i3.SourceMAC.0)      [lindex $src_mac 0]
         set ip_stream_list3($i3.SourceMAC.1)      [lindex $src_mac 1]
         set ip_stream_list3($i3.SourceMAC.2)      [lindex $src_mac 2]
         set ip_stream_list3($i3.SourceMAC.3)      [lindex $src_mac 3]
         set ip_stream_list3($i3.SourceMAC.4)      [lindex $src_mac 4]
         set ip_stream_list3($i3.SourceMAC.5)      [lindex $src_mac 5]
         set ip_stream_list3($i3.TimeToLive)     64
         set ip_stream_list3($i3.DestinationIP.0)  [lindex $dest_ip 0]
         set ip_stream_list3($i3.DestinationIP.1)  [lindex $dest_ip 1]
         set ip_stream_list3($i3.DestinationIP.2)  [lindex $dest_ip 2]
         set ip_stream_list3($i3.DestinationIP.3)  [lindex $dest_ip 3]
         set ip_stream_list3($i3.SourceIP.0)  [lindex $src_ip 0]
         set ip_stream_list3($i3.SourceIP.1)  [lindex $src_ip 1]
         set ip_stream_list3($i3.SourceIP.2)  [lindex $src_ip 2]
         set ip_stream_list3($i3.SourceIP.3)  [lindex $src_ip 3]
         set ip_stream_list3($i3.Netmask)                   {255 255 255 0}
         set ip_stream_list3($i3.Gateway.0)  [lindex $gateway 0]
         set ip_stream_list3($i3.Gateway.1)  [lindex $gateway 1]
         set ip_stream_list3($i3.Gateway.2)  [lindex $gateway 2]
         set ip_stream_list3($i3.Gateway.3)  [lindex $gateway 3]
         set i3 [expr 1 + $i3]
      }
      if {$port == "3B"} {
         set ::slot2_port2_traffic 1
         set ip_stream_list4($i4.ucActive)       1
         set ip_stream_list4($i4.ucProtocolType) $::L3_STREAM_IP
         set ip_stream_list4($i4.uiFrameLength)  $dataLength
         set ip_stream_list4($i4.ucTagField)     0
         set ip_stream_list4($i4.DestinationMAC.0) [lindex $dest_mac 0]
         set ip_stream_list4($i4.DestinationMAC.1) [lindex $dest_mac 1]
         set ip_stream_list4($i4.DestinationMAC.2) [lindex $dest_mac 2]
         set ip_stream_list4($i4.DestinationMAC.3) [lindex $dest_mac 3]
         set ip_stream_list4($i4.DestinationMAC.4) [lindex $dest_mac 4]
         set ip_stream_list4($i4.DestinationMAC.5) [lindex $dest_mac 5]
         set ip_stream_list4($i4.SourceMAC.0)      [lindex $src_mac 0]
         set ip_stream_list4($i4.SourceMAC.1)      [lindex $src_mac 1]
         set ip_stream_list4($i4.SourceMAC.2)      [lindex $src_mac 2]
         set ip_stream_list4($i4.SourceMAC.3)      [lindex $src_mac 3]
         set ip_stream_list4($i4.SourceMAC.4)      [lindex $src_mac 4]
         set ip_stream_list4($i4.SourceMAC.5)      [lindex $src_mac 5]
         set ip_stream_list4($i4.TimeToLive)     64
         set ip_stream_list4($i4.DestinationIP.0)  [lindex $dest_ip 0]
         set ip_stream_list4($i4.DestinationIP.1)  [lindex $dest_ip 1]
         set ip_stream_list4($i4.DestinationIP.2)  [lindex $dest_ip 2]
         set ip_stream_list4($i4.DestinationIP.3)  [lindex $dest_ip 3]
         set ip_stream_list4($i4.SourceIP.0)  [lindex $src_ip 0]
         set ip_stream_list4($i4.SourceIP.1)  [lindex $src_ip 1]
         set ip_stream_list4($i4.SourceIP.2)  [lindex $src_ip 2]
         set ip_stream_list4($i4.SourceIP.3)  [lindex $src_ip 3]
         set ip_stream_list4($i4.Netmask)                   {255 255 255 0}
         set ip_stream_list4($i4.Gateway.0)  [lindex $gateway 0]
         set ip_stream_list4($i4.Gateway.1)  [lindex $gateway 1]
         set ip_stream_list4($i4.Gateway.2)  [lindex $gateway 2]
         set ip_stream_list4($i4.Gateway.3)  [lindex $gateway 3]
         set i4 [expr 1 + $i4]
      }
   }
   if {$i1 > 0} {
      LIBCMD HTSetStructure $::L3_DEFINE_IP_STREAM 0 0 0 ip_stream_list1 0 $hub $slot1 $port1
   }
#   LIBCMD HTSetStructure $::L3_DEFINE_IP_STREAM 0 0 0 ip_stream_list2 0 $hub $slot1 $port2
   if {$i3 > 0} {
      LIBCMD HTSetStructure $::L3_DEFINE_IP_STREAM 0 0 0 ip_stream_list3 0 $hub $slot2 $port1
   }
#   LIBCMD HTSetStructure $::L3_DEFINE_IP_STREAM 0 0 0 ip_stream_list4 0 $hub $slot2 $port2

   unset ip_stream_list1
   unset ip_stream_list2
   unset ip_stream_list3
   unset ip_stream_list4
   unset i1
   unset i2
   unset i3
   unset i4

   LIBCMD HTTransmitMode $::CONTINUOUS_PACKET_MODE $hub $slot1 $port1
#   LIBCMD HTTransmitMode $::CONTINUOUS_PACKET_MODE $hub $slot1 $port2
   LIBCMD HTTransmitMode $::CONTINUOUS_PACKET_MODE $hub $slot2 $port1
#   LIBCMD HTTransmitMode $::CONTINUOUS_PACKET_MODE $hub $slot2 $port2
} 
        
set sectime 2
set msectime [expr $sectime*1000]
  
LIBCMD HTGap 96 $hub $slot1 $port1
#LIBCMD HTGap 96 $hub $slot1 $port2
LIBCMD HTGap 96 $hub $slot2 $port1
#LIBCMD HTGap 96 $hub $slot2 $port2
         
foreach dataLength $dataValues  {
   puts "---------------------------------------------------------------"   
   puts "Running throughput test for frame size $dataLength" 
   puts "---------------------------------------------------------------"

 
   set PACKET_BITS [expr ($PREAMBLE_LENGTH + $CRC_LENGTH) + ($dataLength * 8)]

   # Add minimum gap (96 bit times) to get the total
   set PACKET_PLUS_MIN_GAP [expr $PACKET_BITS + $MIN_GAP]


   # - Set speed on the card and calculate WIRE_RATE  - the rate with a minimum 96             #
   set WIRE_RATE [expr $1GIG / $PACKET_PLUS_MIN_GAP]
   set left 1
   set right $WIRE_RATE
   set framesPerSecond [expr ($left+$right)/2]
   after 2000
   
   while { $left != $right } {
      puts "---------------------------------------------------------------"   
      puts $framesPerSecond
      puts "---------------------------------------------------------------"
   
      puts "Creating streams"
      createStream $dataLength
      puts "Done creating streams"
                
      if {$slot1_port1_traffic == 1} { sendData $framesPerSecond $hub $slot1 $port1 $speed $dataLength }
#      sendData $framesPerSecond $hub $slot1 $port2 $speed $dataLength   
      if {$slot2_port1_traffic == 1} { sendData $framesPerSecond $hub $slot2 $port1 $speed $dataLength }
#      sendData $framesPerSecond $hub $slot2 $port2 $speed $dataLength
   
      after $msectime
                    
      set txRate_slot1 [expr [lindex [displayCount $hub $slot1 $port1] 0] + [lindex [displayCount $hub $slot1 $port1] 0]]
      set txRate_slot2 [expr [lindex [displayCount $hub $slot2 $port1] 0] + [lindex [displayCount $hub $slot2 $port1] 0]]
      set txRate_slot2 0
      set txRate [expr $txRate_slot1 + $txRate_slot2]
      puts "Transmit Rate"
      puts $txRate 
      set t1 $txRate 
      after 1000
      set rxRate_slot1 [expr [lindex [displayCount $hub $slot1 $port1] 1] + [lindex [displayCount $hub $slot1 $port2] 1]]
      set rxRate_slot2 [expr [lindex [displayCount $hub $slot2 $port1] 1] + [lindex [displayCount $hub $slot2 $port2] 1]]
      set rxRate_slot2 0
      set rxRate [expr $rxRate_slot1 + $rxRate_slot2]
      puts "Receive Rate"
      puts $rxRate
      set r1 $rxRate
      set tolerance .5
   
      if {(($r1 <= $t1) && ($r1 > [expr $t1-($t1*$tolerance/100)])) || ($r1 > $t1) }  {
         if {($framesPerSecond < $r1)} {
            set framesPerSecond $r1
         }
         set left [expr $framesPerSecond + 1]
         puts "TX equals RX....\nleft: $left*****right: $right"
         if {($right < $left)} {
            ## exit while
            break
         }
         set framesPerSecond  [expr {($left+$right)/2}]   
      } else { 
         set right $framesPerSecond
         set framesPerSecond  [expr {($left+$right)/2}]
         puts "TX not equals RX....\nleft: $left*****right: $right"
      }
      stop $hub $slot1 $port1
#      stop $hub $slot1 $port2
      stop $hub $slot2 $port1
#      stop $hub $slot2 $port2
   }        

   puts "---------------------------------------------------------------"   
   puts "Throughput of frame size $dataLength bytes is $framesPerSecond fps" 
   set rateMbps [expr ($framesPerSecond*(8*$dataLength+32))/1000000]
   puts "$rateMbps Mbps"
   puts "---------------------------------------------------------------"
   set tput($dataLength) $rateMbps
}

puts "\n\n\n--- Throughput test results (Upstream Only)---\n\n\n"
puts "======================================================="
puts "    Frame Size          Throughput      "
puts "======================================================="

foreach id $dataValues  {
   puts "      [format "%4d" $id]             [format "%10d" $tput($id)] "    
}
      
ETUnLink

