/**
 \file cpu_freq.c
 \brief Parse cpu clock frequency from LWE device tree
 */
/*
 * Copyright (C) 2009-2010 Freescale Semiconductor, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 * NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <bitops.h>
#include <init.h>
#include <libfdt.h>
#include <log.h>
#include "cpu_freq.h"

uint32_t get_cpu_freq(void)
{
	int node;
	uint32_t cpu_freq = 0;
	unsigned int core_no;

	core_no = mfspr(SPR_PIR);

	/*Read clock-frequency from the devtree */
	if ((node =
	     fdt_node_offset_by_prop_value(devtree_ptr, -1, "device_type",
					   "cpu", strlen("cpu") + 1)) < 0) {
		LWE_ERROR("Missing cpu node");
		return 0;
	}

	do {
		int len = 0;

		const uint32_t *reg =
		    fdt_getprop(devtree_ptr, node, "reg", &len);
		if (!reg || len != sizeof(uint32_t)) {
			LWE_ERROR("Missing reg property in cpu node");
			continue;
		}
		if (*reg != core_no)
			continue;

		const uint32_t *freq =
		    fdt_getprop(devtree_ptr, node, "clock-frequency", &len);
		if (!freq || len != sizeof(uint32_t)) {
			LWE_ERROR("Reading clock-frequency failed");
			break;
		}
		cpu_freq = *freq;
		break;

	} while ((node = fdt_node_offset_by_prop_value(devtree_ptr, node,
						       "device_type", "cpu",
						       strlen("cpu") + 1)) !=
		 -FDT_ERR_NOTFOUND);
	return cpu_freq;
}
