/**
 \file print_decorated.c
 \brief Contains functions to print decorated stats info
 */
/*
 * Copyright (C) 2007-2009 Freescale Semiconductor, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 * NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include "common/print_decorated.h"
#include "init.h"
#include <app_common.h>

#define STAT_BANNER		\
	"Stat Name			  Count		       Bytes"

/*
 * Prints 32 bit stats info
 */
void print_stat32(const stat32_t * stat, const char *name, bool print_zero)
{
	if ((print_zero == true) || (*stat != 0)) {
		APP_DEBUG("%s =  0x%08x", name, *stat);
	}
}

/*
 * Prints 64 bit stats info
 */
void print_stat64(const union stat64_t *stat, const char *name, bool print_zero)
{
	if ((print_zero == true) ||
	    (stat->words.hi != 0) || (stat->words.lo != 0)) {
		APP_DEBUG("%s   0x%08x%08x", name, stat->words.hi,
			  stat->words.lo);
	}
}

/*
 * Prints 64 bit stats info pair
 */
void print_stat64_pair(const struct stat64_pair_t *stat, const char *name,
		       bool print_zero)
{
	if ((print_zero == true) ||
	    (stat->inc.words.hi != 0) || (stat->inc.words.lo != 0) ||
	    (stat->acc.words.hi != 0) || (stat->acc.words.lo != 0)) {
		APP_DEBUG("%s   0x%08x%08x	0x%08x%08x",
			  name,
			  stat->inc.words.hi,
			  stat->inc.words.lo,
			  stat->acc.words.hi, stat->acc.words.lo);
	}
}

/*
 * Prints Stats Banner
 */
void print_stats_banner(void)
{
	APP_DEBUG("%s", STAT_BANNER);
}
