/**
 \file dpa_eth.h
 \brief Header file defining structure of DPA nodes.
 */
/*
 * Copyright (C) 2009-2011 Freescale Semiconductor, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 * NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef __DPA_ETH_H__
#define __DPA_ETH_H__

#ifdef __cplusplus
	extern "C" {
#endif

#include <stdbool.h>
#include <ether.h>
#include <fsl_qman.h>
#include <dpa/dpa_bman.h>
#include <dpa/dpa_qman.h>

#define FM_FD_CMD_FCO	0x80000000UL	/*Transmit confirmation override */
#define PARSE_RESULT_OFFSET	16
#define MAX_BPOOL_PER_PORT	8

#define MAX_NUM_PORTS	12
#define INV_PORT	0xFFFF
#define MAX_NUM_LPID	0x100

#define SIZE_TO_STASH_LINES(s) ((s >> 6) + ((s & 0x3F) ? 1 : 0))
#define GET_STASH_LINES(l) \
	(SIZE_TO_STASH_LINES(l) > 3 ? 3 : SIZE_TO_STASH_LINES(l))

/**
 \brief DPA Port Type
*/
enum dpa_eth_t {
	INV_ETH = 0,	/**< Invalid Port Type */
	MAC_ETH,	/**< FMAN Port */
	MAC_LESS_ETH	/**< MAC Less Port */
};

/**
 \brief DPA Port Info
*/
struct dpa_port_t {
	enum dpa_eth_t type;		 /**< MAC_Less or Fman port */
	struct dpa_fq_range_t ingress;	 /**< Default ingress FQs */
	struct dpa_fq_range_t ingress_err;	 /**< Ingress err FQs */
	struct dpa_fq_range_t pcd_ingress;	 /**< PCD ingress FQs */
	struct dpa_fq_range_t egress;		/**< Egress FQ */
	struct dpa_fq_range_t egress_err;	 /**< Egress error FQ */
	struct dpa_fq_range_t egress_confirm; /**< Egress confirm FQ */
	int mac_node;
	uint8_t bpid[MAX_BPOOL_PER_PORT];  /**< List of BPIDs*/
	uint8_t mac_addr[ETH_ALEN]; /**< MAC address*/
};

extern volatile uint32_t g_num_dpa_eth_ports;
extern struct dpa_bpool_t dpa_bpools[MAX_NUM_BMAN_POOLS];
extern struct dpa_port_t dpa_eth_ports[MAX_NUM_PORTS];
extern uint16_t g_lpid_to_port_index[MAX_NUM_LPID];

/*1G DTSEC registers*/
#define MACCFG1_OFFSET		0x100
#define MACCFG2_OFFSET		0x104
#define ECNTRL_OFFSET		0x14

#define RCTRL_OFFSET		0x50

#define MACCFG1_LOOPBACK	0x00000100
#define MACCFG1_RX_EN           0x00000004
#define MACCFG1_TX_EN           0x00000001
#define RCTRL_MPROM             0x00000008
#define RCTRL_UPROM             0x00000001

#define TGEC_CMD_CTRL_REG_OFFSET	0x8
#define CMD_CFG_PROMIS_EN       0x00000010
#define CMD_CFG_RX_EN           0x00000002
#define CMD_CFG_TX_EN           0x00000001
#define CMD_CFG_LOOPBACK_EN	0x00000400

struct fman_mac;

int dpa_eth_read(uint32_t * recv_channel_map);

int dpa_eth_fq_create(struct qman_fq_cb *rx_default_cb,
		      struct qman_fq_cb *rx_pcd_cb,
		      struct qman_fq_cb *rx_err_cb,
		      struct qman_fq_cb *tx_cb,
		      struct qman_fq_cb *tx_confirm_cb,
		      struct qman_fq_cb *tx_err_cb, uint32_t priv_data_size);

int dpa_eth_fq_init(uint32_t data_stash_size,
	uint32_t ann_stash_size, uint32_t ctx_stash_size,
	struct qman_orp_pcd *pcd_orp, struct td_param *pcd_td);

int dpa_eth_enable_ports(const bool is_promisc);

int dpa_eth_disable_ports(const bool disable_rx, const bool disable_tx);

int dpa_eth_rx_fq_retire(void);

int dpa_eth_tx_fq_retire(void);

int shell_cmd_loopback(int argc, char *argv[]);

#ifdef __cplusplus
	}
#endif

#endif /*__DPA_ETH_H__*/
