/**
 \file frame_handler.c
 \brief Frame Handler Functions
 */
/*
 * Copyright (C) 2007-2010 Freescale Semiconductor, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 * NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include <stdbool.h>
#include <stdint.h>
#include "net/frame_handler.h"
#include "net/net_dev.h"
#include "statistics.h"
#include "app_common.h"
#include <sg_api.h>

extern struct dpa_buff_allocator *buff_allocator;

void rx_error_handler(struct fq_context_t *ctxt,
		      struct annotations_t *notes, void *data)
{
	void volatile *stat_addr;
	struct net_dev_stats_t *stats;
	uint32_t len;

	stats = ctxt->dev->stats;
	stat_addr = &(stats->rx_dropped.acc.words.lo);
	len = get_single_fd_length(notes->fd);
	decorated_store_64_inc_acc_32(stat_addr, len);
	APP_INFO("Frame came to rx error handler");
	lwe_free_fd(buff_allocator, notes->fd);
}

void tx_error_handler(struct fq_context_t *ctxt,
		      struct annotations_t *notes, void *data)
{
	void volatile *stat_addr;
	struct net_dev_stats_t *stats;
	uint32_t len;

	stats = ctxt->dev->stats;
	stat_addr = &(stats->tx_dropped.acc.words.lo);
	len = get_single_fd_length(notes->fd);
	decorated_store_64_inc_acc_32(stat_addr, len);
	lwe_free_fd(buff_allocator, notes->fd);
}

void confirm_handler(struct fq_context_t *ctxt __UNUSED,
		     struct annotations_t *notes __UNUSED, void *data __UNUSED)
{
	/* Eventually, we should be decrementing the xmit count here.  Then,
	   if it is non-zero, we re-transmit the frame, and if it is zero, 
	   we are done with the multiple transmissions */
}

void parsing_handler(struct fq_context_t *ctxt,
		     struct annotations_t *notes, void *data)
{
	void volatile *stat_addr;
	struct net_dev_stats_t *stats;
	uint32_t len;

	/* Eventually, we should be decrementing the xmit count here.  Then,
	   if it is non-zero, we re-transmit the frame, and if it is zero, 
	   we are done with the multiple transmissions */
	stats = ctxt->dev->stats;
	stat_addr = &(stats->rx_dropped.acc.words.lo);
	len = get_single_fd_length(notes->fd);
	decorated_store_64_inc_acc_32(stat_addr, len);
	lwe_free_fd(buff_allocator, notes->fd);
}
