/**
 \file notify.c
 */
/*
 * Copyright (C) 2007-2010 Freescale Semiconductor, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 * NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include <stdint.h>
#include <stdlib.h>
#include <compiler.h>
#include <null.h>
#include <hv_misc.h>
#include "net/notify.h"
#include "mmap.h"
#include <bitops.h>

struct notify_chain_t *notify_create_chain(void)
{
	return malloc(sizeof(struct notify_chain_t));
}

void notify_subscribe(struct notify_chain_t *chain, notify_fn callback)
{
	struct notify_t **current_ptr;
	struct notify_t *new_notified;

	assert(chain != NULL);
	assert(&chain->head != NULL);
	assert(callback != NULL);

	spin_lock(&chain->lock);
	current_ptr = &chain->head;

	while ((*current_ptr)->next != NULL)
		current_ptr = &((*current_ptr)->next);

	new_notified = malloc(sizeof(*new_notified));
	if (!new_notified) {
		spin_unlock(&chain->lock);
		return;
	}

	new_notified->callback = callback;
	new_notified->next = NULL;
	(*current_ptr)->next = new_notified;

	spin_unlock(&chain->lock);
}

void notify_publish(struct notify_chain_t *chain, uint32_t event, void *data)
{
	struct notify_t *notified;

	spin_lock(&chain->lock);

	for (notified = chain->head; notified != NULL;
	     notified = notified->next) {
		notified->callback(event, data);
	}
	spin_unlock(&chain->lock);
}
