/**
 \core_perfmon.c
 \brief Core Perfmon library
 */
/*
 * Copyright (C) 2010 Freescale Semiconductor, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN
 * NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "core_perfmon.h"

int32_t core_pm_configure(struct core_pm_events *core_pm_counters,
			  uint8_t count)
{
	uint8_t i;

	if (!core_pm_counters) {
		APP_ERROR("%s: NULL Core_PM_Counter\n", __func__);
		return -EINVAL;
	}

	if (count > MAX_PM_COUNTERS) {
		APP_ERROR("%s: Requested more counters than available\n");
		return -EINVAL;
	}

	for (i = 0; i < count; i++) {
		if (core_pm_counters[i].event == PM_NOTHING)
			continue;

		if (configurePM_event(core_pm_counters[i].event,
				      PM_ENABLE, 0, 0, PMI_DISABLE,
				      &core_pm_counters[i].handle)) {
			printf("%s: Failed to init PM counter\n", __func__);
			return -EINVAL;
		}
		printf("%s: Configured %x PMC for %s\n", __func__,
		       core_pm_counters[i].handle, core_pm_counters[i].name);
	}

	return 0;
}

int32_t core_pm_unconfigure(struct core_pm_events *core_pm_counters,
			    uint8_t count)
{
	uint8_t i;

	if (count > MAX_PM_COUNTERS) {
		APP_ERROR("%s: Requested more counters than available\n");
		return -EINVAL;
	}

	for (i = 0; i < count; i++) {
		if (core_pm_counters[i].event == PM_NOTHING)
			continue;

		if (unconfigurePM_event(core_pm_counters[i].handle)) {
			printf("%s: Failed to init PM counter\n", __func__);
			return -EINVAL;
		}
	}
	return 0;
}

void reset_pmc(void)
{
	mtpmr(PMC0, 0);
	mtpmr(PMC1, 0);
	mtpmr(PMC2, 0);
	mtpmr(PMC3, 0);
}

void core_freeze_counting(void)
{
	uint32_t pmgc0;
	pmgc0 = mfpmr(PMGC0);
	mtpmr(PMGC0, PM_FREEZE_ALL_COUNTERS | pmgc0);
}

void core_unfreeze_counting(void)
{
	uint32_t pmgc0;
	pmgc0 = mfpmr(PMGC0) & PM_FAC_MASK;
	mtpmr(PMGC0, pmgc0);
}
