/**
 \core_perfmon.h
 \brief Core Perfmon library
 */
/*
 * Copyright (C) 2010 Freescale Semiconductor, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN
 * NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef __CORE_PERFMON_H__
#define __CORE_PERFMON_H__
#include <stdlib.h>
#include <io.h>
#include <errno.h>
#include <bitops.h>
#include <init.h>
#include <mmap.h>
#include <sg_api.h>
#include <dpa_alloc.h>
#include <fsl_qman.h>
#include <ip/ip.h>
#include <fm_types.h>
#include <app_common.h>
#include "perf_mon.h"

/* Size of Event string to be displayed */
#define SIZE_OF_EVENT_NAME 20
#define PM_FREEZE_ALL_COUNTERS 0x80000000
#define PM_FAC_MASK 0x7FFFFFFF
#define MAX_PM_COUNTERS 4
#define PMC0		0x10
#define PMC1		0x11
#define PMC2		0x12
#define PMC3		0x13
#define PMGC0		0x190

/**
 \note An array of core_pm_events should be supplied to config and unconfig APIs
 */
struct core_pm_events {
	char name[SIZE_OF_EVENT_NAME]; /**< Event Name */
	enum PM_event event; /**< Event Type as specified in perf_mon.h */
	uint32_t handle;
};

/**
 \brief Configure PerfMon Events
 */
extern int32_t core_pm_configure(struct core_pm_events *, uint8_t);

/**
 \brief UnConfigure PerfMon Events
 */
extern int32_t core_pm_unconfigure(struct core_pm_events *, uint8_t);

/**
 \brief Reset All PerfMon Counters to Zero
 */
extern void reset_pmc(void);

/**
 \brief Freeze All Core PerfMon Counters
 */
extern void core_freeze_counting(void);

/**
 \brief UnFreeze All Core PerfMon Counters
 */
extern void core_unfreeze_counting(void);
#endif
