/**
\file	sys_perfmon.h
\brief	Basic System Profiling library
*/
/*
 * Copyright (C) 2010 Freescale Semiconductor, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 * NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef __SYS_PERFMON_H__
#define __SYS_PERFMON_H__
#include <stdlib.h>
#include <io.h>
#include <errno.h>
#include <bitops.h>
#include <init.h>
#include <mmap.h>
#include <sg_api.h>
#include <dpa_alloc.h>
#include <fsl_qman.h>
#include <ip/ip.h>
#include <fm_types.h>
#include <app_common.h>

/**
\brief DCSR Struct to populate raw register values
*/
struct dcsr_regs {
	uint32_t offset;
	uint32_t value;
};

enum profile_type {
	PROF_NONE = 0,
	PROF_CPC,
	PROF_DDR_CPC
};

#define NEXUS_BASE 0x40000
#define NEXUS_REG_OFFSET 0x40c
#define CORE_PC_OFFSET 0x2c
#define ENABLE_WATCHPOINT_7 0x70000000
#define EVENT_COUNTER_ENABLED 0x80000000
#define CGACRE0_OFFSET 0x22500
#define PASSIVE_EVENT_5 0x1000
#define EPGACR0_OFFSET 0x480
#define CAPTURE_GRP0_COUNTERS 0x10;
#define EPECR0_OFFSET 0x300
#define RAISE_SCU0_EVENT 0x1

/**
 \brief Initialize DCSR Space for pushing debugging information
 */
uint32_t *init_dcsr(void);

/*******************************************************************
For Following APIs to work, DCSR must be mapped with init_dcsr
*******************************************************************/

/* Stop DCSR Counters */
void stop_counters(void);

/* Start Counters */
void start_counters(void);

/**
 \brief Take Snapshot of global counters
 */
void system_snapshot(void);

/**
 \brief Configure DCSR Space for counting system events
 \param[in] set start_counters to start counting system performance counters
 \param[in] set core_counters flag to enable core performance counting
 \return int status
 */
int32_t configure_dcsr(bool start_counters, bool core_counters,
		       enum profile_type);

/**
 \brief Dump all the registers
 */
void dump_multicore_perfcount(void);
#endif
