/**
 \file common_pme.c
 */
/*
 * Copyright (C) 2009-2010 Freescale Semiconductor, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 * NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <compat.h>
#include <libfdt.h>
#include "dcl.h"
#include "app_common.h"
#include "dpa/dpa_bman.h"
#include "dpa_alloc.h"
#include "pme/common_pme.h"

/* Following are per partition variables */

#define REPORT_DISABLED 4

/**
 \brief Returns the report size based on match limit and report verbosity
 \param[in] verbosity Report Verbosity Level
 \param[in] mlimit Match Limit
 \return Size of the report in bytes
 */
uint32_t get_report_size(uint32_t verbosity, uint32_t mlimit)
{
	uint32_t size = 0;

	if (verbosity == 0)
		size = 16;	/* Simple Match report size */
	else if (verbosity == 1)
		size = 43;	/* verbos = 1 */
	else if (verbosity == 2)
		size = 4;	/* verbos = 2 */
	else if (verbosity == 3)
		size = 40;	/* verbos = 3 */

	size = size * mlimit;

	/* SUI report size */
	if (size)
		size += 2;

	return size;
}

/**
 \brief Frees the buffer to the Bpool
 \param[in] buf Pointer to buffer to be freed
 \param[in] bpid Id of the Bpool to which to free the buffer
 \return 0 if successfull, else -1
 */
int free_bpool_buff(void *buf, uint8_t bpid)
{
	struct bm_buffer bman_buf;
	phys_addr_t addr;

	APP_DEBUG(" free_bpool_buff ( bpid = %d)", bpid);
	if (unlikely(0 == dpa_bpools[bpid].size)) {
		APP_INFO("Trying to Free buffer from invalid pool %x \n", bpid);
		return -1;
	}

	addr = vtop(buf);
	bman_buf.hi = (uint8_t) (addr >> 32);
	bman_buf.lo = (uint32_t) addr;
	bman_buf.bpid = bpid;
	if (0 > dpa_allocator_free_buff(buff_allocator, &bman_buf)) {
		APP_ERROR(" dpa allocator free buff fails");
		return -1;
	}
	return 0;
}
