/** 
 \file ipsa_pme.h
 \brief Common datatypes and hash-defines of PME packet scan
	 Application
 */
/*
 * Copyright (C) 2009-2010 Freescale Semiconductor, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 * NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef IPSA_PME_H
#define IPSA_PME_H

#include  "fsl_pme.h"

#ifndef PME_SWITCH
#define PME_SWITCH
#endif
/**
 \brief Prototype for function to send scan request to PME Driver based on fd type
 \param[in] ctxt IP Context Pointer
 \param[in] notes Annotation pointer
 \param[in] ip_hdr_ptr Pointer to IP Header in the frame 
 \return Status
 */
typedef enum IP_STATUS (*pme_send) (struct ip_context_t * ctxt,
				    struct annotations_t * notes,
				    struct ip_header_t * ip_hdr_ptr);

struct ipsa_context_info_t {
	uint8_t valid:1, first_pkt:1, reserved:6;
	uint8_t bpid;
	uint16_t buffer_sz;
	struct pme_ctx *ipsa_pme_ctx;	/**< pme_ctx allocated from bpool */
	struct spa_pme_ctx_stats_t *pme_ctx_stats;	/**< per context statistic information */
	pme_send func;
	uint32_t scan_flags;
	uint8_t set;
	uint16_t subset;
} __PACKED;

/**
 \brief System level Statistics related to a PME
 */
struct pme_stats_t {
	uint32_t g_total_frames;	/**< Number of total frames received by the system */
	uint32_t g_post_pme_pkts;	/**< Number of frames sent to PME for scanning */
	uint32_t g_pme_pattern_pkts;	/**< Number of Scanned frames having matched patterns in them */
};

/**
 \brief Application's PME token sent and recv with the scanned frame 
 */
struct ipsa_pme_token_t {
	struct pme_ctx_token pme_ctx_token;	/**< PME driver token structure */
	uint32_t flow_id;				/**< flow Id */
	struct ip_context_t *ip_ctxt;		/**< Pointer to IP Context */
} __attribute__ ((aligned(CACHE_LINE_SIZE)));

extern volatile struct ipsa_context_info_t ipsa_context_info[];
extern struct rt_dest_t *g_flow_to_entry[];
#endif
