/**
 \file simple_pme_cpdp.h
 \brief Implements a simple, fast cache for looking up IPSec tunnels
 */
/*
 * Copyright (C) 2009-2010 Freescale Semiconductor, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 * NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef LIB_PME_CPDP_H
#define LIB_PME_CPDP_H

#define PME_MAX_FILE_SIZE				(1024 * 1024)

struct lwe_ctrl_spa_cfg_info {
	uint32_t flow_id;	 /**< Flow Id */
	uint8_t set;	     /**< Set Id (0 - 255) in the PME Database */
	uint16_t subset;	/**< Subset Id (0  0xff00) in the PME Database */

#define SET_SCAN_MODE_FLOW 		(1<<0)
#define SET_RESIDUE_ENABLE 		(1<<1)
#define SET_SCAN_FN_TYPE_BLOCK 		(1<<2)
#define SET_SCAN_MODE_DIRECT 		(~(1<<0))
#define SET_RESIDUE_DISABLE 		(~(1<<1))
#define SET_SCAN_FN_TYPE_NON_BLOCK 	(~(1<<2))
	uint32_t flag;	      /**< Flag for setting Scan mode type, residue enabling, and scan fn type */

	uint32_t report_verbosity;	  /**< Report Verbosity */
	uint32_t session_id;	    /**< session Id in the SRE associated with this PME Context (Flow) */
	uint32_t input_buff_chunk;	  /**< Output Buffer Size for the report */
	uint32_t qos_in;	/**< Qos - priority associated with the Q towards PME */
	uint32_t qos_out;	 /**< Qos - priority associated with the Q from PME */
	uint32_t match_limit;	     /**< Max number of matches to be made on this flow */
	uint32_t compare_limit;	       /**< Max number of comparisons to be made on this flow */
	uint32_t output_buff_frm_app;	     /**< Allocate buffer in app for output */
	uint32_t input_buff_size;	 /**< Input Buffer Size of the generated packet */
	uint32_t size_of_file;	/**< size of File */
	uint8_t file[PME_MAX_FILE_SIZE];	/**< File Pointer */
	uint8_t report_parse_enable;
};

#define LWE_CTRL_PME_GET_WB_BUFF(spa_info) (spa_info->free_offset + 4)
#define LWE_CTRL_PME_UPDATE_WB_BUFF(spa_info) {\
	*((uint32_t *)(spa_info->free_offset)) = spa_info->app_info.pme_info.size_of_file; \
	spa_info->free_offset = spa_info->free_offset + 4 + spa_info->app_info.pme_info.size_of_file; }

union lwe_ctrl_app_info {
	struct lwe_ctrl_spa_cfg_info pme_info;
};

/* Full description of state of transformer. */
struct lwe_ctrl_op_info {

#define LWE_CTRL_CMD_STATE_IDLE 0
#define LWE_CTRL_CMD_STATE_BUSY 1
	unsigned int state;					/**< State of Command */

#define LWE_CTRL_CMD_TYPE_PME_ADD			14
#define LWE_CTRL_CMD_TYPE_PME_DEL			15
#define LWE_CTRL_CMD_TYPE_PME_GO			16
	unsigned int msg_type;				/**< Type of Request */

#define LWE_CTRL_RSLT_FAILURE				11
#define LWE_CTRL_RSLT_SUCCESSFULL				1
#define LWE_CTRL_RSLT_FAIL_DEL_UNINIT_FLOW		2
#define LWE_CTRL_RSLT_FAIL_FLOW_ALREADY_INITIALIZE	3
#define LWE_CTRL_RSLT_FAIL_BLOCKING_FLOW_ADDED		4
#define LWE_CTRL_RSLT_FAIL_CSIZE_GT_BUFFSIZE		5
#define LWE_CTRL_RSLT_FAIL_CSIZE_GT_MAX_BPOOL_SIZE	6
#define LWE_CTRL_RSLT_FAIL_DPA_BUF_ALLOC		7
#define LWE_CTRL_RSLT_FAIL_REPORT_PARSE_NOT_ENABLED	8
#define LWE_CTRL_RSLT_FAIL_MAX_FLOW_LIMIT		9
#define LWE_CTRL_RSLT_FAIL_MAX_FRAME_LIMIT		10

	unsigned int result;					/**< Result - Successful, Failure */

	unsigned int app_type;				/**< App Type */
	uint8_t *free_offset;				/**< Free Offset to the empty buffer on the WB */
	union lwe_ctrl_app_info app_info;			/**< App Info */
};


extern volatile struct lwe_ctrl_op_info g_sLweCtrlPmeInfo;

#endif /* ifndef LIB_IPSEC_IPSEC_CPDP_H */
