/**
 \file tcp.h
 \brief This file contains data structures and defines related to TCP Packet
	format
 */
/*
 * Copyright (C) 2010 Freescale Semiconductor, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 * NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef __LIB_TCP_TCP_H
#define __LIB_TCP_TCP_H

struct tcp_hdr_t {
	uint16_t sport;		/* source port */
	uint16_t dport;		/* destination port */
	uint32_t seq;		/* sequence number */
	uint32_t ack;		/* acknowledgement number */
	uint8_t	offset:4;	/* data offset */
	uint8_t	resv:4;		/* (unused) */
	union {
		uint8_t	byte;
		struct {
			uint8_t cwr:1;
			uint8_t ece:1;
			uint8_t urg:1;
			uint8_t ack:1;
			uint8_t push:1;
			uint8_t rst:1;
			uint8_t syn:1;
			uint8_t fin:1;

		} __PACKED bits;
	} __PACKED flags;

#define	TCP_FIN		0x01
#define	TCP_SYN		0x02
#define	TCP_RST		0x04
#define	TCP_PUSH	0x08
#define	TCP_ACK		0x10
#define	TCP_URG		0x20
#define	TCP_ECE		0x40
#define	TCP_CWR		0x80
#define	TCP_FLAGS	(TCP_FIN | TCP_SYN | TCP_RST | TCP_ACK | TCP_URG | \
			TCP_ECE | TCP_CWR)

	uint16_t win;		/* window */
	uint16_t sum;		/* checksum */
	uint16_t urp;		/* urgent pointer */
} __PACKED;

#endif	/* __LIB_TCP_TCP_H */
