/**
\file simple_crypto_cpdp.h
\brief	Common datatypes of control and data plane
*/
/*
 * Copyright (C) 2008-2010 Freescale Semiconductor, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 * NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef __SIMPLE_CRYPTO_CPDP_H
#define __SIMPLE_CRYPTO_CPDP_H

#include <stdbool.h>

#define BMASK_SEC_TEST_MODE    	0x00000001	/**< Bit mask for test Mode */
#define BMASK_SEC_TEST_SET	0x00000010	/**< Bit mask for test set */
#define BMASK_SEC_BUFFER_SIZE	0x00000100	/**< Bit mask for buffer size */
#define BMASK_SEC_BUFFER_NUM	0x00001000	/**< Bit mask for number of buffers */
#define BMASK_SEC_ALG       	0x00010000	/**< Bit mask for SEC algo */
#define BMASK_SEC_ITR_NUM    	0x00100000	/**< Bit mask for number of iteration */
#define BMASK_SEC_PERF_MODE   	0x00111101	/**< valid combination in perf mode */
#define BMASK_SEC_CIPHER_MODE	0x00111011	/**< valid combination in cipher mode */

#define BUFF_NUM_PER_CORE	800	/**< Maximum number of buffers can be
						provided by user */
#define BUFF_SIZE               6400    /**< Maximum buffer size that can be
						provided by user */

/**
 * CIPHER test mode compare ciphertext generated by SEC4.0 whereas it has been
 * skipped in PERF test mode
 */
enum test_mode {
	PERF = 1,
	CIPHER
};

/** SEC4.0 cryptographic raw algorithm supported in the application */
enum sec_algo {
	AES_CBC = 1,
	TDES_CBC,
	SNOW_F8,
	SNOW_F9,
	KASUMI_F8,
	KASUMI_F9,
	CRC,
	HMAC_SHA1,
	SNOW_F8_F9
};

/**
 * This structure is used to communicate user defined SEC parameters from
 * control plane to LWE data plane through WB area.
 */
struct crypto_param {
	enum test_mode mode;		/**< test mode */
	unsigned int test_set;		/**< test set number */
	unsigned int buf_size;		/**< buffer size */
	unsigned int buf_num_per_core;	/**< total number of buffers, max = 5000 */
	unsigned int itr_num;		/**< number of iteration to repeat SEC operation */
	enum sec_algo algo;		/**< SEC operation to perform */
	bool valid_params;		/**< valid parameters flag*/
};

#endif /* __SIMPLE_CRYPTO_CPDP_H */
