#!/usr/bin/python
# Copyright (C) 2010 Freescale Semiconductor, Inc.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
# NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from linux import *
from lwe import *
from uboot import *
from board_start import *
from simics_start import *
import getopt
import sys
import os
import time
import thread
import threading
import commands

mux_ser_path=""
host=""
user=""
port=0
s_port=0
board=""
core_mask=""
num_xterms=3
mux_port_string=""
use_board=0
r_flag=0
r_path=""
max_wait=150
simics_cmd=""

test_set = 0
buf_size = 0
test_mode = 0
algo = 0
buf_num = 0
loop = 0
# Prints the usage information and exits with code 2
def print_usage_and_exit():
	fd.write("\nTestbench for execution of 'simple_crypto' on P4080 Board and simics.\n\n")
	fd.write("This script is the interface to launch the simple_crypto LWE application on\n")
	fd.write("the P4080 boards. Currently, it only supports the 2p configuration of the \n")
	fd.write("hypervisor. Before executing the script, make sure that the following\n")
	fd.write("conditions are satisfied:\n")
	fd.write("  - The 'expect' utility must be installed on the machine from where\n")
	fd.write("    this script is being launched.\n")
	fd.write("  - you have a user_id and password for the gleep server (10.82.176.8)\n")
	fd.write("  - you have claimed one of the boards (ex. onefish, horton, etc.)\n")
	fd.write("  - the board you have claimed is correctly flashed\n")
	fd.write("  - the board is in POWERON condition (the script will reset the board)\n\n\n")
	fd.write("The following are the arguments that this script expects:\n\n")
	fd.write("-H\n")
	fd.write("   Display this help\n")
	fd.write("-s <simics port>\n")
	fd.write("   This option specifies that the script is to be run on simics\n")
	fd.write("   It also specifies the port on which simics is running\n")
	fd.write("   Only one of -s and -b option should be used\n")
	fd.write("-b <board_name | board_ip_address>\n")
	fd.write("   This option specifies that the script is to be run on board \n")
	fd.write("   It also specifies the name of the board (or its IP address) on the\n")
	fd.write("   gleep server on which the test will run.\n")
	fd.write("   Only one of -s and -b option should be used\n")
	fd.write("-u <username>\n")
	fd.write("   The user name to be used to connect to the gleep server\n")
	fd.write("   Used with -b option only\n")
	fd.write("-h <hostname | host_ip_address>\n")
	fd.write("   In case of running on board this option specifies the hostname \n")
	fd.write("   or host IP address for the gleep server, in case of running on simics \n")
	fd.write("   this specifies the local hostname or local IP address\n")
	fd.write("   to which the board is connected, and on which the mux_server will launch.\n")
	fd.write("-p <starting port number for the mux_server>\n")
	fd.write("   This option specifies the starting port number for the mux_server command.\n")
	fd.write("   The script will launch the mux_server with the port number starting from\n")
	fd.write("   what is specified here, and increment the port number for each byte channel\n")
	fd.write("   opened after the first one.\n")
	fd.write("-c <core mask>\n")
	fd.write("   This option specifies the coremask value to be passed onto partman. The\n")
	fd.write("   number must be an 8-bit hex number (prefixed with a 0x), the maximum\n")
	fd.write("   allowed value being 0x7f. Each ON bit in the value of the coremask enables\n")
	fd.write("   a core in the partition. This script automatically launches the mux_server\n")
	fd.write("   with the required number of ports and opens up as many xterms, based on \n")
	fd.write("   the number of cores specified through this option.\n")
	fd.write("-m <path to mux_server> \n")
	fd.write("   This option specifies the path to mux_server in case running on simics\n")
	fd.write("   Used with -s option only\n")
	fd.write("-P <simics command>\n")
	fd.write("   This option specifies the command to run the simics\n")
	fd.write("   Used with -s option only\n")
	fd.write("-w <workspace path>\n")
	fd.write("   This option specifies the path for the simics workspace\n")
	fd.write("   Used with -s option only\n")
	fd.write("-M <Simple Crypto test mode>\n")
	fd.write("   test mode: provide following number\n")
	fd.write("   1 for perf, 2 for crypto\n")
	fd.write("   Following two combinations are valid only\n")
	fd.write("   	-M 1 -S <buf_size> -N <buf_num> -O <algo> -L <itr_num>\n")
	fd.write("   	-M 2 -T <test_set> -N <buf_num> -O <algo> -L <itr_num>\n")
	fd.write("-S <Simple Crypto buffer size>\n")
	fd.write("   This option specifies size of buffer to be processed by SEC\n")
	fd.write("   Used with M=1 only\n")
	fd.write("-T <Simple Crypto test set>\n")
	fd.write("   This option specifies the test set number for selected algo.\n")
	fd.write("   Used with M=2 only\n")
	fd.write("-N <Number of buffers>\n")
	fd.write("   This option specifies total number of buffers.\n")
	fd.write("-O <Simple Crypto SEC algorithm>\n")
	fd.write("   This option specifies SEC4.0 enc/auth RAW algorithm\n")
	fd.write("-L <Simple Crypto number of iteration >\n")
	fd.write("   This option specifies no. of iteration for enc/auth processing\n")
	sys.exit(2)


# Checks whether the coremask value specified is valid, extracts
# the bits which are ON, and based on this, sets the value of the
# 'num_xterms' global variable.
def check_coremask(coremask):
	global num_xterms

	max_core_mask = int("0x7f", 16)
	min_core_mask = 0
	coremask = int(coremask, 16)

	if coremask > max_core_mask:
		fd.write("\nsimple_crypto_test.py: coremask is out of range\n")
		print_usage_and_exit()

	if coremask < 0:
		fd.write("\nsimple_crypto_test.py: coremask is out of range\n")
		print_usage_and_exit()

	while coremask > 0:
		if coremask % 2:
			num_xterms = num_xterms + 1
		coremask = coremask >> 1


# Parses the command line and fills the global variables
def parse_command_line():
	try:
		opts, args = getopt.getopt(sys.argv[1:], "Hu:s:h:p:c:m:b:r:P:w:M:S:T:N:O:L:", ["help"])
	except getopt.GetoptError, error:
        	# print help information and exit:
        	fd.write(str(error) + "\n")
		fd.write("\nCorrect usage------------------------------------------------------>\n")
		print_usage_and_exit()

	global mux_ser, host, user, port, board, core_mask, s_port, r_path, simics_cmd
	global mux_port_string, mux_ser_path, use_board, r_flag, ws_path
	global test_mode, buf_size, test_set, algo, buf_num, loop

	errors=0; u_flag=0; b_flag=0; s_flag=0; m_flag=0; w_flag=0
	h_flag=0; p_flag=0; c_flag=0; P_flag=0
	M_flag=0; S_flag=0; T_flag=0; N_flag=0; O_flag=0; L_flag=0;

    	for opt, arg in opts:
		if opt in ("-H", "help"):
			print_usage_and_exit()

		elif opt == "-u":
			user = arg
			u_flag = 1

		elif opt == "-b":
			if b_flag == 1:
				if board != arg:
					fd.write("\nsimple_crypto_test.py: ERROR: -b specified multiple times with different arguments\n")
					errors = errors+1
				else:
					fd.write("\nsimple_crypto_test.py: -b specified multiple times ... ignored\n")
			board = arg
			b_flag = 1
			use_board=1

		elif opt == "-s":
			if s_flag == 1:
				if board != arg:
					fd.write("\nsimple_crypto_test.py: ERROR: -s specified multiple times with different arguments\n")
					errors = errors+1
				else:
					fd.write("\nsimple_crypto_test.py: -s specified multiple times ... ignored\n")
			s_port = arg
			s_flag = 1

		elif opt == "-h":
			if h_flag == 1:
				if host != arg:
					fd.write("\nsimple_crypto_test.py: ERROR: -h specified multiple times with different arguments\n")
					errors = errors+1
				else:
					fd.write("\nsimple_crypto_test.py: -h specified multiple times ... ignored\n")
			cmd_string = "ping -c 1 " + str(arg)
			output = commands.getoutput(cmd_string)
			if output.rfind("icmp_seq") < 0:
				fd.write("\nsimple_crypto_test_2p.py: ERROR: host " + str(arg) + " is unreachable!\n")
				errors = errors + 1
			else:
				host = arg
			h_flag = 1

		elif opt == "-p":
			cmd_string = "echo "+ str(arg) + " | egrep \"^[0-9]+$\""
			output = commands.getoutput(cmd_string)
			if not output:
				fd.write("\nsimple_crypto_test.py: ERROR: unrecognized characters in the port number specified with -p\n")
				errors = errors+1
			else:
				if p_flag == 1:
					if port != int(arg):
						fd.write("\nsimple_crypto_test.py: ERROR: -p specified multiple times with different arguments\n")
						errors = errors+1
					else:
						fd.write("\nsimple_crypto_test.py: -p specified multiple times ... ignored\n")
				port = int(arg)
				p_flag = 1

		elif opt == "-c":
			if c_flag == 1:
				if core_mask != arg:
					fd.write("\nsimple_crypto_test.py: ERROR: -c specified multiple times with different arguments\n")
					errors = errors+1
				else:
					fd.write("\nsimple_crypto_test.py: -c specified multiple times ... ignored\n")
			core_mask = arg
			check_coremask(core_mask)
			c_flag = 1

		elif opt == "-m":
			if m_flag == 1:
				if mux_ser_path != arg:
					fd.write("\nsimple_crypto_test.py: ERROR: -m specified multiple times with different arguments\n")
					errors = errors+1
				else:
					fd.write("\nsimple_crypto_test.py: -m specified multiple times ... ignored\n")
			mux_ser_path = str(arg)
			m_flag = 1
			if not os.path.isfile(mux_ser_path + "/mux_server"):
				fd.write("\nsimple_crypto_test.py: ERROR: cannot find mux_server in specified path " + mux_ser_path + "\n")
				errors=errors+1

		elif opt == "-r":
			if r_flag == 1:
				if r_path != arg:
					fd.write("\nsimple_crypto_test.py: ERROR: -r specified multiple times with different arguments\n")
					errors = errors+1
				else:
					fd.write("\nsimple_crypto_test.py: -r specified multiple times ... ignored\n")
			r_path = arg
			r_flag = 1

		elif opt == "-P":
			if P_flag == 1:
				if simics_cmd != arg:
					fd.write("\nsimple_crypto_test.py: ERROR: -P flag specified multiple times with different arguments\n")
					errors = errors+1
				else:
					fd.write("\nsimple_crypto_test.py: -P specified multiple times ... ignored\n")
			simics_cmd = arg
			P_flag = 1

		elif opt == "-w":
			if w_flag == 1:
				if ws_path != arg:
					fd.write("\nsimple_crypto_test.py: ERROR: -w flag specified multiple times with different arguments\n")
					errors = errors+1
				else:
					fd.write("\nsimple_crypto_test.py: -w specified multiple times ... ignored\n")
			ws_path = arg
			w_flag = 1

		elif opt == "-M":
			if M_flag == 1:
				if test_mode != arg:
					fd.write("\nsimple_crypto_test.py: ERROR: -M flag specified multiple times with different arguments\n")
					errors = errors+1
				else:
					fd.write("\nsimple_crypto_test.py: -M specified multiple times ... ignored\n")
			test_mode = arg
			M_flag = 1

		elif opt == "-S":
			if S_flag == 1:
				if buf_size != arg:
					fd.write("\nsimple_crypto_test.py: ERROR: -S flag specified multiple times with different arguments\n")
					errors = errors+1
				else:
					fd.write("\nsimple_crypto_test.py: -S specified multiple times ... ignored\n")
			buf_size = arg
			S_flag = 1

		elif opt == "-T":
			if T_flag == 1:
				if test_set != arg:
					fd.write("\nsimple_crypto_test.py: ERROR: -T flag specified multiple times with different arguments\n")
					errors = errors+1
				else:
					fd.write("\nsimple_crypto_test.py: -T specified multiple times ... ignored\n")
			test_set = arg
			T_flag = 1

		elif opt == "-N":
			if N_flag == 1:
				if buf_num != arg:
					fd.write("\nsimple_crypto_test.py: ERROR: -N flag specified multiple times with different arguments\n")
					errors = errors+1
				else:
					fd.write("\nsimple_crypto_test.py: -N specified multiple times ... ignored\n")
			buf_num = arg
			N_flag = 1

		elif opt == "-O":
			if O_flag == 1:
				if algo != arg:
					fd.write("\nsimple_crypto_test.py: ERROR: -O flag specified multiple times with different arguments\n")
					errors = errors+1
				else:
					fd.write("\nsimple_crypto_test.py: -O specified multiple times ... ignored\n")
			algo = arg
			O_flag = 1

		elif opt == "-L":
			if L_flag == 1:
				if loop != arg:
					fd.write("\nsimple_crypto_test.py: ERROR: -L flag specified multiple times with different arguments\n")
					errors = errors+1
				else:
					fd.write("\nsimple_crypto_test.py: -L specified multiple times ... ignored\n")
			loop = arg
			L_flag = 1

		else:
			fd.write("Unhandled option\n")

	if s_flag == 0 and b_flag == 0:
		fd.write("\nsimple_crypto_test.py: ERROR: none of -b or -s parameters specified\n")
		print_usage_and_exit()
	if s_flag == 1 and b_flag == 1:
		fd.write("\nsimple_crypto_test.py: ERROR: both -b and -s parameters specified\n")
		print_usage_and_exit()
	if s_flag == 1 and (m_flag == 0 or p_flag == 0 or c_flag == 0 or P_flag == 0 or w_flag==0):
		fd.write("\nsimple_crypto_test.py: ERROR: required argument(s) missing\n")
		print_usage_and_exit()
	if b_flag == 1 and (p_flag == 0 or c_flag == 0 or h_flag == 0):
		fd.write(p_flag, c_flag, h_flag)
		fd.write("\nsimple_crypto_test.py: ERROR: required argument(s) missing\n")
		print_usage_and_exit()
	if test_mode == 1 and (S_flag == 0 or N_flag == 0 or O_flag == 0 or L_flag == 0):
		fd.write("\nsimple_crypto_test.py: ERROR: required argument(s) missing with M=1\n")
		print_usage_and_exit()
	if test_mode == 2 and (T_flag == 0 or N_flag == 0 or O_flag == 0 or L_flag == 0):
		fd.write("\nsimple_crypto_test.py: ERROR: required argument(s) missing with M=2\n")
		print_usage_and_exit()

	if errors > 0:
		fd.write("\nsimple_crypto_test.py: Please use the -H option to get detailed help\n")
		sys.exit(1)

# Constructs the mux_server port strings based on the
# first port number specified and the number of xterms to open.
# Fills up the 'mux_port_string' global variable.
def construct_mux_port_string():
	global mux_port_string, port
	mux_port_string = str(port)
	temp=port
	i=1

	while i < num_xterms:
		temp=temp+1
		mux_port_string = mux_port_string + " " + str(temp)
		i=i+1


# main
fd = open("logs/common.log", "a")
parse_command_line()

port1 = port


if use_board == 0:
	host = "localhost"

# make log files
cmd_str = "mkdir logs/" + str(host) + "_" + str(algo) + "_" + str(buf_size) + "_" + str(buf_num) + "_" + str(loop) + "_log"
os.system(cmd_str)

# construct the string provided to mux_server
construct_mux_port_string()

if use_board == 0:
	os.system("pkill mux; pkill simics")
	thread.start_new_thread(run_simics, (host, algo, buf_size, buf_num, loop, simics_cmd, ws_path))
	run_mux(mux_ser_path, s_port, mux_port_string)
else:
	run_board(host, user, board, mux_port_string, algo, buf_size, buf_num, loop)
	thread.start_new_thread(start_uboot, (host, port, algo, r_flag, r_path, buf_size, buf_num, loop))

# run linux partition as seperate thread
port=port+1
thread.start_new_thread(linux_partition, (host, port, core_mask, test_mode, buf_size, test_set, buf_num, algo, loop))

port=port+2
start_port=port

# run each of lwe partition as seperate threads
i=3
while i < num_xterms:
	thread.start_new_thread(lwe_partition, (host, port, algo, buf_size, buf_num, loop))
	port=port+1
	i=i+1

fd1 = open("logs/results.log", "a")
# wait till the test is completed
while config.test_complete != 1:
	time.sleep(5)
	max_wait-=5
	if not max_wait:
		fd.write("\nScript took toooo long to execute...\n")
		fd.write("\nSEC 4.0 test FAILED\n\n")
		fd1.write("algo: " + algo + "\n")
		fd1.write("buf_size: " + buf_size + "\n")
		fd1.write("buf_num: " + buf_num + "\n")
		fd1.write("iterations: " + loop + "\n")
		fd1.write("SEC 4.0 test FAILED\n\n")
		if use_board == 0:
			quit_simics()
                else :
                        reset_hv(host, port1, algo, buf_size, buf_num, loop)
		sys.exit(4)

# check weather the test passed or failed
port=start_port
i=3
while i < num_xterms:
	cmd_string = "grep -i pass logs/" + host + "_" + algo + "_" + buf_size + "_" + buf_num + "_" + loop + "_log/simple_crypto." + str(port) + ".log"
	output = commands.getoutput(cmd_string)
	if output:
		fd1.write("algo: " + algo + "\n")
		fd1.write("buf_size: " + buf_size + "\n")
		fd1.write("buf_num: " + buf_num + "\n")
		fd1.write("iterations: " + loop + "\n")
		fd2 = open("logs/" + host + "_" + algo + "_" + buf_size + "_" + buf_num + "_" + loop + "_log/simple_crypto." + str(port) + ".log", "r")
	        lines = fd2.readlines()
        	fd2.close()
        	for i in lines:
                	if i.rfind("hroughput") > 0:
				fd1.write(i)
		fd1.write("SEC 4.0 test PASSED\n\n")
		fd1.close()
		fd.write("\nSEC 4.0 test PASSED\n\n")
		if use_board == 0:
			quit_simics()
		else :
                        reset_hv(host, port1, algo, buf_size, buf_num, loop)
		sys.exit(0)
	port=port+1
	i=i+1

fd1.write("algo: " + algo + "\n")
fd1.write("buf_size: " + buf_size + "\n")
fd1.write("buf_num: " + buf_num + "\n")
fd1.write("iterations: " + loop + "\n")
fd1.write("SEC 4.0 test FAILED\n\n")
fd.write("\nSEC 4.0 test FAILED\n\n")
if use_board == 0:
	quit_simics()
else :
        reset_hv(host, port1, algo, buf_size, buf_num, loop)
sys.exit(3)

fd.close()
fd1.close()
