/**
 \file simple_pme_cp.h
 \brief Basic LWE Simple PME Config Tool defines and Data structures
 */
/*
 * Copyright (C) 2009-2010 Freescale Semiconductor, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 * NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

union stat64_t {
	volatile uint64_t stat;
	struct {
		volatile uint32_t hi;
		volatile uint32_t lo;
	} words;
};

const char *argp_program_version = "argex 1.0";

#define LWE_CTRL_PARAM_BMASK_PME_FLOWID			(1 << 0)
#define LWE_CTRL_PARAM_BMASK_PME_SET			(1 << 1)
#define LWE_CTRL_PARAM_BMASK_PME_SUBSET			(1 << 2)
#define LWE_CTRL_PARAM_BMASK_PME_FLAG			(1 << 3)
#define LWE_CTRL_PARAM_BMASK_PME_REPORT_VERBOSITY	(1 << 6)
#define LWE_CTRL_PARAM_BMASK_PME_SESSION_ID		(1 << 7)
#define LWE_CTRL_PARAM_BMASK_PME_INPUT_BUFF_CHUNK	(1 << 8)
#define LWE_CTRL_PARAM_BMASK_PME_QOS_IN			(1 << 9)
#define LWE_CTRL_PARAM_BMASK_PME_QOS_OUT			(1 << 10)
#define LWE_CTRL_PARAM_BMASK_PME_MATCH_LIMIT		(1 << 11)
#define LWE_CTRL_PARAM_BMASK_PME_COMPARE_LIMIT		(1 << 12)
#define LWE_CTRL_PARAM_BMASK_PME_OUTPUT_BUFF_FRM_APP	(1 << 13)
#define LWE_CTRL_PARAM_BMASK_PME_INPUT_BUFF_SIZE		(1 << 14)
#define LWE_CTRL_PARAM_BMASK_PME_INPUT_FILE		(1 << 15)
#define LWE_CTRL_PARAM_BMASK_PME_REPORT_PARSE_ENABLE	(1 << 16)
#define LWE_CTRL_PARAM_PME_MAX_BIT_NO		17

#define LWE_CTRL_PME_ADD_MDTR_PARAM_MAP (LWE_CTRL_PARAM_BMASK_PME_FLOWID | \
				 LWE_CTRL_PARAM_BMASK_PME_INPUT_FILE)

#define LWE_CTRL_PME_FLOWID_MIN 					0
#define LWE_CTRL_PME_FLOWID_MAX 					63

#define LWE_CTRL_PME_SET_MIN 					0
#define LWE_CTRL_PME_SET_MAX 					0xff
#define LWE_CTRL_PME_DEF_SET					0

#define LWE_CTRL_PME_SUBSET_MIN 					0
#define LWE_CTRL_PME_SUBSET_MAX 					0xffff
#define LWE_CTRL_PME_DEF_SUBSET					0xffff

#define LWE_CTRL_PME_DEF_FLAG					0

#define LWE_CTRL_PME_REPORT_VERBOSITY_MIN				0
#define LWE_CTRL_PME_REPORT_VERBOSITY_MAX				4
#define LWE_CTRL_PME_DEF_REPORT_VERBOSITY				4

#define LWE_CTRL_PME_SESSION_ID_MIN				0
#define LWE_CTRL_PME_SESSION_ID_MAX				0x7ffffff
#define LWE_CTRL_PME_DEF_SESSION_ID				0

#define LWE_CTRL_PME_INPUT_BUFF_CHUNK_MIN				0
#define LWE_CTRL_PME_INPUT_BUFF_CHUNK_MAX				1600
#define LWE_CTRL_PME_DEF_INPUT_BUFF_CHUNK				0

#define LWE_CTRL_PME_QOS_IN_MIN					0
#define LWE_CTRL_PME_QOS_IN_MAX					7
#define LWE_CTRL_PME_DEF_QOS_IN					0

#define LWE_CTRL_PME_QOS_OUT_MIN					0
#define LWE_CTRL_PME_QOS_OUT_MAX					7
#define LWE_CTRL_PME_DEF_QOS_OUT					0

#define LWE_CTRL_PME_MATCH_LIMIT_MIN				0
#define LWE_CTRL_PME_MATCH_LIMIT_MAX				0xffff
#define LWE_CTRL_PME_DEF_MATCH_LIMIT				0xffff

#define LWE_CTRL_PME_COMPARE_LIMIT_MIN				0
#define LWE_CTRL_PME_COMPARE_LIMIT_MAX				0xffff
#define LWE_CTRL_PME_DEF_COMPARE_LIMIT				0xffff

#define LWE_CTRL_PME_OUTPUT_BUFF_FRM_APP_MIN			0
#define LWE_CTRL_PME_OUTPUT_BUFF_FRM_APP_MAX			1
#define LWE_CTRL_PME_DEF_OUTPUT_BUFF_FRM_APP			1

#define LWE_CTRL_PME_REPORT_PARSE_ENABLE_MIN			0
#define LWE_CTRL_PME_REPORT_PARSE_ENABLE_MAX			1
#define LWE_CTRL_PME_DEF_REPORT_PARSE_DISABLE			0
#define LWE_CTRL_PME_DEF_REPORT_PARSE_ENABLE			1

#define LWE_CTRL_PME_INPUT_BUFF_SIZE_MIN				40
#define LWE_CTRL_PME_INPUT_BUFF_SIZE_MAX				0xffffffff
#define LWE_CTRL_PME_DEF_INPUT_BUFF_SIZE				100

static struct argp_option pme_add_options[] = {
	{"f", 'f', "FLOWID", 0, "Flow ID - (0 - 63)", 0},
	{"s", 's', "SET", 0, "Set - {Default: 0}", 0},
	{"u", 'u', "SUBSET", 0, "Subset - {Default: 65535}", 0},
	{"m", 'm', "SCANMODE", 0,
	 "Scan Mode - Direct (0) or Flow (1) - {Default: 0}", 0},
	{"r", 'r', "RESIDUEENBL", 0,
	 "Residue - Disable (0) or Enable (1) - {Default: 0}", 0},
	{"n", 'n', "SCANFNTYPE", 0,
	 "Scan Fn Type - Non-Block (0) or Block (1) - {Default: 0}",
	 0},
	{"p", 'p', "REPRTVRBSTY", 0, "Report Verbosity - {Default: 0}", 0},
	{"i", 'i', "SESSID", 0, "Session Id - {Default: 0}", 0},
	{"b", 'b', "INPUTBUFFCHNK", 0, "Input Buff Chunk - {Default: 0}", 0},
	{"q", 'q', "QOSIN", 0, "Qos In - {Default: 0}", 0},
	{"t", 't', "QOSOUT", 0, "Qos Out - {Default: 0}", 0},
	{"l", 'l', "MATCHLMT", 0, "Match Limit - {Default: 65535}", 0},
	{"o", 'o', "CMPRLMT", 0, "Compare Limit - {Default: 65535}", 0},
	{"e", 'e', "OUTPUTBUFFENBL", 0,
	 "Output Buff Allocation from PME (0) or App (1) - {Default: 1}", 0},
	{"z", 'z', "INPUTBUFFSZ", 0,
	 "Input Buff Size - (40 - 1600) {Default: 100}", 0},
	{"d", 'd', "INPUTFILE", 0, "Input File to be scanned", 0},
	{0, 0, 0, 0, 0, 0}
};

#define LWE_CTRL_PME_DEL_MDTR_PARAM_MAP (LWE_CTRL_PARAM_BMASK_PME_FLOWID )

static struct argp_option pme_del_options[] = {
	{"f", 'f', "FLOWID", 0, "Flow ID - (1 - 1024)", 0},
	{0, 0, 0, 0, 0, 0}
};

#define LWE_CTRL_PME_GO_MDTR_PARAM_MAP 0

static struct argp_option pme_go_options[] = {
	{"v", 'v', "REPORT_PARSE_ENABLE", 0, "Report Parse Enable - (Default: 0)", 0},
	{0, 0, 0, 0, 0, 0}
};
