#!/bin/sh
#
#  Copyright (C) 2009-2010 Freescale Semiconductor, Inc.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
# NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

flows=32
enable=0
value=0
w_flag=0
v_flag=0
error=0
i=0

while getopts "w:v:h" OPTION
do
	case $OPTION in
	w)
		if [ "$w_flag" -eq "1" ]
		then
			echo -e "\n Error : -w option specified multiple times"
			error=$((error+1))
		fi
		w_flag=1
		value="$OPTARG"
		if [ "$value" -eq "0" -o "$value" -gt "64" ]
		then
			echo -e "\n Error : -w (total flows) allowed range in 1 - 64 only"
			error=$((error+1))
		else
			flows=$value
		fi
	;;
	v)
		if [ "$v_flag" -eq "1" ]
		then
			echo -e "\n Error : -v option specified multiple times"
			error=$((error+1))
		fi
		v_flag=1
		value="$OPTARG"
		if [ "$value" -gt "1" ]
		then
			echo -e "\n Error : -v > 1, Only 0 and 1 allowed"
			error=$((error+1))
		else
			enable=$value
		fi
	;;
	h)
		echo -e "\n Script Help  "
		echo -e "  -w number of flows (range 1 to 64  => Default 32)"
		echo -e "  -v 0 - report parse disable (Default)"
		echo -e "     1 - report parse enable"
		exit 1
	;;
	esac
done

if [ "$error" -gt "0" ]
then
	echo -e "\n One or more Invalid arguments passed"
	exit 1
fi

if [ "$v_flag" -eq "0" ]
then
	echo -e "\n Report Parse arguments missing"
	echo -e "\n Default report parsing disabled"
fi

if [ "$w_flag" -eq "0" ]
then
	echo -e "\n Number of Flows arguments missing"
	echo -e "\n Default number of flows is 32"
fi

for w in $(seq 0 $(((flows/4)-1)))
do
	./simple_pme -J -b 100 -d simple_pme_undivided_pattern_text.dat -e 0 -f $i -i 1 -l 65535 -m 1 -n 0 -o 65535 -p 0 -q 0 -r 1 -s 0 -t 0 -u 65280 -z 100
	i=`expr $i + 1`
	./simple_pme -J -b 100 -d simple_pme_undivided_pattern_text.dat -e 0 -f $i -i 1 -l 65535 -m 1 -n 0 -o 65535 -p 1 -q 0 -r 1 -s 0 -t 0 -u 65280 -z 100
	i=`expr $i + 1`
	./simple_pme -J -b 100 -d simple_pme_undivided_pattern_text.dat -e 0 -f $i -i 1 -l 65535 -m 1 -n 0 -o 65535 -p 2 -q 0 -r 1 -s 0 -t 0 -u 65280 -z 100
	i=`expr $i + 1`
	./simple_pme -J -b 100 -d simple_pme_undivided_pattern_text.dat -e 1 -f $i -i 1 -l 65535 -m 1 -n 0 -o 65535 -p 3 -q 0 -r 1 -s 0 -t 0 -u 65280 -z 100
	i=`expr $i + 1`
done

for j in $(seq $i $((flows-1)))
do
	./simple_pme -J -b 100 -d simple_pme_undivided_pattern_text.dat -e 1 -f $j -i 1 -l 65535 -m 1 -n 0 -o 65535 -p 3 -q 0 -r 1 -s 0 -t 0 -u 65280 -z 100
done

./simple_pme -O -v $enable 
