#!/usr/bin/python
# Copyright (C) 2010 Freescale Semiconductor, Inc.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
# NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from simple_pme import *
from simple_pme_exit import *
from simics_start import *
from uboot import *
from board_start import *
import getopt
import sys
import os
import time
import thread
import threading
import commands

mux_ser_path=""
host=""
user=""
port=0
s_port=0
board=""
core_mask=""
test_script=""
num_xterms=3
mux_port_string=""
use_board=0
r_flag=0
r_path=""
max_wait=400
simics_cmd=""

# Prints the usage information and exits with code 2
def print_usage_and_exit():
	print "\nTestbench for execution of 'simple_pme' on P4080 Board and simics.\n"
	print "This script is the interface to launch the simple_pme LWE application on"
	print "the P4080 boards. Currently, it only supports the 2p configuration of the "
	print "hypervisor. Before executing the script, make sure that the following"
	print "conditions are satisfied:"
	print "  - The 'expect' utility must be installed on the machine from where"
	print "    this script is being launched."
	print "  - you have a user_id and password for the gleep server (10.82.176.8)"
	print "  - you have claimed one of the boards (ex. onefish, horton, etc.)"
	print "  - the board you have claimed is correctly flashed"
	print "  - the board is in POWERON condition (the script will reset the board)\n\n"
	print "The following are the arguments that this script expects:\n"
	print "-H"
	print "   Display this help"
	print "-s <simics port>"
	print "   This option specifies that the script is to be run on simics"
	print "   It also specifies the port on which simics is running"
	print "   Only one of -s and -b option should be used"
	print "-b <board_name | board_ip_address>"
	print "   This option specifies that the script is to be run on board "
	print "   It also specifies the name of the board (or its IP address) on the"
	print "   gleep server on which the test will run."
	print "   Only one of -s and -b option should be used"
	print "-u <username>"
	print "   The user name to be used to connect to the gleep server"
	print "   Used with -b option only"
	print "-h <hostname | host_ip_address>"
	print "   In case of running on board this option specifies the hostname "
	print "   or host IP address for the gleep server, in case of running on simics "
	print "   this specifies the local hostname or local IP address"
	print "   to which the board is connected, and on which the mux_server will launch."
	print "-p <starting port number for the mux_server>"
	print "   This option specifies the starting port number for the mux_server command."
	print "   The script will launch the mux_server with the port number starting from"
	print "   what is specified here, and increment the port number for each byte channel"
	print "   opened after the first one."
	print "-c <core_mask>"
	print "   This option specifies the coremask value to be passed onto partman. The"
	print "   number must be an 8-bit hex number (prefixed with a 0x), the maximum"
	print "   allowed value being 0x7f. Each ON bit in the value of the coremask enables"
	print "   a core in the partition. This script automatically launches the mux_server"
	print "   with the required number of ports and opens up as many xterms, based on "
	print "   the number of cores specified through this option."
	print "-t <test_script>"
	print "   This option specifies the test shell script to use."
	print "-m <path_to_mux_server> "
	print "   This option specifies the path to mux_server in case running on simics"
	print "   Used with -s option only"
	print "-P <simics_command>"
	print "   This option specifies the command to run the simics"
	print "   Used with -s option only"
	print "-w <workspace_path>"
	print "   This option specifies the path for the simics workspace"
	print "   Used with -s option only"
	sys.exit(2)


# Checks whether the coremask value specified is valid, extracts
# the bits which are ON, and based on this, sets the value of the
# 'num_xterms' global variable.
def check_coremask(coremask):
	global num_xterms

	max_core_mask = int("0x7f", 16)
	min_core_mask = 0
	coremask = int(coremask, 16)

	if coremask > max_core_mask:
		print "\nsimple_pme_test.py: coremask is out of range"
		print_usage_and_exit()

	if coremask < 0:
		print "\nsimple_pme_test.py: coremask is out of range"
		print_usage_and_exit()

	while coremask > 0:
		if coremask % 2:
			num_xterms = num_xterms + 1
		coremask = coremask >> 1


# Parses the command line and fills the global variables
def parse_command_line():
	try:
		opts, args = getopt.getopt(sys.argv[1:], "Hu:s:h:p:c:t:m:b:r:P:w:", ["help"])
	except getopt.GetoptError, error:
        	# print help information and exit:
        	print str(error)
		print "\nCorrect usage------------------------------------------------------>"
		print_usage_and_exit()

	global mux_ser, host, user, port, board, core_mask, s_port, r_path, simics_cmd
	global test_script, mux_port_string, mux_ser_path, use_board, r_flag, ws_path

	errors=0; u_flag=0; b_flag=0; s_flag=0; m_flag=0; w_flag=0
	h_flag=0; p_flag=0; c_flag=0; t_flag=0; P_flag=0

    	for opt, arg in opts:
		if opt in ("-H", "help"):
			print_usage_and_exit()

		elif opt == "-u":
			user = arg
			u_flag = 1

		elif opt == "-b":
			if b_flag == 1:
				if board != arg:
					print "\nsimple_pme_test.py: ERROR: -b specified multiple times with different arguments"
					errors = errors+1
				else:
					print "\nsimple_pme_test.py: -b specified multiple times ... ignored"
			board = arg
			b_flag = 1
			use_board=1

		elif opt == "-s":
			if s_flag == 1:
				if board != arg:
					print "\nsimple_pme_test.py: ERROR: -s specified multiple times with different arguments"
					errors = errors+1
				else:
					print "\nsimple_pme_test.py: -s specified multiple times ... ignored"
			s_port = arg
			s_flag = 1

		elif opt == "-h":
			if h_flag == 1:
				if host != arg:
					print "\nsimple_pme_test.py: ERROR: -h specified multiple times with different arguments"
					errors = errors+1
				else:
					print "\nsimple_pme_test.py: -h specified multiple times ... ignored"
			cmd_string = "ping -c 1 " + str(arg)
			output = commands.getoutput(cmd_string)
			if output.rfind("icmp_seq") < 0:
				print "\nsimple_pme_test_2p.py: ERROR: host " + str(arg) + " is unreachable!"
				errors = errors + 1
			else:
				host = arg
			h_flag = 1

		elif opt == "-p":
			cmd_string = "echo "+ str(arg) + " | egrep \"^[0-9]+$\""
			output = commands.getoutput(cmd_string)
			if not output:
				print "\nsimple_pme_test.py: ERROR: unrecognized characters in the port number specified with -p"
				errors = errors+1
			else:
				if p_flag == 1:
					if port != int(arg):
						print "\nsimple_pme_test.py: ERROR: -p specified multiple times with different arguments"
						errors = errors+1
					else:
						print "\nsimple_pme_test.py: -p specified multiple times ... ignored"
				port = int(arg)
				p_flag = 1

		elif opt == "-c":
			if c_flag == 1:
				if core_mask != arg:
					print "\nsimple_pme_test.py: ERROR: -c specified multiple times with different arguments"
					errors = errors+1
				else:
					print "\nsimple_pme_test.py: -c specified multiple times ... ignored"
			core_mask = arg
			check_coremask(core_mask)
			c_flag = 1

		elif opt == "-t":
			if t_flag == 1:
				if test_script != arg:
					print "\nsimple_pme_test.py: ERROR: -t specified multiple times with different arguments"
					errors = errors+1
				else:
					print "\nsimple_pme_test.py: -t specified multiple times ... ignored"
			test_script = arg
			t_flag = 1

		elif opt == "-m":
			if m_flag == 1:
				if mux_ser_path != arg:
					print "\nsimple_pme_test.py: ERROR: -m specified multiple times with different arguments"
					errors = errors+1
				else:
					print "\nsimple_pme_test.py: -m specified multiple times ... ignored"
			mux_ser_path = str(arg)
			m_flag=1
			if not os.path.isfile(mux_ser_path + "/mux_server"):
				print "\nsimple_pme_test.py: ERROR: cannot find mux_server in specified path " + mux_ser_path
				errors=errors+1

		elif opt == "-r":
			if r_flag == 1:
				if r_path != arg:
					print "\nsimple_pme_test.py: ERROR: -r specified multiple times with different arguments"
					errors = errors+1
				else:
					print "\nsimple_pme_test.py: -r specified multiple times ... ignored"
			r_path = arg
			r_flag=1

		elif opt == "-P":
			if P_flag == 1:
				if simics_cmd != arg:
					print "\nsimple_pme_test.py: ERROR: -P flag specified multiple times with different arguments"
					errors = errors+1
				else:
					print "\nsimple_pme_test.py: -P specified multiple times ... ignored"
			simics_cmd = arg
			P_flag=1

		elif opt == "-w":
			if w_flag == 1:
				if ws_path != arg:
					print "\nsimple_pme_test.py: ERROR: -w flag specified multiple times with different arguments"
					errors = errors+1
				else:
					print "\nsimple_pme_test.py: -w specified multiple times ... ignored"
			ws_path = arg
			w_flag=1

		else:
			print "Unhandled option"

	if s_flag == 0 and b_flag == 0:
		print "\nsimple_pme_test.py: ERROR: none of -b or -s parameters specified"
		print_usage_and_exit()
	if s_flag == 1 and b_flag == 1:
		print "\nsimple_pme_test.py: ERROR: both -b and -s parameters specified"
		print_usage_and_exit()
	if s_flag == 1 and (m_flag == 0 or p_flag == 0 or c_flag == 0 or t_flag == 0 or h_flag == 0 or P_flag == 0 or w_flag==0):
		print "\nsimple_pme_test.py: ERROR: required argument(s) missing"
		print_usage_and_exit()
	if b_flag == 1 and (p_flag == 0 or c_flag == 0 or t_flag == 0 or h_flag == 0):
		print "\nsimple_pme_test.py: ERROR: required argument(s) missing"
		print_usage_and_exit()

	if errors > 0:
		print "\nsimple_pme_test.py: Please use the -H option to get detailed help"
		sys.exit(1)

# Constructs the mux_server port strings based on the
# first port number specified and the number of xterms to open.
# Fills up the 'mux_port_string' global variable.
def construct_mux_port_string():
	global mux_port_string, port
	mux_port_string = str(port)
	temp=port
	i=1

	while i < num_xterms:
		temp=temp+1
		mux_port_string = mux_port_string + " " + str(temp)
		i=i+1


# main
parse_command_line()

# remove previous log files
cmd_string = "rm -rf " + host + "_" + test_script + "_log"
os.system(cmd_string)
cmd_string = "mkdir " + host + "_" + test_script + "_log"
os.system(cmd_string)

# construct the string provided to mux_server
construct_mux_port_string()

if use_board == 0:
	thread.start_new_thread(run_simics, (host, test_script, simics_cmd, ws_path))
	run_mux(mux_ser_path, s_port, mux_port_string)
else:
	run_board(host, user, board, mux_port_string)
	start_uboot(host, port, test_script, r_flag, r_path)

# run linux partition as seperate thread
port=port+1
thread.start_new_thread(linux_partition, (host, port, core_mask, test_script))

port=port+2
start_port=port

# run each of lwe partition as seperate threads
i=3
while i < num_xterms:
	thread.start_new_thread(lwe_partition, (host, port, test_script))
	port=port+1
	i=i+1

# wait till the test is completed
while config.test_complete != 1:
	time.sleep(5)
	max_wait-=5
	if not max_wait:
		print "Script took toooo long to execute..."
		print "Test FAILED"
		if use_board == 0:
			quit_simics()
		sys.exit(4)

# check weather the test passed or failed
port=start_port
i=3
while i < num_xterms:
	cmd_string = "grep -i pass simple_pme." + host + "." + str(port) + ".log"
	output = commands.getoutput(cmd_string)
	if output:
		print "Test PASSED"
		if use_board == 0:
			quit_simics()
		sys.exit(0)
	port=port+1
	i=i+1

print "Test FAILED"
if use_board == 0:
	quit_simics()
sys.exit(3)
