/**
 \file  queue_test.c          
 \brief QMan Torture test control plane file.
 */
/*
 * Copyright (C) 2008-2010 Freescale Semiconductor, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 * NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include "queue_test.h"

int no_of_queues = 0;
int buffers_per_queue = 0;
int iterate = 0;

volatile struct torture_config_s *torture_conf;

/**
 \brief Configuration structure initialization routine.
 \param[in] Void
 \return NULL
 */
void init_config_structure(void)
{
	torture_conf->fq_start = 19000;
	torture_conf->num_fq = no_of_queues;
	torture_conf->channel = 0;
	torture_conf->wq_id = 1;
	torture_conf->num_buff_per_fq = buffers_per_queue;
	torture_conf->barrier = 0;
	torture_conf->no_of_iterations = iterate;

}

/**
 \brief   Main function of CP-Queuing torture code
 \details It accepts command line arguments from user, 
	  handle: to run LWE application onboot up required partition and 
	  use those handles to start specified partitions. 
	  It also accepts number of queues and number of buffers per queue,
	  which are initialized by the control partition.
 */
int main(int argc, char **argv)
{
	char *load_command;
	int32_t hflag = 0;
	uint32_t mode = 0, mflag = 0;
	int qflag = 0;
	int bflag = 0;
	int iflag = 0;
	int c, index = 0;
	int loop, str1_size;
	int32_t arr[7] = { -1, -1, -1, -1, -1, -1, -1 };
	void *where;
	int fd;

	opterr = 0;

	printf("\nArguments are: %d.", argc);

	if (argc < 9) {
		printf("\r\n\r\n");
		printf("\r\nArguments are not complete!");
		printf("\r\nProper way of entering the arguments.");
		printf("\r\n====================================>");
		printf
		    ("\r\nEnter no of iterations(Recommended 1 to 5000) of qman_poll \
				for running application in finite mode with -i option.");
		printf
		    ("\r\nEnter mode of running app: 1 for infinite running mode, \
						2 for finite mode with -m option");
		printf
		    ("\r\nEnter handle of partition you want the application to run on.");
		printf("\r\nEnter each handle number followed by -h option.");
		printf
		    ("\r\nEnter number of queues followed by -q option(1 to 10000).");
		printf
		    ("\r\nEnter number of buffers per queue followed by -b option(1 to 50).");
		printf("\r\n");
		return 0;

	}

	while ((c = getopt(argc, argv, "i:m:h:q:b:")) != -1) {

		switch (c) {

		case 'i':
			iflag = atoi(optarg);
			if (iflag <= 5000) {
				iterate = iflag;
			} else {
				printf
				    ("\n Number of iterations enteres are out of range(Recommended 1 to 5000).\n");
				return 1;
			}

			break;
		case 'm':
			mflag = atoi(optarg);
			if (mflag == 1 || mflag == 2) {
				mode = mflag;
			} else {
				printf("\n Enter valid mode id.\n");
				return 1;
			}

			break;
		case 'h':
			hflag = atoi(optarg);
			if (hflag >= 0) {
				arr[index++] = hflag;
			} else {
				printf
				    ("\n Enter appropriate handle number of the LWE core.\n");
				return 1;
			}
			break;

		case 'q':

			qflag = atoi(optarg);
			if (qflag >= 1 && qflag <= 10000) {
				no_of_queues = qflag;
			} else {
				printf
				    ("\n Enter valid number of queues between 1 and 10000.\n");
				return 1;
			}
			break;

		case 'b':

			bflag = atoi(optarg);
			if (bflag >= 1 && bflag <= 50) {
				buffers_per_queue = bflag;
			} else {
				printf
				    ("\n enter valid number of buffers between 1 and 50.\n");
				return 1;
			}
			break;

		case '?':

			if (optopt == 'h' || optopt == 'q' || optopt == 'b' || optopt == 'i' || optopt == 'm')
				fprintf(stderr,
					"Option -%c requires an argument.\n",
					optopt);
			else if (isprint(optopt))
				fprintf(stderr, "Unknown option '-%c' .\n",
					optopt);
			else
				fprintf(stderr,
					"Unknown option character '\\x%x'.\n",
					optopt);
			return 1;

		}

	}

	if ((no_of_queues) && (buffers_per_queue)) {
		printf("\r\nNumber of queues are: %d	\
			\r\nBuffers per queue are: %d\n", no_of_queues, buffers_per_queue);
	}

	str1_size = 256;
	load_command = malloc(str1_size + 1);
	if (!load_command) {
		printf("malloc fails");
		return -ENOMEM;
	}

	fd = open("/dev/mem", O_RDWR);
	if (fd < 0) {
		printf("Failed to open\n");
		free(load_command);
		return 0;
	}

	where = mmap(0,
		     LWE_CTRL_SHM_SIZE,
		     PROT_READ | PROT_WRITE, MAP_SHARED, fd, LWE_CTRL_SHM_ADDR);
	if (where == (void *)0xffffffff) {
		close(fd);
		printf("mmap failed\n");
		free(load_command);
		return -1;
	}

	torture_conf = (struct torture_config_s *)where;

	init_config_structure();

	/* Loading and running elf file for torture test on specified handle number. */

	for (loop = 0; loop < 7; loop++) {
		if (arr[loop] != -1) {
			if (mode == 1) {
				sprintf(load_command,
					"partman start -f"
						"torture_test_func.elf -h %ld",
					arr[loop]);
			} else {
				sprintf(load_command,
					"partman start -f torture_test_"
						"hybrid.elf -h %ld",
					arr[loop]);
			}
			printf("\nLoading command is : %s", load_command);
			if (system(load_command) == -1)
				printf("\r\nImage Loading failed.");
			else
				printf("\r\nImage loaded successfully.");
		}
	}

	printf("\nCP before barrier");
	torture_conf->barrier = 1;
	printf("\nReleased the barrier.");

	free(load_command);
	close(fd);
	return 0;
}
