/**
 \file  queue_test.h         
 \brief Torture test control plane file.
 */
/*
 * Copyright (C) 2008-2009 Freescale Semiconductor, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 * NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef _QUEUE_TEST_H
#define _QUEUE_TEST_H

#include <stdio.h>
#include<ctype.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>
#include <fcntl.h>
#include <error.h>
#include <errno.h>
#include <stropts.h>
#include <unistd.h>
#include "app_cpdp.h"
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <fcntl.h>
#include <sys/mman.h>

/**
 \brief Structure to keep Configuartion parameters.
        This structure is kept in shared no heap area for CP-DP communication.
        Parameters entered by user through CP are written in this structure,
        which are used by the corresponding application running on 
        data plane for initializations.
 */
struct torture_config_s {
	uint32_t fq_start;		/**< Start Frame Number */
	uint32_t num_fq;		/**< Number of frame queues need to be initialized */
	uint32_t channel;		/**< Channel number in which frame queues will be created */
	uint32_t wq_id;			/**< Work queue id of the frame queues */
	uint32_t num_buff_per_fq;	/**< Number of buffers in each frame queue */
	uint32_t no_of_iterations;	/**< Number of times qman_poll should iterate in hybrid mode */
	uint32_t barrier;		/**< Barrier variable for CP-DP synchronization */
};

/**
 \brief Configuration structure initialization routine.
        Initialize fields of configuration structure.
	this contains required parameters for creating and initializing 
	frame queues in data plane. 
 \param[in] Void
 \return NULL
 */
void init_config_structure(void);

/**
 \brief Image loading function
       Run image loading and start command that loads elf image 
       of corresponding data plane application and starts its execution.
 \param[in] const char * command that need to be executed as system command
 \return NULL
 */
void run_command(const char *);

#endif /* _QUEUE_TEST_H */
