/**
 \file torture_test.c
 \brief Queuing Torture Test( Data plane component)
 */
/*
 * Copyright (C) 2008-2010 Freescale Semiconductor, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 * NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include "torture_test.h"
#include "timer_wheel.h"
#include "timebase.h"
#include "swi.h"

uint32_t torture_lock;
uint32_t g_boot_core = INV_CORE_ID;
struct lwe_barrier qman_barrier1;
volatile uint32_t *total_iterations = NULL;
volatile uint32_t *total_frames_processed = NULL;
struct qman_fq **torture_fq;
volatile uint32_t fq_end;
__PERCPU uint32_t time_stamp = 0;
__PERCPU uint32_t dq_no = 0;
__PERCPU uint32_t idle_no = 0;
__PERCPU uint32_t current_core;
__PERCPU uint32_t atb_start = 0;
__PERCPU uint32_t total_cycles = 0;
__PERCPU uint32_t processing_cycles = 0;
__PERCPU uint32_t entry_time = 0;
__PERCPU uint8_t dqrr_enter = 0;

static uint32_t grand_total_processing_cycles = 0;
static uint32_t grand_total_cycles = 0;
__PERCPU uint32_t enq_max_time = 0;
__PERCPU uint32_t enq_min_time = 9999;
__PERCPU uint32_t deq_max_time = 0;
__PERCPU uint32_t deq_min_time = 9999;
__PERCPU uint32_t total_deq_time = 0;
__PERCPU uint32_t total_enq_time = 0;
__PERCPU uint32_t deq_time = 0;
__PERCPU uint32_t enq_time = 0;

uint32_t recv_channel_map;
int32_t channel_id_lwe;

volatile struct torture_config_s *torture_conf;

static enum qman_cb_dqrr_result dqrr_entry_handler(struct qman_portal *qm,
						   struct qman_fq *fq,
						   const struct qm_dqrr_entry
						   *dqrr);

static void cb_notimplemented_ern(struct qman_portal *qm, struct qman_fq *fq,
				 const struct qm_mr_entry *msg);
static void cb_notimplemented_state_chng(struct qman_portal *qm, struct qman_fq *fq,
				 const struct qm_mr_entry *msg);

const struct qman_fq_cb qman_test_fq_cb = {
	.dqrr = dqrr_entry_handler,
	.ern = cb_notimplemented_ern,
	.dc_ern = NULL,
	.fqs = cb_notimplemented_state_chng
};

static void cb_notimplemented_ern(struct qman_portal *qm, struct qman_fq *fq,
				 const struct qm_mr_entry *msg)
{
	APP_ERROR("%s: No handling for ERNs", __func__);
}

static void cb_notimplemented_state_chng(struct qman_portal *qm, struct qman_fq *fq,
				 const struct qm_mr_entry *msg)
{
	APP_ERROR("%s: No handling for FQ's state change", __func__);
}

/**
 \brief Configuration structure initialization routine.
 \param[in] Void
 \return NULL
 */
void init_config_structure(void)
{

	APP_DEBUG("INIT CONFIG STRUCTURE:");
	APP_DEBUG("Start of frame queues: %lu", torture_conf->fq_start);
	APP_DEBUG("No of frame queues: %lu", torture_conf->num_fq);
	APP_DEBUG("Channel number: %lu", torture_conf->channel);
	APP_DEBUG("Work Queue ID: %lu", torture_conf->wq_id);
	APP_DEBUG("No of buffers in each FQ: %lu",
		  torture_conf->num_buff_per_fq);
	APP_DEBUG("No of iterations: %lu", torture_conf->no_of_iterations);
	fq_end = torture_conf->fq_start + torture_conf->num_fq;
}

/**
 \brief Dequeue Handler for Qman Torture Application
 \param[in] struct qman_portal * Qman Portal Pointer
 \param[in] struct qman_fq * Pointer to a Qman frame Queue descriptor
 \param[in] struct qm_dqrr_entry * Dequeue Receive Ring entry to be processed
 \param[out] NULL
 */
static enum qman_cb_dqrr_result dqrr_entry_handler(struct qman_portal *qm,
						   struct qman_fq *fq,
						   const struct qm_dqrr_entry
						   *dqrr)
{
	uint32_t frameq_id;
	uint32_t time_after_enq, time_before_enq;
	uint32_t fq_start;

	if (!dqrr_enter) {
		dqrr_enter += 1;
		entry_time = mfspr(SPR_ATBL);
	}

	deq_time = mfspr(SPR_ATBL) - time_stamp;

	fq_start = torture_conf->fq_start;
	frameq_id = dqrr->fqid;
	dq_no++;
	decorated_notify_inc_32(total_frames_processed);

	total_deq_time = total_deq_time + deq_time;
	if (deq_max_time < deq_time)
		deq_max_time = deq_time;
	if (deq_min_time > deq_time)
		deq_min_time = deq_time;

	markpoint(22);
	if (frameq_id >= ((torture_conf->num_fq + torture_conf->fq_start) - 1))
		frameq_id = torture_conf->fq_start;
	else
		frameq_id = frameq_id + 1;

	time_before_enq = mfspr(SPR_ATBL);
	/* NB: the use of QMAN_ENQUEUE_FLAG_WAIT is prohibited in LWE
	 * applications, and indeed was unsafe when called from a dequeue
	 * callback (as is the case here). To avoid thrashing, we implement a
	 * throttled poll for retrying the enqueue. */
loop:
	if (qman_enqueue(torture_fq[frameq_id - fq_start], &(dqrr->fd), 0)) {
		uint64_t now, then = get_atb();
		do {
			now = get_atb();
		} while (now < (then + 1000));
		goto loop;
	}

	time_after_enq = mfspr(SPR_ATBL);
	enq_time = time_after_enq - time_before_enq;
	total_enq_time = total_enq_time + enq_time;

	if (enq_max_time < enq_time)
		enq_max_time = enq_time;
	if (enq_min_time > enq_time)
		enq_min_time = enq_time;

	time_stamp = mfspr(SPR_ATBL);
	markpoint(21);
	return qman_cb_dqrr_consume;

}

/**
 \brief Frame queues creation routine.
 \param[in] Void
 \return Negative - Failure
	Non-Negative - Success
 */
int create_frame_queues()
{
	uint32_t flags;
	uint32_t fq_id, fq_start, index;

	fq_start = torture_conf->fq_start;
	flags = QMAN_FQ_FLAG_LOCKED;
	for (fq_id = fq_start; fq_id < fq_end; fq_id++) {
		index = fq_id - fq_start;
		torture_fq[index]->cb = qman_test_fq_cb;

		if (unlikely
		    (0 != qman_create_fq(fq_id, flags, torture_fq[index]))) {
			APP_ERROR("qman_create_fq failed for FQ ID: %lu",
				  fq_id);
			return -1;
		}

	}

	APP_INFO("Created FQs %lu - %lu", fq_start, (fq_end - 1));
	return 0;
}

/**
 \brief Frame queues initialization routine.
 \param[in] Void
 \return Negative - Failure
	Non-Negative - Success
 */
int initialize_fqs(void)
{
	struct qm_mcc_initfq opts;
	uint32_t fq_id, flags;
	uint32_t fq_start, index;

	flags = QMAN_INITFQ_FLAG_SCHED;
	opts.we_mask = QM_INITFQ_WE_DESTWQ;
	opts.fqd.dest.channel = (uint16_t)channel_id_lwe;
	opts.fqd.dest.wq = (uint16_t)torture_conf->wq_id;
	fq_start = torture_conf->fq_start;

	for (fq_id = fq_start; fq_id < fq_end; fq_id++) {
		index = fq_id - fq_start;
		if (unlikely(0 != qman_init_fq(torture_fq[index],
					       flags, &opts))) {
			APP_ERROR("qm_init_fq failed for fq_id: %lu", fq_id);
			return -1;
		}
	}

	APP_INFO("Init FQs Base: %lu, Count: %lu, "
		 "Channel: %lu, WQ: %lu",
		 fq_start, (fq_end - 1),
		 channel_id_lwe, torture_conf->wq_id);

	return 0;
}

/**
 \brief Buffers enqueuing routine.
 \param[in] Void
 \return NULL
 */
void test_buffers_enq(void)
{
	uint32_t buff_count, q_cnt;
	struct qm_fd *p_new_fd, new_fd;
	uint32_t desc_count;

	p_new_fd = &new_fd;
	memset(p_new_fd, 0, sizeof(struct qm_fd));
	for (q_cnt = 0, desc_count = 0; q_cnt < torture_conf->num_fq; q_cnt++) {
		for (buff_count = 0; buff_count < torture_conf->num_buff_per_fq;
		     buff_count++, desc_count++) {
			p_new_fd->length20 = BUFFER_SIZE;
			/* NB: the use of QMAN_ENQUEUE_FLAG_WAIT is prohibited
			 * in LWE applications. To avoid thrashing, we implement
			 * a throttled poll for retrying the enqueue. */
loop:
			if (qman_enqueue(torture_fq[q_cnt], p_new_fd, 0)) {
				uint64_t now, then = get_atb();
				do {
					now = get_atb();
				} while (now < (then + 1000));
				goto loop;
			}
		}
	}

	markpoint(2);
	APP_INFO("Enqueued %lu buffers in each FQ",
		 torture_conf->num_buff_per_fq);
}

/**
 \brief Stats printing routine
 \param[in] Timer identifier
 \param[in] void pointer
 \return NULL
 */
void print_stats(uint32_t timer_id, void *p_data)
{
	static uint32_t num_expiry = 1;
	uint32_t time_elapsed;
	markpoint(7);
	APP_INFO("\r\n\r\n");
	APP_INFO("QMAN torture test statistics:");
	APP_INFO("============================>");

	APP_INFO("Torture statistics. System time : %lu ms",
		 get_current_time());
	APP_INFO("========================================================");
	time_elapsed = (num_expiry * STATS_PRINT_INTERVAL_MS);
	APP_INFO("Time elapsed since application started : %lu ms",
		 time_elapsed);
	APP_INFO("Total frames enqueued/dequeued by current core are: %lu",
		 dq_no);
	APP_INFO("Frames enqueued/dequeued by current core in one ms: %lu",
		 (dq_no / time_elapsed));
	num_expiry++;
}

/**
\brief   Main function of Queuing torture test.
\details Data plane routine.
	 The first partition that comes up takes the role of the control partition. 
	 Only control partition perform the initializations of frame queues and 
	 enqueue buffers in each frame queue. 
	 The per partition resources like qman portals and timer wheel are initialized 
	 by each partitions. After the initializations are done, the partitions 
	 enter into an infinite loop dequeueing frames from common pool channel 
	 and enqueuing them in some other frame queue of common pool channel.
\note    Should never return.
*/
int main(int argc, char **argv)
{
	int i;

	APP_INFO("Welcome to Queuing Torture Test!");

	/* setting up the value of g_boot_core variable 
	   as the first booted partition number. */

	current_core = get_lwe_id();
	APP_INFO("Current patition number is:%lu", current_core);

	spin_lock(&torture_lock);
	if (INV_CORE_ID == g_boot_core) {
		g_boot_core = current_core;
		APP_DEBUG("Control patition is:%lu", current_core);
		total_iterations =
		    stats_memalign(CACHE_LINE_SIZE, sizeof(uint32_t));
		*total_iterations = 0;
		total_frames_processed =
		    stats_memalign(CACHE_LINE_SIZE, sizeof(uint32_t));
		*total_frames_processed = 0;
		torture_conf = (struct torture_config_s *)&_whiteboard_start;
		barrier_init(&qman_barrier1, get_online_core_mask());
	}
	spin_unlock(&torture_lock);

#ifdef QMAN_FUNC
	/* Initialise the timer framework */
	if (0 != timer_framework_start()) {
		APP_ERROR("timer_framework_start Failed");
		return -1;
	}

	APP_INFO("Timer Framework has been initialized.");
#endif

	init_config_structure();

	/* Frame queues creation, their initialization and enqueuing test 
	   buffers in them: Done by only one core in a partition */
	if (current_core == g_boot_core) {
		if (dpa_eth_read(&recv_channel_map) < 0) {
			APP_ERROR("dpa_eth_read failed.");
			goto err_out;
		}

		if (0 >
		(channel_id_lwe = extract_qman_pool_channel(devtree_ptr))) {
			APP_ERROR("local_extract_qman_channel_id failed");
			return -EINVAL;
		} else
			APP_INFO("Extracted Channel ID is :%u", channel_id_lwe);
	
		channel_id_lwe = channel_id_lwe + qm_channel_pool1 - 1;
		APP_INFO("Pool Channel ID is :%u", channel_id_lwe);

		torture_fq =
		    (struct qman_fq **)memalign(CACHE_LINE_SIZE,
						(torture_conf->num_fq) *
						sizeof(struct qman_fq *));
		for (i = 0; i < torture_conf->num_fq; i++) {
			if (unlikely(!torture_fq)) {
				APP_ERROR("Out of memory");
				return -1;
			}
			torture_fq[i] =
			    (struct qman_fq *)memalign(CACHE_LINE_SIZE,
						       sizeof(struct qman_fq));
		}

		if (create_frame_queues() == -1) {
			APP_ERROR("Creation of Frame Queues failed.");
			APP_INFO("failed.");
			goto err_out;
		}
		APP_INFO("Frame Queues created.");

		if (0 != initialize_fqs()) {
			APP_ERROR("initialize_fqs failed.");
			return -1;
		}
		APP_INFO("Initialized Frame Queues.");
	}
#ifdef QMAN_FUNC
	/* Start a periodic timer for stats display */
	if (INV_TIMER_ID == start_timer(STATS_PRINT_INTERVAL_MS, true, NULL,
					SWI_PRI_LOW, print_stats, NULL)) {
		APP_ERROR("Stats timer start failed");
		goto err_out;
	}
	APP_INFO("Stats Timer has been started.");
#endif
	if (current_core == g_boot_core) {
		test_buffers_enq();
		while (!torture_conf->barrier) ;
		grand_total_cycles = 0;
		grand_total_processing_cycles = 0;
	}

	processing_cycles = 0;
	total_cycles = 0;
	barrier_sync(&qman_barrier1);

	MAGIC_BREAKPOINT;
	qman_static_dequeue_add(QM_SDQCR_CHANNELS_POOL(channel_id_lwe)
			| recv_channel_map);

/* Code for functional mode execution */
#ifdef QMAN_FUNC
	void *p_data;
	uint32_t swi_id;

	while (1) {
		time_stamp = mfspr(SPR_ATBL);
		qman_poll();
		while (likely(SWI_ID_INV != (swi_id = swi_activate(&p_data)))) {
			switch (swi_id) {
			case SWI_ID_TIMER_EXPIRY:
				proc_timer_expiry((uint32_t) p_data);
				break;
			default:
				break;
			}
		}
	}
#else
	uint32_t underruns = 0;

	/* Code for hybrid mode execution */
	atb_start = mfspr(SPR_ATBL);
	while (*total_iterations < (torture_conf->no_of_iterations)) {
		time_stamp = mfspr(SPR_ATBL);
		markpoint(16);
		qman_poll();
		markpoint(6);
		if (dqrr_enter) {
			dqrr_enter = 0;
			decorated_notify_inc_32(total_iterations);
			processing_cycles += (mfspr(SPR_ATBL) - entry_time);
		} else {
			markpoint(4);
			underruns++;
		}
	}

	total_cycles = mfspr(SPR_ATBL) - atb_start;

	spin_lock(&torture_lock);
	grand_total_cycles += total_cycles;
	grand_total_processing_cycles += processing_cycles;
	spin_unlock(&torture_lock);

	barrier_sync(&qman_barrier1);

	if (current_core == g_boot_core) {
		APP_INFO
		    ("Total %d packets processed in %d total cycles, Cycles per Frame = %d 	\
					Productive Cycles = %d, Processing Cycles/Frame = %d Total Underruns = %d",
		     *total_frames_processed, grand_total_cycles, grand_total_cycles / *total_frames_processed, grand_total_processing_cycles,
		     grand_total_processing_cycles / *total_frames_processed, underruns);
	}

	APP_INFO("\r\n");
	APP_INFO("QMAN torture test Statistics:");
	APP_INFO("=====================================>");
	APP_INFO
	    ("Total calls to qman_poll made by current core to dequeue frames are: %lu",
	     *total_iterations);
	APP_INFO("Total frames enqueued/dequeued by current core are: %lu",
		 dq_no);
	APP_INFO
	    ("Maximum/Minimum time taken in enqueue operation(in cycles):%lu/%lu",
	     enq_max_time, enq_min_time);
	APP_INFO
	    ("Maximum/Minimum time taken in dequeue operation(in cycles):%lu/%lu",
	     deq_max_time, deq_min_time);

	barrier_sync(&qman_barrier1);

	barrier_destroy(&qman_barrier1);

	markpoint(30);

	APP_INFO("QMAN torture test Complete!");
	return 0;

#endif

err_out:
	abort();
	return 0;
}
